@@ -3,7 +3,7 @@ from taurus.qt.qtgui.panel import TaurusForm
 from taurus.qt.qtgui.display import TaurusLabel
 from taurus.qt.qtgui.application import TaurusApplication
 
-app = TaurusApplication(sys.argv)
+app = TaurusApplication(sys.argv, cmd_line_parser=None)
 
 panel = TaurusForm()
 props = ['state', 'status', 'position', 'velocity', 'acceleration']
