@@ -2,7 +2,7 @@ import sys
 from taurus.external.qt import Qt
 from taurus.qt.qtgui.application import TaurusApplication
 
-app = TaurusApplication(sys.argv)
+app = TaurusApplication(sys.argv, cmd_line_parser=None)
 panel = Qt.QWidget()
 layout = Qt.QHBoxLayout()
 panel.setLayout(layout)
