@@ -2,8 +2,9 @@ import sys
 from taurus.external.qt import Qt
 from taurus.qt.qtgui.container import TaurusWidget
 from taurus.qt.qtgui.display import TaurusLabel
+from taurus.qt.qtgui.application import TaurusApplication
 
-app = Qt.QApplication(sys.argv)
+app = TaurusApplication(sys.argv, cmd_line_parser=None)
 panel = TaurusWidget()
 layout = Qt.QVBoxLayout()
 panel.setLayout(layout)
@@ -11,12 +12,9 @@ panel.setModel('sys/taurustest/1')
 w1 = TaurusLabel()
 w2 = TaurusLabel()
 w3 = TaurusLabel()
-w1.setUseParentModel(True)
-w2.setUseParentModel(True)
-w3.setUseParentModel(True)
-w1.setModel('/state')
-w2.setModel('/position')
-w3.setModel('/simulationmode')
+w1.setModel('sys/taurustest/1/state')
+w2.setModel('sys/taurustest/1/position')
+w3.setModel('sys/taurustest/1/simulationmode')
 w1.setShowQuality(False)
 
 layout.addWidget(w1)
