@@ -32,7 +32,9 @@ from .core import release as __R
 class Release:
     pass
 
-Release.__dict__.update(__R.__dict__)
+for attr, value in __R.__dict__.items():
+	setattr(Release, attr, value)
+
 Release.__doc__ = __R.__doc__
 
 from .core.taurushelper import *
