@@ -31,7 +31,7 @@ __all__ = ["TangoObjectType", "EVENT_TO_POLLING_EXCEPTIONS",
 __docformat__ = "restructuredtext"
 
 from taurus.core.util.enumeration import Enumeration
-from taurus.external.enum import IntEnum
+from enum import IntEnum
 
 TangoObjectType = Enumeration("TangoObjectType",
                               ["Authority", "Server", "Class", "Device",
@@ -65,10 +65,12 @@ EVENT_TO_POLLING_EXCEPTIONS = ('API_AttributePollingNotStarted',
                                'API_EventTimeout',
                                'API_EventPropertiesNotSet',
                                'API_CommandNotFound',
+                               'API_PollObjNotFound',
                                )
 #                                   'API_BadConfigurationProperty')
 
 FROM_TANGO_TO_NUMPY_TYPE = {
+    PyTango.DevVoid: None,
     PyTango.DevBoolean: numpy.bool8,
     PyTango.DevUChar: numpy.uint8,
     PyTango.DevShort: numpy.short,
@@ -83,6 +85,7 @@ FROM_TANGO_TO_NUMPY_TYPE = {
 }
 
 FROM_TANGO_TO_STR_TYPE = {
+    PyTango.DevVoid: None,
     PyTango.DevBoolean: 'bool8',
     PyTango.DevUChar: 'uint8',
     PyTango.DevShort: 'short',
