@@ -25,21 +25,24 @@
 
 """This module provides the `TangoFactory` object"""
 
-__all__ = ["TangoFactory"]
-
-__docformat__ = "restructuredtext"
+from __future__ import absolute_import
+from future.utils import string_types
 
 try:
-    import PyTango
+    pass
 except ImportError:
+
     # note that if PyTango is not installed the factory will not be available
     from taurus.core.util.log import debug
     msg = 'cannot import PyTango module. ' + \
           'Taurus will not support the "tango" scheme'
     debug(msg)
     raise
+import PyTango
 
-from taurus.core.taurusbasetypes import TaurusElementType
+from taurus import tauruscustomsettings
+from taurus.core.taurusbasetypes import (TaurusElementType,
+                                         TaurusSerializationMode)
 from taurus.core.taurusfactory import TaurusFactory
 from taurus.core.taurusbasetypes import OperationMode
 from taurus.core.taurusexception import TaurusException, DoubleRegistration
@@ -56,6 +59,10 @@ _Attribute = TangoAttribute
 _Device = TangoDevice
 
 
+__all__ = ["TangoFactory"]
+__docformat__ = "restructuredtext"
+
+
 class TangoFactory(Singleton, TaurusFactory, Logger):
     """A :class:`TaurusFactory` singleton class to provide Tango-specific
     Taurus Element objects (TangoAuthority, TangoDevice, TangoAttribute)
@@ -105,10 +112,14 @@ class TangoFactory(Singleton, TaurusFactory, Logger):
         self._polling_enabled = True
         self.reInit()
         self.scheme = 'tango'
+        self._serialization_mode = TaurusSerializationMode.get(
+            getattr(tauruscustomsettings, 'TANGO_SERIALIZATION_MODE',
+                    'TangoSerial'))
 
     def reInit(self):
         """Reinitialize the singleton"""
         self._default_tango_host = None
+        self._tango_subscribe_enabled = True
         self.dft_db = None
         self.tango_db = CaselessWeakValueDict()
         self.tango_db_queries = CaselessWeakValueDict()
@@ -162,13 +173,50 @@ class TangoFactory(Singleton, TaurusFactory, Logger):
         return dict(self.tango_db)
 
     def set_default_tango_host(self, tango_host):
-        """Sets the new default tango host.
-
-        :param tango_host: (str) the new tango host
         """
-        self._default_tango_host = tango_host
+        Sets the new default tango host. The method will transform the given
+        name to an Authority URI.
+        
+        .. note:: Calling this method also clears the device alias cache.
+
+        :param tango_host: (str) the new tango host. It accepts any valid Tango
+                                 authority name or None to use the defined by
+                                 $TANGO_HOST env. var.
+
+        """
+        # Translate to Authority URI
+        if tango_host and "//" not in tango_host:
+            tango_host = "//{0}".format(tango_host)
+        v = self.getAuthorityNameValidator()
+        self._default_tango_host = v.getUriGroups(tango_host)["authority"]
+        self.tango_alias_devs.clear()
         self.dft_db = None
 
+    def get_default_tango_host(self):
+        """Retruns the current default tango host
+        """
+        return self._default_tango_host
+    
+    def set_tango_subscribe_enabled(self,value):
+        """ If True, enables event subscribing on TangoAttribute objects
+
+        .. warning:: This method belongs to a "Delayed Event Subscription" API
+                 added in v.4.2.1-alpha as an *experimental* feature. This API
+                 may not be stable and/or it may be removed in a future release
+                 (even on a minor version change)
+        """
+        self._tango_subscribe_enabled = value
+    
+    def is_tango_subscribe_enabled(self):
+        """ Returns the current tango_subscribe_enabled status
+
+        .. warning:: This method belongs to a "Delayed Event Subscription" API
+                 added in v.4.2.1-alpha as an *experimental* feature. This API
+                 may not be stable and/or it may be removed in a future release
+                 (even on a minor version change)
+        """
+        return self._tango_subscribe_enabled
+
     def registerAttributeClass(self, attr_name, attr_klass):
         """Registers a new attribute class for the attribute name.
 
@@ -185,7 +233,7 @@ class TangoFactory(Singleton, TaurusFactory, Logger):
 
            :param attr_name: (str) attribute name
         """
-        if self.tango_attr_klasses.has_key(attr_name):
+        if attr_name in self.tango_attr_klasses:
             del self.tango_attr_klasses[attr_name]
 
     def registerDeviceClass(self, dev_klass_name, dev_klass):
@@ -204,7 +252,7 @@ class TangoFactory(Singleton, TaurusFactory, Logger):
 
            :param dev_klass_name: (str) tango device class name
         """
-        if self.tango_dev_klasses.has_key(dev_klass_name):
+        if dev_klass_name in self.tango_dev_klasses:
             del self.tango_dev_klasses[dev_klass_name]
 
     def getDatabase(self, name=None):
@@ -259,7 +307,8 @@ class TangoFactory(Singleton, TaurusFactory, Logger):
                 self.debug("Could not create Authority %s", groups['authority'],
                            exc_info=1)
 
-            self.tango_db[name] = ret
+            if ret is not None:
+                self.tango_db[name] = ret
         return ret
 
     def getDevice(self, dev_name, create_if_needed=True, **kw):
@@ -287,6 +336,7 @@ class TangoFactory(Singleton, TaurusFactory, Logger):
 
         validator = _Device.getNameValidator()
         groups = validator.getUriGroups(dev_name)
+
         if groups is None:
             raise TaurusException("Invalid Tango device name '%s'" % dev_name)
 
@@ -362,7 +412,7 @@ class TangoFactory(Singleton, TaurusFactory, Logger):
                         attr_klass = self._getAttributeClass(
                             attr_name=attr_name)
                         kwargs['storeCallback'] = self._storeAttribute
-                        if not kwargs.has_key('pollingPeriod'):
+                        if 'pollingPeriod' not in kwargs:
                             kwargs[
                                 'pollingPeriod'] = self.getDefaultPollingPeriod()
                         attr = attr_klass(full_attr_name, dev, **kwargs)
@@ -402,7 +452,7 @@ class TangoFactory(Singleton, TaurusFactory, Logger):
 
            :return: (taurus.core.tango.TangoAttribute) configuration object
         """
-        if isinstance(param, str):
+        if isinstance(param, string_types):
             return self.getAttribute(param)
         return param
 
@@ -478,10 +528,10 @@ class TangoFactory(Singleton, TaurusFactory, Logger):
             raise KeyError("Device %s not found" % dev_or_dev_name)
         dev.cleanUp()
         full_name = dev.getFullName()
-        if self.tango_devs.has_key(full_name):
+        if full_name in self.tango_devs:
             del self.tango_devs[full_name]
         simp_name = dev.getSimpleName()
-        if self.tango_alias_devs.has_key(simp_name):
+        if simp_name in self.tango_alias_devs:
             del self.tango_alias_devs[simp_name]
 
     def removeExistingAttribute(self, attr_or_attr_name):
@@ -497,7 +547,7 @@ class TangoFactory(Singleton, TaurusFactory, Logger):
             raise KeyError("Attribute %s not found" % attr_or_attr_name)
         attr.cleanUp()
         full_name = attr.getFullName()
-        if self.tango_attrs.has_key(full_name):
+        if full_name in self.tango_attrs:
             del self.tango_attrs[full_name]
 
     def isPollingEnabled(self):
@@ -512,14 +562,14 @@ class TangoFactory(Singleton, TaurusFactory, Logger):
         if not self.isPollingEnabled():
             return
         self._polling_enabled = False
-        for period, timer in self.polling_timers.iteritems():
+        for period, timer in self.polling_timers.items():
             timer.stop()
 
     def enablePolling(self):
         """Enable the application tango polling"""
         if self.isPollingEnabled():
             return
-        for period, timer in self.polling_timers.iteritems():
+        for period, timer in self.polling_timers.items():
             timer.start()
         self._polling_enabled = True
 
@@ -531,17 +581,17 @@ class TangoFactory(Singleton, TaurusFactory, Logger):
 
     def getAuthorityNameValidator(self):
         """Return TangoAuthorityNameValidator"""
-        import tangovalidator
+        from . import tangovalidator
         return tangovalidator.TangoAuthorityNameValidator()
 
     def getDeviceNameValidator(self):
         """Return TangoDeviceNameValidator"""
-        import tangovalidator
+        from . import tangovalidator
         return tangovalidator.TangoDeviceNameValidator()
 
     def getAttributeNameValidator(self):
         """Return TangoAttributeNameValidator"""
-        import tangovalidator
+        from . import tangovalidator
         return tangovalidator.TangoAttributeNameValidator()
 
     def setOperationMode(self, mode):
