@@ -34,7 +34,7 @@ import weakref
 from .taurusmodel import TaurusModel
 from taurus.core.taurusbasetypes import TaurusElementType, DataType
 from taurus.core.util.log import deprecation_decorator
-from taurus.external.pint import Quantity, UR
+from taurus.core.units import Quantity
 
 
 class TaurusAttribute(TaurusModel):
@@ -43,7 +43,7 @@ class TaurusAttribute(TaurusModel):
     _description = "A Taurus Attribute"
     defaultFragmentName = "rvalue"  # fragment to be used if none is specified
 
-    def __init__(self, name, parent, **kwargs):
+    def __init__(self, name='', parent=None, **kwargs):
         self.call__init__(TaurusModel, name, parent)
 
         # User enabled/disabled polling
@@ -75,10 +75,15 @@ class TaurusAttribute(TaurusModel):
         self._range = [None, None]
         self._alarm = [None, None]
         self._warning = [None, None]
+        self.precision = None
 
     def cleanUp(self):
         self.trace("[TaurusAttribute] cleanUp")
-        self._unsubscribeEvents()
+        if hasattr(self, '_unsuscribeEvents'):
+            self.deprecated(
+                dep='TaurusAttribute._unsuscribeEvents API',
+                alt='If you need it called in cleanUp, re-implement cleanUp')
+            self._unsuscribeEvents()
         TaurusModel.cleanUp(self)
 
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
@@ -145,14 +150,6 @@ class TaurusAttribute(TaurusModel):
         raise NotImplementedError("Not allowed to call AbstractClass" +
                                   " TaurusAttribute.poll")
 
-    def _subscribeEvents(self):
-        raise NotImplementedError("Not allowed to call AbstractClass" +
-                                  " TaurusAttribute._subscribeEvents")
-
-    def _unsubscribeEvents(self):
-        raise NotImplementedError("Not allowed to call AbstractClass" +
-                                  " TaurusAttribute._unsubscribeEvents")
-
     def isUsingEvents(self):
         raise NotImplementedError("Not allowed to call AbstractClass" +
                                   " TaurusAttribute.isUsingEvents")
@@ -289,19 +286,20 @@ class TaurusAttribute(TaurusModel):
 
     def getDisplayDescrObj(self, cache=True):
         name = self.getLabel(cache=cache)
-        obj = [('name', name)]
+        obj = [('name', name),
+               ('model', self.getFullName() or '')]
         descr = self.description
         if descr:
             _descr = descr.replace("<", "&lt;").replace(">", "&gt;")
             obj.append(('description', _descr))
 
         if isinstance(self.rvalue, Quantity):
-            idDimensionless = self.rvalue.dimensionless
+            _unitless = self.rvalue.unitless
             range = self._range
             alarm = self._alarm
             warning = self._warning
             if range != [None, None]:
-                if not idDimensionless:
+                if not _unitless:
                     low = range[0]
                     high = range[1]
                 else:
@@ -309,7 +307,7 @@ class TaurusAttribute(TaurusModel):
                     high = range[1].magnitude
                 obj.append(('range', "[%s, %s]" % (low, high)))
             if alarm != [None, None]:
-                if not idDimensionless:
+                if not _unitless:
                     low = alarm[0]
                     high = alarm[1]
                 else:
@@ -317,7 +315,7 @@ class TaurusAttribute(TaurusModel):
                     high = alarm[1].magnitude
                 obj.append(('alarm', "[%s, %s]" % (low, high)))
             if warning != [None, None]:
-                if not idDimensionless:
+                if not _unitless:
                     low = warning[0]
                     high = warning[1]
                 else:
@@ -448,4 +446,4 @@ class TaurusAttribute(TaurusModel):
     label = property(getLabel, setLabel)
     range = property(getRange, setRange)
     warnings = property(getWarnings, setWarnings)
-    alarms = property(getAlarms, setAlarms)
+    alarms = property(getAlarms, setAlarms)
\ No newline at end of file
