@@ -22,9 +22,18 @@
 ##
 #############################################################################
 
-'''
+"""
 a misc collection of basic types
-'''
+"""
+
+import datetime
+
+from .util.enumeration import Enumeration
+from .util.log import taurus4_deprecation
+from enum import IntEnum
+from future.utils import PY2
+from builtins import object
+
 
 __all__ = ["TaurusSWDevState", "TaurusSWDevHealth", "OperationMode",
            "TaurusSerializationMode", "SubscriptionState", "TaurusEventType",
@@ -35,12 +44,6 @@ __all__ = ["TaurusSWDevState", "TaurusSWDevHealth", "OperationMode",
 
 __docformat__ = "restructuredtext"
 
-import datetime
-
-from .util.enumeration import Enumeration
-from .util.log import taurus4_deprecation
-from taurus.external.enum import IntEnum
-
 
 class TaurusDevState(IntEnum):
     """Enumeration of possible states of :class:`taurus.core.TaurusDevice`
@@ -72,7 +75,8 @@ OperationMode = Enumeration(
 TaurusSerializationMode = Enumeration(
     'TaurusSerializationMode', (
         'Serial',
-        'Concurrent'
+        'Concurrent',
+        'TangoSerial',
     ))
 
 TaurusEventType = Enumeration(
@@ -149,14 +153,15 @@ DataType = Enumeration(
 __PYTHON_TYPE_TO_TAURUS_DATATYPE = {
     str: DataType.String,
     int: DataType.Integer,
-    long: DataType.Integer,
+    # long : DataType.Integer,  # (only in py2) see below...
     float: DataType.Float,
     bool: DataType.Boolean,
-    # bytes : DataType.Bytes, # see below...
+    # bytes : DataType.Bytes,  # (only in py>=3) see below...
 }
-if str is not bytes:  # Python >=3
+if PY2:  # Python 2
+    __PYTHON_TYPE_TO_TAURUS_DATATYPE[long] = DataType.Integer
+else:  # Python >=3
     __PYTHON_TYPE_TO_TAURUS_DATATYPE[bytes] = DataType.Bytes
-# Note: in Python2, DataType.from_python_type(bytes) --> DataType.String
 DataType.from_python_type = __PYTHON_TYPE_TO_TAURUS_DATATYPE.get
 
 SubscriptionState = Enumeration(
@@ -246,7 +251,7 @@ class TaurusTimeVal(object):
         return self.totime()
 
     def totime(self):
-        return self.tv_usec * 1e-9 + self.tv_usec * 1e-6 + self.tv_sec
+        return self.tv_nsec * 1e-9 + self.tv_usec * 1e-6 + self.tv_sec
 
     def todatetime(self):
         return datetime.datetime.fromtimestamp(self.totime())
