@@ -55,21 +55,23 @@
   <https://tools.ietf.org/html/rfc3986#section-3.3>)
 
 """
+from __future__ import absolute_import
+from builtins import object
+
+import atexit
+from weakref import WeakValueDictionary
+from .taurusbasetypes import TaurusElementType
+from .taurusauthority import TaurusAuthority
+from .taurusdevice import TaurusDevice
+from .taurusattribute import TaurusAttribute
+from .taurusconfiguration import TaurusConfiguration, TaurusConfigurationProxy
+from .taurusexception import TaurusException
+from taurus.core.tauruspollingtimer import TaurusPollingTimer
 
 __all__ = ["TaurusFactory"]
 
 __docformat__ = "restructuredtext"
 
-import atexit
-from weakref import WeakValueDictionary
-from taurusbasetypes import TaurusElementType
-from taurusauthority import TaurusAuthority
-from taurusdevice import TaurusDevice
-from taurusattribute import TaurusAttribute
-from taurusconfiguration import TaurusConfiguration, TaurusConfigurationProxy
-from taurusexception import TaurusException
-from taurus.core.tauruspollingtimer import TaurusPollingTimer
-
 
 class TaurusFactory(object):
     """The base class for valid Factories in Taurus."""
@@ -93,7 +95,7 @@ class TaurusFactory(object):
         self._devs = WeakValueDictionary()
         self._auths = WeakValueDictionary()
 
-        import taurusmanager
+        from . import taurusmanager
         manager = taurusmanager.TaurusManager()
         self._serialization_mode = manager.getSerializationMode()
 
@@ -147,7 +149,7 @@ class TaurusFactory(object):
             raise TaurusException(msg)
 
         fullname, _, _ = v.getNames(name)
-        auth = self._devs.get(fullname)
+        auth = self._auths.get(fullname)
         if auth is not None:
             return auth
 
@@ -271,7 +273,7 @@ class TaurusFactory(object):
 
         :return: (bool) True if the scheme is supported (False otherwise)
         """
-        return scheme in self.shemes
+        return scheme in self.schemes
 
     def findObject(self, absolute_name):
         """ Must give an absolute name"""
@@ -323,14 +325,14 @@ class TaurusFactory(object):
         if not self.isPollingEnabled():
             return
         self._polling_enabled = False
-        for period, timer in self.polling_timers.iteritems():
+        for period, timer in self.polling_timers.items():
             timer.stop()
 
     def enablePolling(self):
         """Enable the application tango polling"""
         if self.isPollingEnabled():
             return
-        for period, timer in self.polling_timers.iteritems():
+        for period, timer in self.polling_timers.items():
             timer.start()
         self._polling_enabled = True
 
@@ -353,7 +355,7 @@ class TaurusFactory(object):
            :param attribute: (str) attribute name.
         """
         p = None
-        for period, timer in self.polling_timers.iteritems():
+        for period, timer in self.polling_timers.items():
             if timer.containsAttribute(attribute):
                 timer.removeAttribute(attribute)
                 if timer.getAttributeCount() == 0:
@@ -394,6 +396,17 @@ class TaurusFactory(object):
             ret.append(TaurusElementType.Authority)
         return ret
 
+    def getValidatorFromName(self, name):
+        """
+        Obtain the validator object corresponding to the given model
+        name. If the model name is not valid for any TaurusModel class,
+        it returns None
+        """
+        modeltypes = self.getValidTypesForName(name)
+        if not modeltypes:
+            return None
+        return self.elementTypesMap[modeltypes[0]].getNameValidator()
+
     def findObjectClass(self, absolute_name):
         """
         Obtain the class object corresponding to the given name.
