@@ -25,17 +25,17 @@
 
 """This module contains the polling class"""
 
+import weakref
+import threading
+
+from .util.log import Logger
+from .util.containers import CaselessWeakValueDict
+from .util.timer import Timer
+
 __all__ = ["TaurusPollingTimer"]
 
 __docformat__ = "restructuredtext"
 
-import time
-import threading
-
-from .util.log import Logger, DebugIt
-from .util.containers import CaselessDict
-from .util.timer import Timer
-
 
 class TaurusPollingTimer(Logger):
     """ Polling timer manages a list of attributes that have to be polled in
@@ -74,7 +74,7 @@ class TaurusPollingTimer(Logger):
         self.lock.acquire()
         try:
             attr_dict = self.dev_dict.get(dev)
-            return attr_dict and attr_dict.has_key(attr_name)
+            return attr_dict and attr_name in attr_dict
         finally:
             self.lock.release()
 
@@ -97,9 +97,9 @@ class TaurusPollingTimer(Logger):
         attr_dict = self.dev_dict.get(dev)
         if attr_dict is None:
             if attribute.factory().caseSensitive:
-                self.dev_dict[dev] = attr_dict = {}
+                self.dev_dict[dev] = attr_dict = weakref.WeakValueDictionary()
             else:
-                self.dev_dict[dev] = attr_dict = CaselessDict()
+                self.dev_dict[dev] = attr_dict = CaselessWeakValueDict()
         if attr_name not in attr_dict:
             attr_dict[attr_name] = attribute
             self.attr_nb += 1
@@ -107,7 +107,7 @@ class TaurusPollingTimer(Logger):
             self.start()
         else:
             import taurus
-            taurus.Manager().addJob(attribute.poll, None)
+            taurus.Manager().enqueueJob(attribute.poll)
 
     def removeAttribute(self, attribute):
         """Unregisters the attribute from this polling. If the number of registered
