@@ -31,8 +31,8 @@ __docformat__ = 'restructuredtext'
 
 import numpy
 import functools
-from taurus.external import unittest
-from taurus.external.pint import Quantity
+import unittest
+from taurus.core.units import Quantity
 import taurus
 from taurus.test import insertTest
 from taurus.core import (TaurusElementType, TaurusAttribute, TaurusDevice,
@@ -212,7 +212,7 @@ class IsValidNameTestCase(unittest.TestCase):
             elementType = [elementType]
         manager = taurus.Manager()
         scheme = manager.getScheme(name)
-        supportedSchemes = manager.getPlugins().keys()
+        supportedSchemes = list(manager.getPlugins().keys())
         if scheme not in supportedSchemes:
             self.skipTest('"%s" scheme not supported' % scheme)
         returned = taurus.isValidName(name, etypes=elementType, strict=strict)
@@ -240,7 +240,7 @@ class AuthorityTestCase(unittest.TestCase):
             klass = TaurusAuthority
         manager = taurus.Manager()
         scheme = manager.getScheme(name)
-        supportedSchemes = manager.getPlugins().keys()
+        supportedSchemes = list(manager.getPlugins().keys())
         if scheme not in supportedSchemes:
             self.skipTest('"%s" scheme not supported' % scheme)
         a = taurus.Authority(name)
@@ -254,11 +254,8 @@ class AuthorityTestCase(unittest.TestCase):
 @insertTest(helper_name='get_object', name='eval://localhost/@Foo')
 @insertTest(helper_name='get_object', name='eval:@Foo')
 @insertTest(helper_name='get_object', name='eval://dev=Foo')
-@insertTest(helper_name='get_object',
-            name='eval:@taurus.core.evaluation.dev_example.FreeSpaceDevice')
-@insertTest(helper_name='get_object',
-            name='eval://dev=taurus.core.evaluation.dev_example.FreeSpaceDevice'
-            )
+@insertTest(helper_name='get_object', name='eval:@datetime.*')
+@insertTest(helper_name='get_object', name='eval:@d=datetime.date(2017,3,29)')
 class DeviceTestCase(unittest.TestCase):
     '''TestCase for the taurus.Device helper'''
 
@@ -268,7 +265,7 @@ class DeviceTestCase(unittest.TestCase):
             klass = TaurusDevice
         manager = taurus.Manager()
         scheme = manager.getScheme(name)
-        supportedSchemes = manager.getPlugins().keys()
+        supportedSchemes = list(manager.getPlugins().keys())
         if scheme not in supportedSchemes:
             self.skipTest('"%s" scheme not supported' % scheme)
 
@@ -406,6 +403,12 @@ class DeviceTestCase(unittest.TestCase):
             expected=dict(rvalue=1e6, value=1e6, wvalue=None, w_value=None,
                           label='Q("1km").to("mm").magnitude',
                           type=DataType.Float))
+@insertTest(helper_name='read_attr',
+            name='eval:@d=datetime.date(1931,4,14)/d.isoformat()',
+            expected=dict(rvalue='1931-04-14', value='1931-04-14',
+                          wvalue=None, w_value=None,
+                          label='d.isoformat()',
+                          type=DataType.String))
 @insertTest(helper_name='read_attr', name='eval:1#')
 class AttributeTestCase(unittest.TestCase):
     '''TestCase for the taurus.Attribute helper'''
@@ -416,7 +419,7 @@ class AttributeTestCase(unittest.TestCase):
             klass = TaurusAttribute
         manager = taurus.Manager()
         scheme = manager.getScheme(name)
-        supportedSchemes = manager.getPlugins().keys()
+        supportedSchemes = list(manager.getPlugins().keys())
         if scheme not in supportedSchemes:
             self.skipTest('"%s" scheme not supported' % scheme)
         a = taurus.Attribute(name)
@@ -438,7 +441,7 @@ class AttributeTestCase(unittest.TestCase):
         msg = ('read() for "%s" did not return a TaurusAttrValue (got a %s)' %
                (name, readvalue.__class__.__name__))
         self.assertTrue(isinstance(readvalue, TaurusAttrValue), msg)
-        for k, exp in expected.iteritems():
+        for k, exp in expected.items():
             try:
                 got = getattr(readvalue, k)
             except AttributeError:
@@ -463,5 +466,19 @@ class AttributeTestCase(unittest.TestCase):
                 self.assertTrue(chk, msg)
 
 
+class ValidatorFromName(unittest.TestCase):
+    """TestCase for the taurus.getValidatorFromName helper"""
+
+    def test_getValidatorFromName(self):
+        """check that getValidatorFromName returns the expected values"""
+
+        self.assertIsInstance(
+            taurus.getValidatorFromName('eval:@foo'),
+            taurus.core.evaluation.evalvalidator.EvaluationDeviceNameValidator
+        )
+        self.assertIsNone(taurus.getValidatorFromName('eval:@/'))
+        self.assertIsNone(taurus.getValidatorFromName('unsupported:scheme'))
+
+
 if __name__ == '__main__':
     pass
