@@ -33,15 +33,15 @@ The json implementation follows the rule:
     #. if python >= 2.6 use standard json from python distribution
     #. otherwise use private implementation distributed with taurus
 """
+from __future__ import absolute_import
+import taurus.tauruscustomsettings
 
 __docformat__ = "restructuredtext"
 
-import taurus.tauruscustomsettings
-
 LIGHTWEIGHT_IMPORTS = getattr(
     taurus.tauruscustomsettings, 'LIGHTWEIGHT_IMPORTS', False)
 
 if LIGHTWEIGHT_IMPORTS:
-    from init_lightweight import *
+    from .init_lightweight import *
 else:
-    from init_bkcomp import *
+    from .init_bkcomp import *
