@@ -23,6 +23,8 @@
 ##
 #############################################################################
 
+# TODO: This module should be updated to use argparse instead of optparse
+
 """Helper command line parser for taurus based on :mod:`optparse`.
 Suppose you have an application file::
 
@@ -82,6 +84,12 @@ __all__ = ["get_taurus_parser", "init_taurus_args", "parse_taurus_args",
 __docformat__ = "restructuredtext"
 
 
+from taurus.core.util import log as __log
+
+__log.deprecated(dep='taurus.core.util.argparse', rel='4.5.4',
+                 alt='argparse or (better) click')
+
+
 def get_taurus_parser(parser=None):
     """ Returns a :class:`optparse.OptionParser` initialized with a
     :class:`optparse.OptionGroup` containning some taurus options.
@@ -103,11 +111,12 @@ def get_taurus_parser(parser=None):
 
         help_tauruslog = "taurus log level. Allowed values are (case insensitive): critical, "\
                          "error, warning/warn, info, debug, trace"
-        help_tangohost = "Tango host name"
+        help_tangohost = "Tango host name (either HOST:PORT or a Taurus URI, e.g. tango://foo:1234)"
         help_tauruspolling = "taurus global polling period in milliseconds"
         help_taurusserial = "taurus serialization mode. Allowed values are (case insensitive): "\
             "serial, concurrent (default)"
         help_rcport = "enables remote debugging using the given port"
+        help_formatter = "Override the default formatter"
         group.add_option("--taurus-log-level", dest="taurus_log_level", metavar="LEVEL",
                          help=help_tauruslog, type="str", default="info")
         group.add_option("--taurus-polling-period", dest="taurus_polling_period", metavar="MILLISEC",
@@ -118,6 +127,9 @@ def get_taurus_parser(parser=None):
                          help=help_tangohost, type="str", default=None)
         group.add_option("--remote-console-port", dest="remote_console_port", metavar="PORT",
                          help=help_rcport, type="int", default=None)
+        group.add_option("--default-formatter", dest="default_formatter",
+                         metavar="FORMATTER", help=help_formatter, type="str",
+                         default=None)
         parser.add_option_group(group)
     return parser
 
@@ -171,7 +183,8 @@ def init_taurus_args(parser=None, args=None, values=None):
     # initialize tango host
     if options.tango_host is not None:
         tango_factory = taurus.Factory("tango")
-        tango_factory.set_default_tango_host(options.tango_host)
+        tango_host = options.tango_host
+        tango_factory.set_default_tango_host(tango_host)
 
     # initialize taurus polling period
     if options.taurus_polling_period is not None:
@@ -193,8 +206,14 @@ def init_taurus_args(parser=None, args=None, values=None):
             rfoo.utils.rconsole.spawn_server(port=options.remote_console_port)
             taurus.info("rconsole started. You can connect to it by typing: rconsole -p %d",
                         options.remote_console_port)
-        except Exception, e:
-            taurus.warning("Cannot spawn debugger. Reason: %s", str(e))
+        except Exception as e:
+            taurus.warning("Cannot spawn debugger. Reason: %s", e)
+
+    # initialize default formatter
+    if options.default_formatter is not None:
+        from taurus import tauruscustomsettings
+        setattr(tauruscustomsettings, 'DEFAULT_FORMATTER',
+                options.default_formatter)
 
     return parser, options, args
 
