@@ -62,21 +62,32 @@ A Taurus related example::
     >>> codec = CodecFactory().getCodec(v.format)
     >>> f, d = codec.decode((v.format, v.value))
 """
-
-__all__ = ["Codec", "NullCodec", "ZIPCodec", "BZ2Codec", "JSONCodec",
-           "FunctionCodec", "PlotCodec", "CodecPipeline", "CodecFactory"]
-
-__docformat__ = "restructuredtext"
+from __future__ import absolute_import
+from builtins import str, bytes
 
 import copy
 
 # need by VideoImageCodec
 import struct
+import sys
 import numpy
 
-from singleton import Singleton
-from log import Logger
-from containers import CaselessDict
+from future.utils import (PY2, string_types)
+
+from .singleton import Singleton
+from .log import Logger
+from .containers import CaselessDict
+
+__all__ = ["Codec", "NullCodec", "ZIPCodec", "BZ2Codec", "JSONCodec",
+           "Utf8Codec", "FunctionCodec", "PlotCodec", "CodecPipeline",
+           "CodecFactory"]
+
+__docformat__ = "restructuredtext"
+
+if PY2:
+    buffer_types = buffer, memoryview,
+else:
+    buffer_types = memoryview,
 
 
 class Codec(Logger):
@@ -161,7 +172,7 @@ class ZIPCodec(Codec):
         'Hello world\\nHello wo'"""
 
     def encode(self, data, *args, **kwargs):
-        """encodes the given data to a gzip string. The given data **must** be a string
+        """encodes the given data to gzip bytes. The given data **must** be bytes
 
         :param data: (sequence[str, obj]) a sequence of two elements where the first item is the encoding format of the second item object
 
@@ -173,7 +184,7 @@ class ZIPCodec(Codec):
         return format, zlib.compress(data[1])
 
     def decode(self, data, *args, **kwargs):
-        """decodes the given data from a gzip string.
+        """decodes the given data from a gzip bytes.
 
         :param data: (sequence[str, obj]) a sequence of two elements where the first item is the encoding format of the second item object
 
@@ -204,7 +215,7 @@ class BZ2Codec(Codec):
         'Hello world\\nHello wo'"""
 
     def encode(self, data, *args, **kwargs):
-        """encodes the given data to a bz2 string. The given data **must** be a string
+        """encodes the given data to bz2 bytes. The given data **must** be bytes
 
         :param data: (sequence[str, obj]) a sequence of two elements where the first item is the encoding format of the second item object
 
@@ -216,7 +227,7 @@ class BZ2Codec(Codec):
         return format, bz2.compress(data[1])
 
     def decode(self, data, *args, **kwargs):
-        """decodes the given data from a bz2 string.
+        """decodes the given data from bz2 bytes.
 
         :param data: (sequence[str, obj]) a sequence of two elements where the first item is the encoding format of the second item object
 
@@ -249,7 +260,7 @@ class PickleCodec(Codec):
         {'hello': 'world', 'goodbye': 1000}"""
 
     def encode(self, data, *args, **kwargs):
-        """encodes the given data to a pickle string. The given data **must** be
+        """encodes the given data to pickle bytes. The given data **must** be
         a python object that :mod:`pickle` is able to convert.
 
         :param data: (sequence[str, obj]) a sequence of two elements where the
@@ -278,8 +289,8 @@ class PickleCodec(Codec):
             return data
         format = data[0].partition('_')[2]
 
-        if isinstance(data[1], buffer):
-            data = data[0], str(data[1])
+        if isinstance(data[1], buffer_types):
+            data = data[0], bytes(data[1])
 
         return format, pickle.loads(data[1])
 
@@ -338,16 +349,16 @@ class JSONCodec(Codec):
 
         ensure_ascii = kwargs.pop('ensure_ascii', False)
 
-        if isinstance(data[1], buffer):
+        if isinstance(data[1], buffer_types):
             data = data[0], str(data[1])
-
+        
         data = json.loads(data[1])
         if ensure_ascii:
             data = self._transform_ascii(data)
         return format, data
 
     def _transform_ascii(self, data):
-        if isinstance(data, unicode):
+        if isinstance(data, string_types):
             return data.encode('utf-8')
         elif isinstance(data, dict):
             return self._transform_dict(data)
@@ -363,11 +374,61 @@ class JSONCodec(Codec):
 
     def _transform_dict(self, dct):
         newdict = {}
-        for k, v in dct.iteritems():
+        for k, v in dct.items():
             newdict[self._transform_ascii(k)] = self._transform_ascii(v)
         return newdict
 
 
+class Utf8Codec(Codec):
+    """A codec able to encode/decode utf8 strings to/from bytes.
+    Useful to adapt i/o encodings in a codec pipe.
+
+    Example::
+
+        >>> from taurus.core.util.codecs import CodecFactory
+
+        >>> cf = CodecFactory()
+        >>> codec = cf.getCodec('zip_utf8_json')
+        >>>
+        >>> # first encode something
+        >>> data = { 'hello' : 'world', 'goodbye' : 1000 }
+        >>> format, encoded_data = codec.encode(("", data))
+        >>>
+        >>> # now decode it
+        >>> _, decoded_data = codec.decode((format, encoded_data))
+        >>> print decoded_data
+    """
+
+    def encode(self, data, *args, **kwargs):
+        """
+        Encodes the given utf8 string to bytes.
+
+        :param data: (sequence[str, obj]) a sequence of two elements where the
+                     first item is the encoding format of the second item object
+
+        :return: (sequence[str, obj]) a sequence of two elements where the
+                 first item is the encoding format of the second item object
+        """
+        format = 'utf8'
+        fmt, data = data
+        if len(fmt):
+            format += '_%s' % fmt
+        return format, str(data).encode()
+
+    def decode(self, data, *args, **kwargs):
+        """decodes the given data from a bytes.
+
+        :param data: (sequence[str, obj]) a sequence of two elements where the
+                     first item is the encoding format of the second item object
+
+        :return: (sequence[str, obj]) a sequence of two elements where the
+                 first item is the encoding format of the second item object
+        """
+        fmt, data = data
+        fmt = fmt.partition('_')[2]
+        return fmt, bytes(data).decode()
+
+
 class BSONCodec(Codec):
     """A codec able to encode/decode to/from bson format. It uses the
     :mod:`bson` module.
@@ -425,7 +486,7 @@ class BSONCodec(Codec):
         return format, data
 
     def _transform_ascii(self, data):
-        if isinstance(data, unicode):
+        if isinstance(data, string_types):
             return data.encode('utf-8')
         elif isinstance(data, dict):
             return self._transform_dict(data)
@@ -441,7 +502,7 @@ class BSONCodec(Codec):
 
     def _transform_dict(self, dct):
         newdict = {}
-        for k, v in dct.iteritems():
+        for k, v in dct.items():
             newdict[self._transform_ascii(k)] = self._transform_ascii(v)
         return newdict
 
@@ -538,7 +599,19 @@ class VideoImageCodec(Codec):
 
         imgBuffer = data[1][struct.calcsize(self.VIDEO_HEADER_FORMAT):]
         dtype = self.__getDtypeId(header['imageMode'])
-        if header['imageMode'] == 7:
+
+        if header['imageMode'] == 6:
+            # RGB24, 3 bytes per pixel
+            rgba = numpy.fromstring(imgBuffer, dtype)
+            bbuf = rgba[0::3]
+            gbuf = rgba[1::3]
+            rbuf = rgba[2::3]
+            r = rbuf.reshape(header['height'], header['width'])
+            g = gbuf.reshape(header['height'], header['width'])
+            b = bbuf.reshape(header['height'], header['width'])
+            img2D = numpy.dstack((r, g, b))
+
+        elif header['imageMode'] == 7:
             # RGBA 4 bytes per pixel
             rgba = numpy.fromstring(imgBuffer, dtype)
             bbuf = rgba[0::4]
@@ -659,7 +732,7 @@ class VideoImageCodec(Codec):
     def __packHeader(self, imgMode, frameNumber, width, height):
         magic = 0x5644454f
         version = 1
-        endian = ord(struct.pack('=H', 1)[-1])
+        endian = 0 if sys.byteorder == 'little' else 1
         hsize = struct.calcsize(self.VIDEO_HEADER_FORMAT)
         return struct.pack(self.VIDEO_HEADER_FORMAT,
                            magic,
@@ -686,7 +759,7 @@ class VideoImageCodec(Codec):
             # TODO: other modes
             #'RGB555'     : 4,#Core.RGB555,
             #'RGB565'     : 5,#Core.RGB565,
-            #'RGB24'      : 6,#Core.RGB24,
+            'RGB24': 6,  # Core.RGB24,
             'RGB32': 7,  # Core.RGB32,
             #'BGR24'      : 8,#Core.BGR24,
             #'BGR32'      : 9,#Core.BGR32,
@@ -707,7 +780,7 @@ class VideoImageCodec(Codec):
                 3: 'L',
                 #'RGB555'     : Core.RGB555,
                 #'RGB565'     : Core.RGB565,
-                #'RGB24'      : Core.RGB24,
+                6: 'RGB24',  # Core.RGB24,
                 7: 'RGB32',  # Core.RGB32,
                 # 8     : 'BGR24',#Core.BGR24,
                 #'BGR32'      : Core.BGR32,
@@ -728,7 +801,7 @@ class VideoImageCodec(Codec):
                 3: 'uint64',
                 #'RGB555'     : Core.RGB555,
                 #'RGB565'     : Core.RGB565,
-                # 6      : 'uint8', # Core.RGB24,
+                6: 'uint8',  # Core.RGB24,
                 7: 'uint8',  # Core.RGB32,
                 #'BGR24'      : Core.BGR24,
                 #'BGR32'      : Core.BGR32,
@@ -842,6 +915,7 @@ class CodecFactory(Singleton, Logger):
     #: Default minimum map of registered codecs
     CODEC_MAP = CaselessDict({
         'json': JSONCodec,
+        'utf8': Utf8Codec,
         'bson': BSONCodec,
         'bz2': BZ2Codec,
         'zip': ZIPCodec,
@@ -879,7 +953,7 @@ class CodecFactory(Singleton, Logger):
         self._codec_klasses[format] = klass
 
         # del old codec if exists
-        if self._codecs.has_key(format):
+        if format in self._codecs:
             del self._codecs[format]
 
     def unregisterCodec(self, format):
@@ -889,10 +963,10 @@ class CodecFactory(Singleton, Logger):
         :param format: (str) the codec id
 
         :raises: KeyError"""
-        if self._codec_klasses.has_key(format):
+        if format in self._codec_klasses:
             del self._codec_klasses[format]
 
-        if self._codecs.has_key(format):
+        if format in self._codecs:
             del self._codecs[format]
 
     def getCodec(self, format):
