@@ -26,6 +26,17 @@
 """
 event.py:
 """
+from __future__ import print_function
+from __future__ import absolute_import
+from builtins import range
+from builtins import object
+import sys
+import weakref
+import threading
+import time
+import collections
+import taurus.core
+
 
 __all__ = ["BoundMethodWeakref", "CallableRef", "EventGenerator",
            "ConfigEventGenerator", "ListEventGenerator", "EventListener",
@@ -33,14 +44,6 @@ __all__ = ["BoundMethodWeakref", "CallableRef", "EventGenerator",
 
 __docformat__ = "restructuredtext"
 
-import sys
-import weakref
-import threading
-import time
-import operator
-
-import taurus.core
-
 
 class BoundMethodWeakref(object):
     """This class represents a weak reference to a method of an object since
@@ -48,8 +51,8 @@ class BoundMethodWeakref(object):
 
     def __init__(self, bound_method, del_cb=None):
         cb = (del_cb and self._deleted)
-        self.func_ref = weakref.ref(bound_method.im_func, cb)
-        self.obj_ref = weakref.ref(bound_method.im_self, cb)
+        self.func_ref = weakref.ref(bound_method.__func__, cb)
+        self.obj_ref = weakref.ref(bound_method.__self__, cb)
         if cb:
             self.del_cb = CallableRef(del_cb)
         self.already_deleted = 0
@@ -73,11 +76,21 @@ class BoundMethodWeakref(object):
 
     def __cmp__(self, other):
         if other.__class__ == self.__class__:
+            from past.builtins import cmp
             ret = cmp((self.func_ref, self.obj_ref),
                       (other.func_ref, other.obj_ref))
             return ret
         return 1
 
+    def __eq__(self, other):
+        if hasattr(other, 'func_ref') and hasattr(other, 'obj_ref'):
+            return ((self.func_ref, self.obj_ref)
+                    == (other.func_ref, other.obj_ref))
+        return False
+
+    def __ne__(self, other):
+        return not self == other
+
     def __repr__(self):
         obj, func = self.obj_ref(), self.func_ref()
         return 'BoundMethodWeakRef of %s.%s' % (obj, func)
@@ -93,13 +106,41 @@ def CallableRef(object, del_cb=None):
     :type del_cb: callable object or None
 
     :return: a weak reference for the given callable
-    :rtype: BoundMethodWeakref or weakref.ref"""
-    if hasattr(object, 'im_self'):
-        if object.im_self is not None:
-            return BoundMethodWeakref(object, del_cb)
+    :rtype: taurus.core.util.BoundMethodWeakref or weakref.ref
+    """
+    im_self = None
+    if hasattr(object, '__self__'):
+        im_self = object.__self__
+    elif hasattr(object, 'im_self'):
+        im_self = object.im_self
+
+    if im_self is not None:
+        return BoundMethodWeakref(object, del_cb)
     return weakref.ref(object, del_cb)
 
 
+# Reimplementation of BoundMethodWeakref class to avoid to have a hard
+# reference in the event callbacks.
+# Related to "Keeping references to event callbacks after unsubscribe_event"
+# PyTango #185 issue.
+class _BoundMethodWeakrefWithCall(BoundMethodWeakref):
+
+    def __init__(self, bound_method, del_cb=None):
+        """ Reimplementation of __init__ method"""
+        super(_BoundMethodWeakrefWithCall, self).__init__(bound_method,
+                                                          del_cb=del_cb)
+        self.__name__ = self.func_ref().__name__
+
+    def __call__(self, *args, **kwargs):
+        """ Retrieve references and call callback with arguments
+        """
+        obj = self.obj_ref()
+        if obj is not None:
+            func = self.func_ref()
+            if func is not None:
+                return func(obj, *args, **kwargs)
+
+
 class EventStack(object):
     "internal usage event stack"
 
@@ -135,7 +176,7 @@ class EventStack(object):
         return read
 
 
-from object import Object
+from .object import Object
 
 
 class EventGenerator(Object):
@@ -172,16 +213,21 @@ class EventGenerator(Object):
         else:
             pass
 
-    def subscribeEvent(self, cb, data=None):
+    def subscribeEvent(self, cb, data=None, with_first_event=True):
         """Subscribes to the event
 
         :param cb: a callable object
         :type  cb: callable
         :param data: extra data to send each time an event is triggered on the
-                     given callback. Default is None
-        :type  data: object"""
+            given callback. Default is None.
+        :type  data: object
+        :param with_first_event: whether call the callback with the first event
+            value (the most recent value) during the subscription process.
+            Default is True.
+        :type data: boolean
+        """
         if not self.events_active:
-            raise RuntimeError, ('%s does not have '
+            raise RuntimeError('%s does not have '
                                  'events/polling active' % self.event_name)
 
         cb_ref = CallableRef(cb, self.unsubscribeDeletedEvent)
@@ -189,10 +235,11 @@ class EventGenerator(Object):
         try:
             self.lock()
             if (cb_ref, data) in self.cb_list:
-                raise RuntimeError, ('Callback %s(%s) already reg. on %s' %
+                raise RuntimeError('Callback %s(%s) already reg. on %s' %
                                      (cb, data, self.event_name))
             self.cb_list.append((cb_ref, data))
-            cb(data, self.first_event_val)
+            if with_first_event:
+                cb(data, self.first_event_val)
         finally:
             self.unlock()
 
@@ -201,7 +248,7 @@ class EventGenerator(Object):
         try:
             self.lock()
             aux_list = list(self.cb_list)
-            for i in xrange(len(aux_list) - 1, -1, -1):
+            for i in range(len(aux_list) - 1, -1, -1):
                 pair = self.cb_list[i]
                 if pair[0] is cb_ref:
                     del self.cb_list[i]
@@ -292,7 +339,7 @@ class EventGenerator(Object):
         :return: the value of the event that unblocked the wait
         :rtype: object"""
         if not self.events_active:
-            raise RuntimeError, ('%s does not have '
+            raise RuntimeError('%s does not have '
                                  'events/polling active' % self.event_name)
         try:
             self.lock()
@@ -415,7 +462,7 @@ class EventListener(object):
                 if t and t >= after:
                     return
             else:
-                for v, t in s.items():
+                for v, t in list(s.items()):
                     if v == val:
                         continue
                     if t >= after:
@@ -517,8 +564,8 @@ class AttributeEventWait(object):
                 name = th.name
             else:
                 name = "<unknown>"
-            print "WARNING: Thread %s trying to unlock condition previously " \
-                  "locked by thread %s" % (curr_th.name, name)
+            print("WARNING: Thread %s trying to unlock condition previously " \
+                  "locked by thread %s" % (curr_th.name, name))
 
     def clearEventSet(self):
         "Clears the internal event buffer"
@@ -534,7 +581,7 @@ class AttributeEventWait(object):
         elif t == taurus.core.taurusbasetypes.TaurusEventType.Error:
             self.fireEvent(None)
         else:
-            self.fireEvent(v.value)
+            self.fireEvent(v.rvalue)
 
     def fireEvent(self, v):
         """Notifies that a given event has arrived
@@ -630,9 +677,9 @@ class AttributeEventWait(object):
                             return
                 self._cond.wait(timeout)
                 retries -= 1
-        except Exception, e:
+        except Exception as e:
             sys.stderr.write(
-                "AttributeEventWait: Caught exception while waitting: %s\n" % str(e))
+                "AttributeEventWait: Caught exception while waiting: %s\n" % str(e))
             raise e
         finally:
             self.unlock()
@@ -648,7 +695,7 @@ class AttributeEventIterator(object):
             self.connect(attrs)
 
     def connect(self, attrs):
-        if not operator.isSequenceType(attrs):
+        if not isinstance(attrs, collections.Sequence):
             attrs = (attrs,)
         self.disconnect()
         self._attrs = attrs
@@ -671,13 +718,13 @@ class AttributeEventIterator(object):
             lock = getattr(self._cond, "_Condition__lock")
             th = getattr(lock, "_RLock__owner")
             curr_th = threading.current_thread()
-            print "WARNING: Thread %s trying to unlock condition previously " \
-                  "locked by thread %s" % (curr_th.name, th.name)
+            print(("WARNING: Thread %s trying to unlock condition previously "
+                   + "locked by thread %s") % (curr_th.name, th.name))
 
     def eventReceived(self, s, t, v):
         if t not in (taurus.core.taurusbasetypes.TaurusEventType.Change, taurus.core.taurusbasetypes.TaurusEventType.Periodic):
             return
-        self.fireEvent(s, v.value)
+        self.fireEvent(s, v.rvalue)
 
     def fireEvent(self, s, v):
         t = time.time()
@@ -694,7 +741,7 @@ class AttributeEventIterator(object):
             while True:
                 self._cond.wait(timeout)
                 yield self._data
-        except Exception, e:
-            print "INFO: Caught exception while waiting", str(e)
+        except Exception as e:
+            print("INFO: Caught exception while waiting", str(e))
         finally:
             self.unlock()
