@@ -2,24 +2,24 @@
 
 #############################################################################
 ##
-## This file is part of Taurus
-## 
-## http://taurus-scada.org
+# This file is part of Taurus
 ##
-## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
-## 
-## Taurus is free software: you can redistribute it and/or modify
-## it under the terms of the GNU Lesser General Public License as published by
-## the Free Software Foundation, either version 3 of the License, or
-## (at your option) any later version.
-## 
-## Taurus is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU Lesser General Public License for more details.
-## 
-## You should have received a copy of the GNU Lesser General Public License
-## along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
+# http://taurus-scada.org
+##
+# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
+##
+# Taurus is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+##
+# Taurus is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser General Public License for more details.
+##
+# You should have received a copy of the GNU Lesser General Public License
+# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
 ##
 #############################################################################
 
@@ -34,6 +34,7 @@ import threading
 
 from .log import Logger
 
+
 class Timer(Logger):
     """
     Timer Object.
@@ -57,7 +58,7 @@ class Timer(Logger):
         self.__start_nb = 0
         self.__thread = None
         self.__strict_timing = strict_timing
-        
+
     def start(self):
         """ Start Timer Object """
         self.__lock.acquire()
@@ -68,19 +69,20 @@ class Timer(Logger):
                 self.__alive = True
                 self.__start_nb += 1
                 thread_name = "TimerLoop %d" % self.__start_nb
-                self.__thread = threading.Thread(target=self.__run, name = thread_name)
+                self.__thread = threading.Thread(
+                    target=self.__run, name=thread_name)
                 self.__thread.setDaemon(True)
                 self.__thread.start()
         finally:
             self.__lock.release()
-    
+
     def stop(self):
         """ Stop Timer Object """
         self.debug("Timer::stop()")
         self.__lock.acquire()
         self.__loop = False
         self.__lock.release()
-    
+
     def __run(self):
         """ Private Thread Function """
         self.debug("Timer thread starting")
