@@ -23,26 +23,9 @@
 ##
 ##############################################################################
 
-from __future__ import absolute_import
+from taurus.core.util import log as __log
 
-from taurus.core.util.log import deprecated
+__log.deprecated(dep='taurus.external.ordereddict', rel='4.0.3',
+                 alt='collections.OrderedDict')
 
-
-deprecated(dep='taurus.external.ordereddict', rel='4.0', alt='ordereddict')
-
-
-try:
-    # ordereddict from python 2.7 or from ordereddict installed package?
-    from ordereddict import *
-except ImportError:
-    # ordereddict from local import
-    import os
-    import sys
-    import warnings
-    warnings.warn("ordereddict not available. Using local ordereddict",
-                  ImportWarning)
-    sys.path.append(os.path.dirname(__file__))
-    from ordereddict import *
-    del warnings
-    del sys
-    del os
+from collections import OrderedDict
