@@ -27,7 +27,10 @@
 comunications.py:
 """
 
-from taurus.external.qt import QtCore
+from __future__ import print_function
+
+from taurus.external.qt import QtCore, compat
+
 import weakref
 
 _DEBUG = False
@@ -54,7 +57,7 @@ class DataModel(QtCore.QObject):
     DataModels are singletons.
     '''
 
-    dataChanged = QtCore.pyqtSignal(object)
+    dataChanged = QtCore.pyqtSignal(compat.PY_OBJECT)
 
     def __init__(self, parent, dataUID, defaultData=None):
         '''
@@ -327,16 +330,16 @@ class SharedDataManager(QtCore.QObject):
 
         :returns: (list<str>) UIDs of currently shared data.
         '''
-        return self.__models.keys()
+        return list(self.__models.keys())
 
     def debugReader(self, data):
         '''
         A slot which you can connect as a reader for debugging. It will print info to the stdout
         '''
-        print "SharedDataManager: \n\tSender=: %s\n\tData=%s" % (self.sender(), repr(data))
+        print("SharedDataManager: \n\tSender=: %s\n\tData=%s" % (self.sender(), repr(data)))
 
     def info(self):
         s = ""
-        for uid, m in sorted(self.__models.iteritems()):
+        for uid, m in sorted(self.__models.items()):
             s += m.info() + '\n'
         return s
