@@ -24,6 +24,15 @@
 #############################################################################
 
 """This module provides widgets that display the database in a tree format"""
+# TODO: tango-centric
+
+from builtins import str, bytes
+
+from taurus.external.qt import Qt
+from taurus.core.taurusbasetypes import TaurusElementType, TaurusDevState
+import taurus.qt.qtcore.mimetypes
+from .taurusmodel import TaurusBaseTreeItem, TaurusBaseModel, TaurusBaseProxyModel
+
 
 __all__ = ["TaurusTreeDevicePartItem", "TaurusTreeDeviceDomainItem",
            "TaurusTreeDeviceFamilyItem", "TaurusTreeDeviceMemberItem", "TaurusTreeSimpleDeviceItem",
@@ -38,13 +47,6 @@ __all__ = ["TaurusTreeDevicePartItem", "TaurusTreeDeviceDomainItem",
 
 __docformat__ = 'restructuredtext'
 
-from taurus.external.qt import Qt
-from taurus.core.taurusbasetypes import TaurusElementType, TaurusDevState
-import taurus.qt.qtcore.mimetypes
-
-from taurus.core.tango.tangodatabase import TangoInfo, TangoDatabase
-
-from .taurusmodel import TaurusBaseTreeItem, TaurusBaseModel, TaurusBaseProxyModel
 
 ElemType = TaurusElementType
 
@@ -80,7 +82,16 @@ def getDevStateToolTip(*args, **kwargs):
 
 
 class TaurusTreeDbBaseItem(TaurusBaseTreeItem):
-    DisplayFunc = TangoInfo.name
+    try:
+        # TODO: tango-centric
+        import taurus.core.tango
+
+        @staticmethod
+        def DisplayFunc(obj):
+            from taurus.core.tango.tangodatabase import TangoInfo
+            return TangoInfo.name(obj)
+    except:
+        pass
 
 
 class TaurusTreeDevicePartItem(TaurusTreeDbBaseItem):
@@ -413,13 +424,15 @@ class TaurusDbBaseModel(TaurusBaseModel):
             mime_data_item = tree_item.mimeData(index)
             if mime_data_item is None:
                 continue
-            data.append(mime_data_item)
+            data.append(bytes(mime_data_item, encoding='utf8'))
         ret.setData(
-            taurus.qt.qtcore.mimetypes.TAURUS_MODEL_LIST_MIME_TYPE, "\r\n".join(data))
-        ret.setText(", ".join(data))
+            taurus.qt.qtcore.mimetypes.TAURUS_MODEL_LIST_MIME_TYPE,
+            b"\r\n".join(data)
+        )
+        ret.setText(", ".join(map(str, data)))
         if len(data) == 1:
             ret.setData(
-                taurus.qt.qtcore.mimetypes.TAURUS_MODEL_MIME_TYPE, str(data[0]))
+                taurus.qt.qtcore.mimetypes.TAURUS_MODEL_MIME_TYPE, data[0])
         return ret
 
     def pyData(self, index, role):
@@ -458,6 +471,8 @@ class TaurusDbBaseModel(TaurusBaseModel):
     def setupModelData(self, data):
         if data is None:
             return
+
+        from taurus.core.tango.tangodatabase import TangoDatabase
         if isinstance(data, TangoDatabase):
             data = data.cache()
         devices = data.devices()
@@ -487,6 +502,8 @@ class TaurusDbSimpleDeviceAliasModel(TaurusDbBaseModel):
     def setupModelData(self, data):
         if data is None:
             return
+
+        from taurus.core.tango.tangodatabase import TangoDatabase
         if isinstance(data, TangoDatabase):
             data = data.cache()
         devices = data.devices()
@@ -510,6 +527,8 @@ class TaurusDbPlainDeviceModel(TaurusDbBaseModel):
     def setupModelData(self, data):
         if data is None:
             return
+
+        from taurus.core.tango.tangodatabase import TangoDatabase
         if isinstance(data, TangoDatabase):
             data = data.cache()
         devices = data.devices()
@@ -522,7 +541,7 @@ class TaurusDbPlainDeviceModel(TaurusDbBaseModel):
 
 
 class TaurusDbDeviceModel(TaurusDbBaseModel):
-    """A Qt model that structures device elements is a 3 level tree organized
+    """A Qt model that structures device elements in a 3 level tree organized
        as:
 
            - <domain>
@@ -534,19 +553,25 @@ class TaurusDbDeviceModel(TaurusDbBaseModel):
     def setupModelData(self, data):
         if data is None:
             return
+        try:
+            # TODO: Tango-centric
+            # TODO: is this try needed? (not done in, e.g. TaurusDbPlainDeviceModel)
+            from taurus.core.tango.tangodatabase import TangoDatabase
+        except ImportError:
+            return
         if isinstance(data, TangoDatabase):
             data = data.deviceTree()
 
         rootItem = self._rootItem
-        for domain in data.keys():
+        for domain in data:
             families = data[domain]
             domainItem = TaurusTreeDeviceDomainItem(
                 self, domain.upper(), rootItem)
-            for family in families.keys():
+            for family in families:
                 members = families[family]
                 familyItem = TaurusTreeDeviceFamilyItem(
                     self, family.upper(), domainItem)
-                for member in members.keys():
+                for member in members:
                     dev = members[member]
                     memberItem = TaurusTreeDeviceItem(
                         self, dev, parent=familyItem)
@@ -564,6 +589,7 @@ class TaurusDbPlainServerModel(TaurusDbBaseModel):
         if data is None:
             return
 
+        from taurus.core.tango.tangodatabase import TangoDatabase
         if isinstance(data, TangoDatabase):
             data = data.cache()
 
@@ -594,6 +620,7 @@ class TaurusDbServerModel(TaurusDbBaseModel):
         if data is None:
             return
 
+        from taurus.core.tango.tangodatabase import TangoDatabase
         if isinstance(data, TangoDatabase):
             data = data.cache()
 
@@ -645,6 +672,7 @@ class TaurusDbDeviceClassModel(TaurusDbBaseModel):
         if data is None:
             return
 
+        from taurus.core.tango.tangodatabase import TangoDatabase
         if isinstance(data, TangoDatabase):
             data = data.cache()
 
@@ -708,13 +736,11 @@ class TaurusDbDeviceProxyModel(TaurusDbBaseProxyModel):
     def deviceMatches(self, device, regexp):
         name = device.name()
 
-        # if Qt.QString(name).contains(regexp):
         if regexp.indexIn(name) != -1:
             return True
         name = device.alias()
         if name is None:
             return False
-        # return Qt.QString(name).contains(regexp)
         return regexp.indexIn(name) != -1
 
 
@@ -732,14 +758,13 @@ class TaurusDbServerProxyModel(TaurusDbBaseProxyModel):
             serverName = treeItem.display()
             serverInstances = sourceModel.getServerNameInstances(serverName)
             for serverInstance in serverInstances:
-                # if Qt.QString(serverInstance.name()).contains(regexp):
                 if regexp.indexIn(serverInstance.name()) != -1:
                     return True
             return False
 
         if isinstance(treeItem, TaurusTreeServerItem):
             # return treeItem.qdisplay().contains(regexp)
-            return regexp.indexIn(treeItem.qdisplay()) != -1
+            return regexp.indexIn(treeItem.display()) != -1
 
         return True
 
@@ -758,4 +783,4 @@ class TaurusDbDeviceClassProxyModel(TaurusDbBaseProxyModel):
         regexp = self.filterRegExp()
 
         # return treeItem.qdisplay().contains(regexp)
-        return regexp.indexIn(treeItem.qdisplay()) != -1
+        return regexp.indexIn(treeItem.display()) != -1
