@@ -4,38 +4,42 @@
 ##
 # This file is part of Taurus
 ##
-# http://www.tango-controls.org/static/tau/latest/doc/html/index.html
+# http://taurus-scada.org
 ##
-# (copyleft) CELLS / ALBA Synchrotron, Bellaterra, Spain
+# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
 ##
-# This is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
+# Taurus is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 ##
-# This software is distributed in the hope that it will be useful,
+# Taurus is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Lesser General Public License for more details.
 ##
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, see <http://www.gnu.org/licenses/>.
-###########################################################################
+# You should have received a copy of the GNU Lesser General Public License
+# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
+##
+#############################################################################
 
 """
-emitter.py: This module provides a task scheduler used by TaurusGrid and TaurusDevTree widgets
+emitter.py: This module provides a task scheduler used by TaurusGrid and 
+    TaurusDevTree widgets
 """
 
-import Queue
+from future import standard_library
+standard_library.install_aliases()
+from builtins import str
+from builtins import object
+from queue import Queue, Empty
 import traceback
-from functools import partial
-from collections import Iterable
+
+from future.utils import string_types
 
 import taurus
 from taurus.external.qt import Qt
 from taurus.core.util.log import Logger
-from taurus.core.util.singleton import Singleton
-
 
 
 ###############################################################################
@@ -43,7 +47,7 @@ from taurus.core.util.singleton import Singleton
 
 
 def isString(seq):
-    if isinstance(seq, basestring):
+    if isinstance(seq, string_types):
         return True  # It matches most python str-like classes
     if any(s in str(type(seq)).lower() for s in ('vector', 'array', 'list',)):
         return False
@@ -77,7 +81,7 @@ class MethodModel(object):
 class QEmitter(Qt.QObject):
     """Emitter class providing two signals."""
 
-    doSomething = Qt.pyqtSignal(Iterable)
+    doSomething = Qt.pyqtSignal(list)
     somethingDone = Qt.pyqtSignal()
     newQueue = Qt.pyqtSignal()
 
@@ -87,36 +91,43 @@ class QEmitter(Qt.QObject):
 
 class TaurusEmitterThread(Qt.QThread):
     """
-    The TaurusEmitterThread Class
-    ==========================
-
-    This object get items from a python Queue and performs a thread safe operation on them.
+    This object get items from a python Queue and performs a thread safe 
+    operation on them.
     It is useful to serialize Qt tasks in a background thread.
 
     :param parent: a Qt/Taurus object
     :param name: identifies object logs
-    :param queue: if None parent.getQueue() is used, if not then the queue passed as argument is used
+    :param queue: if None parent.getQueue() is used, if not then the queue 
+        passed as argument is used
     :param method: the method to be executed using each queue item as argument
-    :param cursor: if True or QCursor a custom cursor is set while the Queue is not empty
+    :param cursor: if True or QCursor a custom cursor is set while 
+        the Queue is not empty
 
-    How TaurusEmitterThread works
-    --------------------------
+    How TaurusEmitterThread works:
 
-    TaurusEmitterThread is a worker thread that processes a queue of iterables passed as arguments to the specified method every time that  ``doSomething()`` is called:
+    TaurusEmitterThread is a worker thread that processes a queue of iterables 
+    passed as arguments to the specified method every time that  
+    ``doSomething()`` is called:
 
-     * ``self.method(*item)`` will be called if TaurusEmitterThread.method has been initialized.
-     * ``item[0](item[1:])`` will be called if ``method`` is not initialized and the first element of the queued iterable is *callable*.
+     * ``self.method(*item)`` will be called if TaurusEmitterThread.method 
+        has been initialized.
+     * ``item[0](item[1:])`` will be called if ``method`` is not initialized 
+        and the first element of the queued iterable is *callable*.
 
     TaurusEmitterThread uses two queues:
 
      * ``self.queue`` manages the objects added externally:
 
-      + the ``next()`` method passes objects from ``self.queue`` to ``self.todo queue``
+      + the ``next()`` method passes objects from ``self.queue`` 
+        to ``self.todo queue``
       + every time that a *somethingDone* signal arrives ``next()`` is called.
-      + ``next()`` can be called also externally to ensure that the main queue is being processed.
+      + ``next()`` can be called also externally to ensure that the main queue 
+        is being processed.
       + the queue can be accessed externally using ``getQueue()``
-      + ``getQueue().qsize()`` returns the number of remaining objects in queue.
-      + while there are objects in queue the ``.next()`` method will override applications cursor. a call to ``next()`` with an empty queue will restore the original cursor.
+      + ``getQueue().qsize()`` returns number of remaining objects in queue.
+      + while there are objects in queue the ``.next()`` method will 
+        override applications cursor. a call to ``next()`` with an empty queue 
+        will restore the original cursor.
 
      * ``self.todo`` is managed by the ``run()/start()`` method:
 
@@ -124,13 +135,12 @@ class TaurusEmitterThread(Qt.QThread):
       - if an object is found, it is sent in a *doSomething* signal.
       - if *"exit"* is found the loop exits.
 
-    Usage example
-    -------------
+    Usage example:
 
     .. code-block:: python
 
         #Applying TaurusEmitterThread to an existing class:
-        import Queue
+        from queue import Queue
         from functools import partial
 
         def modelSetter(args):
@@ -141,18 +151,21 @@ class TaurusEmitterThread(Qt.QThread):
             ...
             def __init__(self, parent = None, designMode = False):
                 ...
-                self.modelsQueue = Queue.Queue()
-                self.modelsThread = TaurusEmitterThread(parent=self,queue=self.modelsQueue,method=modelSetter )
+                self.modelsQueue = Queue()
+                self.modelsThread = TaurusEmitterThread(parent=self,
+                        queue=self.modelsQueue,method=modelSetter )
                 ...
             def build_widgets(...):
                 ...
-                            previous,synoptic_value = synoptic_value,TaurusValue(cell_frame)
+                            previous,synoptic_value = \
+                                synoptic_value,TaurusValue(cell_frame)
                             #synoptic_value.setModel(model)
                             self.modelsQueue.put((synoptic_value,model))
                 ...
             def setModel(self,model):
                 ...
-                if hasattr(self,'modelsThread') and not self.modelsThread.isRunning():
+                if hasattr(self,'modelsThread') and \
+                        not self.modelsThread.isRunning():
                     self.modelsThread.start()
                 elif self.modelsQueue.qsize():
                     self.modelsThread.next()
@@ -160,21 +173,36 @@ class TaurusEmitterThread(Qt.QThread):
 
     """
 
-    def __init__(self, parent=None, name='', queue=None, method=None, cursor=None, sleep=5000):
+    def __init__(self, parent=None, name='', queue=None, method=None,
+                 cursor=None, sleep=5000, polling=0, loopwait=5):
         """
-        Parent most not be None and must be a TaurusGraphicsScene!
+        Parent must be not None and must be a TaurusGraphicsScene!
+
+        :param queue: pass an external action queue (optional)
+        :param method: action processor (e.g. modelSetter)
+        :param cursor: QCursor during process (optional)
+        :param sleep: delay in ms before thread start
+        :param polling: process actions at fix period (milliseconds)
+        :param loopwait: wait N milliseconds between actions
         """
         Qt.QThread.__init__(self, parent)
         self.name = name
         self.log = Logger('TaurusEmitterThread(%s)' % self.name)
         self.log.setLogLevel(self.log.Info)
-        self.queue = queue or Queue.Queue()
-        self.todo = Queue.Queue()
+        self.queue = queue or Queue()
+        self.todo = Queue()
         self.method = method
         self.cursor = Qt.QCursor(
             Qt.Qt.WaitCursor) if cursor is True else cursor
         self._cursor = False
-        self.timewait = sleep
+        self.timewait = int(sleep)
+        self.polling = int(polling)
+        self.loopwait = int(loopwait)
+        if self.polling:
+            self.refreshTimer = Qt.QTimer()
+            self.refreshTimer.timeout.connect(self.onRefresh)
+        else:
+            self.refreshTimer = None
 
         self.emitter = QEmitter()
         self.emitter.moveToThread(Qt.QApplication.instance().thread())
@@ -184,7 +212,20 @@ class TaurusEmitterThread(Qt.QThread):
         self._done = 0
         # Moved to the end to prevent segfaults ...
         self.emitter.doSomething.connect(self._doSomething)
-        self.emitter.somethingDone.connect(self.next)
+
+        if not self.refreshTimer:
+            self.emitter.somethingDone.connect(self.next)
+
+    def onRefresh(self):
+        try:
+            size = self.getQueue().qsize()
+            if size:
+                self.log.info('onRefresh(%s)' % size)
+                self.next()
+            else:
+                self.log.debug('onRefresh()')
+        except:
+            self.log.warning(traceback.format_exc())
 
     def getQueue(self):
         if self.queue:
@@ -196,7 +237,8 @@ class TaurusEmitterThread(Qt.QThread):
 
     def getDone(self):
         """ Returns % of done tasks in 0-1 range """
-        return self._done / (self._done + self.getQueue().qsize()) if self._done else 0.
+        pending = self.getQueue().qsize()
+        return float(self._done) / (self._done + pending)
 
     def clear(self):
         while not self.todo.empty():
@@ -205,8 +247,11 @@ class TaurusEmitterThread(Qt.QThread):
             self.getQueue().get()
             self._done += 1
 
-    def purge(obj):
-        nqueue = Queue.Queue()
+    def purge(self, obj):
+        """
+        Remove a given object from all queues
+        """
+        nqueue = Queue()
         while not self.todo.empty():
             i = self.todo.get()
             if obj not in i:
@@ -229,20 +274,25 @@ class TaurusEmitterThread(Qt.QThread):
             try:
                 method(*args)
             except:
-                self.log.error('At TaurusEmitterThread._doSomething(%s): \n%s' % (
-                    map(str, args), traceback.format_exc()))
+                self.log.error('At TaurusEmitterThread._doSomething(%s): \n%s'
+                               % (list(map(str, args)), traceback.format_exc()))
         self.emitter.somethingDone.emit()
         self._done += 1
         return
 
     def next(self):
         queue = self.getQueue()
-        msg = 'At TaurusEmitterThread.next(), %d items remaining.' % queue.qsize()
-        (queue.empty() and self.log.info or self.log.debug)(msg)
+        msg = ('At TaurusEmitterThread.next(), %d items remaining.'
+               % queue.qsize())
+        if (queue.empty() and not self.polling):
+            self.log.info(msg)
+        else:
+            self.log.debug(msg)
         try:
             if not queue.empty():
                 if not self._cursor and self.cursor is not None:
-                    Qt.QApplication.instance().setOverrideCursor(Qt.QCursor(self.cursor))
+                    Qt.QApplication.instance().setOverrideCursor(
+                        Qt.QCursor(self.cursor))
                     self._cursor = True
                 # A blocking get here would hang the GUIs!!!
                 item = queue.get(False)
@@ -252,7 +302,7 @@ class TaurusEmitterThread(Qt.QThread):
                 Qt.QApplication.instance().restoreOverrideCursor()
                 self._cursor = False
 
-        except Queue.Empty:
+        except Empty:
             self.log.warning(traceback.format_exc())
             pass
         except:
@@ -260,11 +310,14 @@ class TaurusEmitterThread(Qt.QThread):
         return
 
     def run(self):
-        Qt.QApplication.instance().thread().sleep(int(self.timewait / 1000)
-                                                  if self.timewait > 10 else int(self.timewait))  # wait(self.sleep)
+        Qt.QApplication.instance().thread().msleep(self.timewait)
         self.log.info('#' * 80)
         self.log.info('At TaurusEmitterThread.run()')
         self.next()
+
+        if self.refreshTimer:
+            self.refreshTimer.start(self.polling)
+
         while True:
             self.log.debug('At TaurusEmitterThread.run() loop.')
             item = self.todo.get(True)
@@ -275,33 +328,115 @@ class TaurusEmitterThread(Qt.QThread):
                     continue
             self.log.debug('Emitting doSomething signal ...')
             self.emitter.doSomething.emit(item)
-            # End of while
+            if self.loopwait:
+                self.msleep(self.loopwait)
+                # End of while
         self.log.info(
             '#' * 80 + '\nOut of TaurusEmitterThread.run()' + '\n' + '#' * 80)
         # End of Thread
 
 
-class SingletonWorker():  # Qt.QObject):
+class DelayedSubscriber(Logger):
     """
-    The SingletonWorker works
-    =========================
+    DelayedSubscriber(schema) will use a TaurusEmitterThread to perform
+    a thread safe delayed subscribing on all Attributes of a given 
+    Taurus Schema that has not been previously subscribed.
 
-    The SingletonWorker class is constructed using the same arguments than the TaurusTreadEmitter class ; but instead of creating a QThread for each instance of the class it creates a single QThread for all instances.
+    .. warning:: This class belongs to a "Delayed Event Subscription" API added
+                 in v.4.2.1-alpha as an *experimental* feature. This API may
+                 not be stable and/or it may be removed in a future release
+                 (even on a minor version change)
+    """
 
-    The Queue is still different for each of the instances; it is connected to the TaurusEmitterThread signals (*next()* and *somethingDone()*) and each Worker queue is used as a feed for the shared QThread.
+    def __init__(self, schema, parent=None, sleep=10000, pause=5, period=0):
+
+        self._schema = schema
+        self.call__init__(Logger, 'DelayedSubscriber(%s)' % self._schema, None)
+        self._factory = taurus.Factory(schema)
+
+        self._modelsQueue = Queue()
+        self._modelsThread = TaurusEmitterThread(parent=parent,
+                                                 queue=self._modelsQueue,
+                                                 method=self._modelSubscriber,
+                                                 sleep=sleep, loopwait=pause,
+                                                 polling=period)
+
+        self._modelsQueue.put((self.addUnsubscribedAttributes,))
+        self._modelsThread.start()
+
+    def _modelSubscriber(self, method, args=[]):
+        self.debug('modelSubscriber(%s,%s)' % (method, args))
+        return method(*args)
+
+    def getUnsubscribedAttributes(self):
+        """Check all pending subscriptions in the current factory
+        """
+        attrs = []
+        for name, attr in self._factory.getExistingAttributes().items():
+            if attr is None:
+                continue
+            elif attr.hasListeners() and not attr.isUsingEvents():
+                attrs.append(attr)
+
+        return attrs
+
+    def addUnsubscribedAttributes(self):
+        """Schedule subscription for all pending attributes
+        """
+        try:
+            items = self.getUnsubscribedAttributes()
+            if len(items):
+                self.info('addUnsubscribedAttributes([%d])' % len(items))
+                for attr in items:
+                    self._addModelObj(attr)
+                self._modelsThread.next()
+                self.info('Thread queue: [%d]' % (self._modelsQueue.qsize()))
+        except:
+            self.warning(traceback.format_exc())
+
+    def _addModelObj(self, modelObj):
+        parent = modelObj.getParentObj()
+        if parent:
+            proxy = parent.getDeviceProxy()
+            if not proxy:
+                self.debug('addModelObj(%s), proxy not available' % modelObj)
+                return
+
+        self._modelsQueue.put((modelObj.subscribePendingEvents,))
+        self.debug('addModelObj(%s)' % str(modelObj))
+
+    def cleanUp(self):
+        self.trace("[DelayedSubscriber] cleanUp")
+        self._modelsThread.stop()
+        Logger.cleanUp(self)
+
+
+class SingletonWorker(object):
+    """
+    SingletonWorker is used to manage TaurusEmitterThread as Singleton objects
+
+    SingletonWorker is constructed using the same arguments 
+    than TaurusTreadEmitter ; but instead of creating a QThread for each 
+    instance it creates a single QThread for all instances.
+
+    The Queue is still different for each of the instances; it is connected 
+    to the TaurusEmitterThread signals (*next()* and *somethingDone()*) 
+    and each Worker queue is used as a feed for the shared QThread.
 
     This implementation reduced the cpu of vacca application in a 50% factor.
 
     :param parent: a Qt/Taurus object
     :param name: identifies object logs
-    :param queue: if None parent.getQueue() is used, if not then the queue passed as argument is used
+    :param queue: if None parent.getQueue() is used, if not then the queue 
+        passed as argument is used
     :param method: the method to be executed using each queue item as argument
-    :param cursor: if True or QCursor a custom cursor is set while the Queue is not empty
-    This class is used to manage TaurusEmitterThread as Singleton objects:
+    :param cursor: if True or QCursor a custom cursor is set while 
+        the Queue is not empty
     """
     _thread = None
 
-    def __init__(self, parent=None, name='', queue=None, method=None, cursor=None, sleep=5000, log=Logger.Warning, start=True):
+    def __init__(self, parent=None, name='', queue=None, method=None,
+                 cursor=None, sleep=5000, log=Logger.Warning, start=True):
         self.name = name
         self.log = Logger('SingletonWorker(%s)' % self.name)
         self.log.setLogLevel(log)
@@ -313,7 +448,7 @@ class SingletonWorker():  # Qt.QObject):
             SingletonWorker._thread = TaurusEmitterThread(
                 parent, name='SingletonWorker', cursor=cursor, sleep=sleep)
         self.thread = SingletonWorker._thread
-        self.queue = queue or Queue.Queue()
+        self.queue = queue or Queue()
         if start:
             self.start()
 
@@ -328,9 +463,11 @@ class SingletonWorker():  # Qt.QObject):
             self.put(item)
         elif self.queue.empty():
             return
-        msg = 'At SingletonWorker.next(), %d items not passed yet to Emitter.' % self.queue.qsize()
+        msg = ('At SingletonWorker.next(), '
+               '%d items not passed yet to Emitter.'
+               % self.queue.qsize())
         self.log.info(msg)
-        #(queue.empty() and self.log.info or self.log.debug)(msg)
+        # (queue.empty() and self.log.info or self.log.debug)(msg)
         try:
             i = 0
             while not self.queue.empty():
@@ -343,7 +480,7 @@ class SingletonWorker():  # Qt.QObject):
                 i += 1
             self.log.info('%d Items added to emitter queue' % i)
             self.thread.emitter.newQueue.emit()
-        except Queue.Empty:
+        except Empty:
             self.log.warning(traceback.format_exc())
         except:
             self.log.warning(traceback.format_exc())
@@ -375,20 +512,25 @@ class SingletonWorker():  # Qt.QObject):
     def clear(self):
         """
         This method will clear queue only if next() has not been called.
-        If you call self.thread.clear() it will clear objects for all workers!, be careful
+        If you call self.thread.clear() it will clear objects for all workers!,
+        be careful
         """
         while not self.queue.empty():
             self.queue.get()
-        # self.thread.clear()
+            # self.thread.clear()
 
-    def purge(obj):
-        nqueue = Queue.Queue()
+    def purge(self, obj):
+        """
+        Remove a given object from all queues
+        """
+        nqueue = Queue()
         while not self.queue.empty():
             i = self.queue.get()
             if obj not in i:
                 nqueue.put(i)
         while not nqueue.empty():
             self.queue.put(nqueue.get())
+        self.next()
 
     def isRunning(self):
         return self._running
