@@ -27,16 +27,14 @@
 """This module provides the set of base classes from which the Qt taurus widgets
 should inherit to be considered valid taurus widgets."""
 
-__all__ = ["TaurusBaseComponent", "TaurusBaseWidget",
-           "TaurusBaseWritableWidget"]
-
-__docformat__ = 'restructuredtext'
-
 import sys
 import threading
+from types import MethodType
+from future.builtins import str
+from future.utils import string_types
 
 from taurus.external.qt import Qt
-from taurus.external.enum import Enum
+from enum import Enum
 
 import taurus
 from taurus.core.util import eventfilters
@@ -44,20 +42,46 @@ from taurus.core.util.timer import Timer
 from taurus.core.taurusbasetypes import TaurusElementType, TaurusEventType
 from taurus.core.taurusattribute import TaurusAttribute
 from taurus.core.taurusdevice import TaurusDevice
-from taurus.core.taurusconfiguration import (TaurusConfiguration,
-                                             TaurusConfigurationProxy)
+from taurus.core.taurusconfiguration import TaurusConfigurationProxy
 from taurus.core.tauruslistener import TaurusListener, TaurusExceptionListener
 from taurus.core.taurusoperation import WriteAttrOperation
 from taurus.core.util.eventfilters import filterEvent
 from taurus.core.util.log import deprecation_decorator
-from taurus.qt.qtcore.util.signal import baseSignal
+from taurus.qt.qtcore.util import baseSignal
 from taurus.qt.qtcore.configuration import BaseConfigurableClass
-from taurus.qt.qtcore.mimetypes import TAURUS_ATTR_MIME_TYPE, TAURUS_DEV_MIME_TYPE, TAURUS_MODEL_MIME_TYPE
+from taurus.qt.qtcore.mimetypes import TAURUS_ATTR_MIME_TYPE
+from taurus.qt.qtcore.mimetypes import TAURUS_DEV_MIME_TYPE
+from taurus.qt.qtcore.mimetypes import TAURUS_MODEL_MIME_TYPE
 from taurus.qt.qtgui.util import ActionFactory
 
+from taurus.core.units import Quantity
+
+
+__all__ = ["TaurusBaseComponent", "TaurusBaseWidget",
+           "TaurusBaseWritableWidget", "defaultFormatter"]
+
+__docformat__ = 'restructuredtext'
+
+
 DefaultNoneValue = "-----"
 
 
+def defaultFormatter(dtype=None, basecomponent=None, **kwargs):
+    """
+    Default formatter callable. Returns a format string based on dtype
+    and the mapping provided by :attr:`TaurusBaseComponent.defaultFormatDict`
+
+    :param dtype: (object) data type
+    :param basecomponent: widget whose display is to be formatted
+    :param kwargs: other keyworld arguments
+
+    :return: (str) The format string corresponding to the given dtype.
+    """
+    if issubclass(dtype, Enum):
+        dtype = Enum
+    return basecomponent.defaultFormatDict.get(dtype, "{0}")
+
+
 class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
     """A generic Taurus component.
 
@@ -74,9 +98,19 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
     _showQuality = True
     _eventBufferPeriod = 0
 
+    # Python format string or Formatter callable
+    # (None means that the default formatter will be used)
+    FORMAT = None
+
+    # Dictionary mapping dtypes to format strings
+    defaultFormatDict = {float: "{:.{bc.modelObj.precision}f}",
+                         Enum: "{0.name}",
+                         Quantity: "{:~.{bc.modelObj.precision}f}"
+                         }
+
     taurusEvent = baseSignal('taurusEvent', object, object, object)
 
-    def __init__(self, name, parent=None, designMode=False):
+    def __init__(self, name='', parent=None, designMode=False, **kwargs):
         """Initialization of TaurusBaseComponent"""
         self.modelObj = None
         self.modelName = ''
@@ -87,10 +121,15 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
 
         BaseConfigurableClass.__init__(self)
 
-        self.taurusMenu = None
-        self.taurusMenuData = ''
+        # --------------------------------------------------------------
+        # Deprecated API for context menu
+        self.taurusMenu = None  # deprecated since 4.5.3a. Do not use
+        self.taurusMenuData = ''  # deprecated since 4.5.3a. Do not use
+        self.__explicitPopupMenu = False
+        # --------------------------------------------------------------
 
         # attributes storing property values
+        self._format = None
         self._localModelName = ''
         self._useParentModel = False
         self._showText = True
@@ -114,13 +153,28 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
         else:
             self._exception_listener = set([TaurusExceptionListener()])
 
+        # Use default formatter if none has been set by the class
+        if self.FORMAT is None:
+            self.setFormat(getattr(taurus.tauruscustomsettings,
+                                   'DEFAULT_FORMATTER',
+                                   defaultFormatter))
+
         # register configurable properties
-        self.registerConfigProperty(
-            self.isModifiableByUser, self.setModifiableByUser, "modifiableByUser")
+        self.registerConfigProperty(self.isModifiableByUser,
+                                    self.setModifiableByUser,
+                                    "modifiableByUser")
         self.registerConfigProperty(
             self.getModelInConfig, self.setModelInConfig, "ModelInConfig")
+        self.registerConfigProperty(self.getFormat, self.setFormat,
+                                    'formatter')
         self.resetModelInConfig()
 
+        # connect taurusEvent signal to filterEvent
+        try:
+            self.taurusEvent.connect(self.filterEvent)
+        except Exception as e:
+            self.warning('Could not connect taurusEvent signal: %r', e)
+
     @deprecation_decorator(rel='4.0')
     def getSignaller(self):
         return self
@@ -169,18 +223,39 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
         return taurus.Factory(scheme)
 
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
-    # Popup menu behavior
+    # Context menu behavior
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 
     def contextMenuEvent(self, event):
-        """Handle the popup menu event
+        """
+        DEPRECATED:
+        Until v4.5.3a, the default implementation of contextMenuEvent showed
+        the content of taurusMenu as a context menu. But this resulted in
+        unwanted behaviour when the widget already implemented its own context
+        menu (see https://github.com/taurus-org/taurus/issues/905 )
 
-        :param event: the popup menu event
+        Therefore this feature was disabled in 4.5.3a.
+
+        If you still want to show the contents of taurusMenu as a context menu,
+        you can explicitly reimplement the contextMenuEvent method as::
+
+            def contextMenuEvent(self, event):
+                self.taurusMenu.exec_(event.globalPos())
         """
         if self.taurusMenu is not None:
-            self.taurusMenu.exec_(event.globalPos())
-        else:
-            event.ignore()
+            if self.__explicitPopupMenu:
+                # bck-compat: show taurusMenu as a contextMenu if it was
+                # explicitly created via the (deprecated) "setTaurusPopupMenu"
+                # API
+                self.taurusMenu.exec_(event.globalPos())
+                return
+            else:
+                self.deprecated(
+                    dep='taurusMenu context Menu API',
+                    alt='custom contextMenuEvent to show taurusMenu',
+                    rel='4.5.3a'
+                 )
+        event.ignore()
 
     #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
     # Mandatory methods to be implemented in subclass implementation
@@ -600,6 +675,16 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
             fragmentName = self.modelFragmentName
         return self.modelObj.getFragmentObj(fragmentName)
 
+    def getModelIndexValue(self):
+        """
+        Called inside getDisplayValue to use with spectrum attributes.
+        By default not used, but some widget might want to support this
+        feature.
+
+        Override when needed.
+        """
+        return None
+
     def getFormatedToolTip(self, cache=True):
         """Returns a string with contents to be displayed in a tooltip.
 
@@ -631,15 +716,136 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
         return ret
 
     def displayValue(self, v):
-        """Returns a string representation of the given value
+        """
+        Returns a string representation of the given value
+
+        This method will use a format string which is determined 
+        dynamically from :attr:`FORMAT`.
+
+        By default `TaurusBaseComponent.FORMAT` is set to 
+        :func:`defaultFormatter`, which makes use of
+        :attr:`defaultFormatDict`.
+ 
+        In order to customize the formatting behaviour, one can
+        use :meth:`setFormat` to alter the formatter of an specific instance
+        (recommended) or change :attr:`defaultFormatDict` or
+        :attr:`FORMAT` directly at class level.
+        
+        The formatter can be set to a python format string [1] or a callable
+        that returns a python format string.
+        If a callable is used, it will be called with the following 
+        keyword arguments:
+        - dtype: the data type of the value to be formatted
+        - basecomponent: the affected widget
+  
+        The following are some examples for customizing the formatting::
+
+        - Change the format for widget instance `foo`::
+
+            foo.setFormat("{:.2e}")
+
+        - Change FORMAT for all widgets (using a string)::
+
+            TaurusBaseComponent.FORMAT = "{:.2e}"
+            
+        - Change FORMAT for all TaurusLabels (using a callable)::
+        
+            def baseFormatter(dtype=None, basecomponent=None, **kwargs):
+                return "{:.1f}"
+
+            TaurusLabel.FORMAT = baseFormatter
+
+        - Use the defaultFormatDict but modify the format string for
+          dtype=str::
+
+            TaurusLabel.defaultFormatDict.update({"str": "{!r}"})
+
+
+        .. seealso:: :attr:`tauruscustomsettings.DEFAULT_FORMATTER`,
+                     `--default-formatter` option in :class:`TaurusApplication`,
+                     :meth:`TaurusBaseWidget.onSetFormatter`
+
+
+        [1] https://docs.python.org/2/library/string.html
 
         :param v: (object) the value to be translated to string
 
         :return: (str) a string representing the given value
         """
-        if isinstance(v, Enum):
-            return v.name
-        return str(v)
+        if self._format is None:
+            try:
+                self._updateFormat(type(v))
+            except Exception as e:
+                self.warning(('Cannot update format. Reverting to default.' +
+                              ' Reason: %r'), e)
+                self.setFormat(defaultFormatter)
+        try:
+            fmt_v = self._format.format(v, bc=self)
+        except Exception:
+            self.debug("Invalid format %r for %r. Using '{0}'", self._format, v)
+            fmt_v = "{0}".format(v)
+
+        return fmt_v
+
+    def _updateFormat(self, dtype, **kwargs):
+        """ Method to update the internal format string used by 
+        :meth:`displayValue`
+        The internal format string is calculated using :attribute:`FORMAT`,
+        which can be a string or a callable that returns a string
+        (see :meth:`displayValue`).
+
+        :param dtype: (object) data type
+        :param kwargs: keyword arguments that will be passed to
+                       :attribute:`FORMAT` if it is a callable
+        """
+        if not isinstance(self.FORMAT, string_types):
+            # unbound method to callable
+            if isinstance(self.FORMAT, MethodType):
+                self.FORMAT = self.FORMAT.__func__
+            self._format = self.FORMAT(dtype=dtype, basecomponent=self,  
+                                       **kwargs)
+        else:
+            self._format = self.FORMAT
+
+    def setFormat(self, format):
+        """ Method to set the `FORMAT` attribute for this instance.
+        It also resets the internal format string, which will be recalculated
+        in the next call to :meth:`displayValue`
+
+        :param format: (str or callable) A format string
+                       or a formatter callable (or the callable name in
+                       "full.module.callable" format)
+        """
+        # Check if the format is a callable string representation
+        if isinstance(format, string_types):
+            try:
+                moduleName, formatterName = format.rsplit('.', 1)
+                __import__(moduleName)
+                module = sys.modules[moduleName]
+                format = getattr(module, formatterName)
+            except:
+                format = str(format)
+        self.FORMAT = format
+        self.resetFormat()
+
+    def getFormat(self):
+        """ Method to get the `FORMAT` attribute for this instance.
+
+        :return: (str) a string of the current format. It could be a python
+                 format string or a callable string representation.
+        """
+        if isinstance(self.FORMAT, string_types):
+            formatter = self.FORMAT
+        else:
+            formatter = '{0}.{1}'.format(self.FORMAT.__module__,
+                                         self.FORMAT.__name__)
+        return formatter
+
+    def resetFormat(self):
+        """Reset the internal format string. It forces a recalculation
+        in the next call to :meth:`displayValue`.
+        """
+        self._format = None
 
     def getDisplayValue(self, cache=True, fragmentName=None):
         """Returns a string representation of the model value associated with
@@ -660,6 +866,15 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
             v = self.getModelFragmentObj(fragmentName=fragmentName)
         except:
             return self.getNoneValue()
+
+        idx = self.getModelIndexValue()
+        if v is not None and idx:
+            try:
+                for i in idx:
+                    v = v[i]
+            except Exception as e:
+                self.debug('Problem with applying model index: %r', e)
+                return self.getNoneValue()
         return self.displayValue(v)
 
     def setNoneValue(self, v):
@@ -711,15 +926,11 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
 
     def preAttach(self):
         """Called inside self.attach() before actual attach is performed.
-        Default implementation just emits a signal.
+        Default implementation does nothing.
 
         Override when necessary.
         """
-        try:
-            self.taurusEvent.connect(self.filterEvent)
-        except:
-            # self.error("In %s.preAttach() ... failed!" % str(type(self)))
-            pass
+        pass
 
     def postAttach(self):
         """Called inside self.attach() after actual attach is performed.
@@ -731,15 +942,11 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
 
     def preDetach(self):
         """Called inside self.detach() before actual deattach is performed.
-        Default implementation just disconnects a signal.
+        Default implementation does nothing.
 
         Override when necessary.
         """
-        try:
-            self.taurusEvent.disconnect(self.filterEvent)
-        except:
-            # self.error("In %s.preDetach() ... failed!" % str(type(self)))
-            pass
+        pass
 
     def postDetach(self):
         """Called inside self.detach() after actual deattach is performed.
@@ -942,6 +1149,7 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
 
         :param model: (str) the new model name"""
         self.setModelCheck(model)
+        self.resetFormat()
         self.updateStyle()
 
     def setModelCheck(self, model, check=True):
@@ -993,6 +1201,9 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
 
         :param yesno: (bool) whether or not to use parent model
         """
+        if yesno:
+            self.deprecated(dep='setUseParentModel(True)', rel="4.3.2",
+                            alt='explicit models including the parent model')
         if yesno == self._useParentModel:
             return
         self._useParentModel = yesno
@@ -1050,14 +1261,18 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
         """Resets the showing of the display value to True"""
         self.setShowText(True)
 
+    @deprecation_decorator(rel='4.5.3a')
     def setTaurusPopupMenu(self, menuData):
         """Sets/unsets the taurus popup menu
 
         :param menuData: (str) an xml representing the popup menu"""
         self.taurusMenuData = str(menuData)
+        self.__explicitPopupMenu = True
         factory = ActionFactory()
         self.taurusMenu = factory.getNewMenu(self, self.taurusMenuData)
 
+
+    @deprecation_decorator(rel='4.5.3a')
     def getTaurusPopupMenu(self):
         """Returns an xml string representing the current taurus popup menu
 
@@ -1065,9 +1280,11 @@ class TaurusBaseComponent(TaurusListener, BaseConfigurableClass):
         """
         return self.taurusMenuData
 
+    @deprecation_decorator(rel='4.5.3a')
     def resetTaurusPopupMenu(self):
         """Resets the taurus popup menu to empty"""
         self.taurusMenuData = ''
+        self.__explicitPopupMenu = False
 
     def isModifiableByUser(self):
         '''whether the user can change the contents of the widget
@@ -1118,7 +1335,7 @@ class TaurusBaseWidget(TaurusBaseComponent):
 
     _dragEnabled = False
 
-    def __init__(self, name, parent=None, designMode=False):
+    def __init__(self, name='', parent=None, designMode=False, **kwargs):
         self._disconnect_on_hide = False
         self._supportedMimeTypes = None
         self._autoTooltip = True
@@ -1126,6 +1343,44 @@ class TaurusBaseWidget(TaurusBaseComponent):
                           parent=parent, designMode=designMode)
         self._setText = self._findSetTextMethod()
 
+    def showFormatterDlg(self):
+        """
+        showFormatterDlg show a dialog to get the formatter from the user.
+        :return: formatter: python fromat string or formatter callable
+        (in string version) or None
+        """
+        current_format = self.getFormat()
+
+        formatter, ok = Qt.QInputDialog.getText(self, "Set formatter",
+                                                "Enter a formatter:",
+                                                Qt.QLineEdit.Normal,
+                                                current_format)
+        if ok and formatter:
+            return formatter
+
+        return None
+
+    def onSetFormatter(self):
+        """Slot to allow interactive setting of the Formatter.
+
+        .. seealso:: :meth:`TaurusBaseWidget.showFormatterDlg`,
+                     :meth:`TaurusBaseComponent.displayValue`,
+                     :attr:`tauruscustomsettings.DEFAULT_FORMATTER`
+        """
+        format = self.showFormatterDlg()
+        if format is not None:
+            self.debug(
+                'Default format has been changed to: {0}'.format(format))
+            # -----------------------------------------------------------------
+            # TODO: Tango-centric (replace by agnostic entry point solution)
+            # shortcut to setup the tango formatter
+            if format.strip() == "tangoFormatter":
+                from taurus.core.tango.util.formatter import tangoFormatter
+                format = tangoFormatter
+            # -----------------------------------------------------------------
+            self.setFormat(format)
+        return format
+
     # It makes the GUI to hang... If this needs implementing, we should
     # reimplement it using the Qt parent class, not QWidget...
     # def destroy(self):
@@ -1501,7 +1756,7 @@ class TaurusBaseWidget(TaurusBaseComponent):
         formats = mimeData.formats()
         for mtype in supported:
             if mtype in formats:
-                d = str(mimeData.data(mtype))
+                d = bytes(mimeData.data(mtype)).decode('utf-8')
                 if d is None:
                     return None
                 try:
@@ -1522,15 +1777,15 @@ class TaurusBaseWidget(TaurusBaseComponent):
         :return: (QMimeData)
         '''
         mimeData = Qt.QMimeData()
-        modelname = self.getModelName()
+        modelname = str(self.getModelName()).encode(encoding='utf8')
         mimeData.setData(TAURUS_MODEL_MIME_TYPE, modelname)
         try:
             modelclass = self.getModelClass()
         except:
             modelclass = None
-        if issubclass(modelclass, TaurusDevice):
+        if modelclass and issubclass(modelclass, TaurusDevice):
             mimeData.setData(TAURUS_DEV_MIME_TYPE, modelname)
-        elif issubclass(modelclass, TaurusAttribute):
+        elif modelclass and issubclass(modelclass, TaurusAttribute):
             mimeData.setData(TAURUS_ATTR_MIME_TYPE, modelname)
         return mimeData
 
@@ -1684,7 +1939,8 @@ class TaurusBaseWritableWidget(TaurusBaseWidget):
 
     applied = baseSignal('applied')
 
-    def __init__(self, name, taurus_parent=None, designMode=False):
+    def __init__(self, name='', taurus_parent=None, designMode=False,
+                 **kwargs):
         self.call__init__(TaurusBaseWidget, name,
                           parent=taurus_parent, designMode=designMode)
 
@@ -1877,6 +2133,21 @@ class TaurusBaseWritableWidget(TaurusBaseWidget):
         '''
         return []
 
+    def getDisplayValue(self, cache=True, fragmentName=None):
+        """Reimplemented from class:`TaurusBaseWidget`"""
+        # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+        # The widgets inheriting from this class interact with
+        # writable models and therefore the fragmentName should fall back to 'wvalue' 
+        # instead of 'rvalue'. 
+        # But changing it now is delicate due to risk of introducing API
+        # incompatibilities for widgets already assuming the current default.
+        # So instead of reimplementing it here, the fix was constrained to 
+        # TaurusValueLineEdit.getDisplayValue()
+        # TODO: Consider reimplementing this to use wvalue by default
+        return TaurusBaseWidget.getDisplayValue(self, cache=cache,
+                                                fragmentName=fragmentName)
+        # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
     def getValue(self):
         '''
         This method must be implemented in derived classes to return
@@ -1911,34 +2182,10 @@ class TaurusBaseWritableWidget(TaurusBaseWidget):
                     v_str, model_v_str)
             self.setToolTip(toolTip)
 
-    def _updateValidator(self, evt_value):  # TODO: Remove this method
-        # re-set the validator ranges if applicable
-        if evt_value is None:
-            return
-        v = self.validator()
-        if isinstance(v, Qt.QIntValidator):
-            bottom = evt_value.min_value
-            top = evt_value.max_value
-            bottom = int(
-                bottom) if bottom != TaurusConfiguration.no_min_value else -sys.maxint
-            top = int(
-                top) if top != TaurusConfiguration.no_max_value else sys.maxint
-            v.setRange(bottom, top)
-            self.debug("Validator range set to %i-%i" % (bottom, top))
-        elif isinstance(v, Qt.QDoubleValidator):
-            bottom = evt_value.min_value
-            top = evt_value.max_value
-            bottom = float(
-                bottom) if bottom != TaurusConfiguration.no_min_value else -float("inf")
-            top = float(
-                top) if top != TaurusConfiguration.no_max_value else float("inf")
-            v.setBottom(bottom)
-            v.setTop(top)
-            self.debug("Validator range set to %f-%f" % (bottom, top))
-
     @classmethod
     def getQtDesignerPluginInfo(cls):
         '''reimplemented from :class:`TaurusBaseWidget`'''
         ret = TaurusBaseWidget.getQtDesignerPluginInfo()
         ret['group'] = 'Taurus Input'
         return ret
+
