@@ -23,14 +23,22 @@
 ##
 #############################################################################
 
-"""This package contains a collection of taurus console widgets"""
+"""DEPRECATED since 4.0,4. Use `silx.gui.console`"""
 
 __docformat__ = 'restructuredtext'
 
+
+from taurus.core.util.log import deprecated
+
+deprecated(dep='TaurusConsole', alt='silx.gui.console.IPythonWidget',
+           rel='4.0.4')
+
 try:
-    from .taurusconsole import TaurusConsole
-except Exception, e:
+    from silx.gui.console import IPythonWidget as TaurusConsole
+except Exception as e:
     from taurus.qt.qtgui.display import TaurusFallBackWidget
 
     class TaurusConsole(TaurusFallBackWidget):
         pass
+
+
