@@ -24,13 +24,15 @@
 #############################################################################
 
 """This module provides a set of dialog based widgets"""
+from __future__ import print_function
+
+from taurus.external.qt import Qt
+
 
 __all__ = ["TaurusInputDialog", "get_input"]
 
 __docformat__ = 'restructuredtext'
 
-from taurus.external.qt import Qt
-
 
 def get_input(input_data, parent=None, input_panel_klass=None):
     """Static convenience function to get an input from the user using a
@@ -199,7 +201,7 @@ def main():
               data_type='Text', key="Memo", default_value="By default a memo is\na long thing")
 
     for d in [d1, d2, d3, d4, d5, d6, d7, d8]:
-        print get_input(input_data=d, title=d['prompt'])
+        print(get_input(input_data=d, title=d['prompt']))
 
 if __name__ == "__main__":
     main()
