@@ -73,6 +73,7 @@ def demo():
             control_l.setSpacing(M)
             control_panel.setLayout(control_l)
             model_widget = Qt.QLineEdit()
+            format_widget = Qt.QLineEdit()
             model_index_widget = Qt.QLineEdit()
             fg_widget = Qt.QComboBox()
             bg_widget = Qt.QComboBox()
@@ -82,6 +83,7 @@ def demo():
             control_l.addRow("model index:", model_index_widget)
             control_l.addRow("foreground role:", fg_widget)
             control_l.addRow("background role:", bg_widget)
+            control_l.addRow("formatter:", format_widget)
             control_l.addRow("prefix text:", prefix_widget)
             control_l.addRow("suffix text:", suffix_widget)
 
@@ -91,19 +93,22 @@ def demo():
             fg_widget.addItems(["", "rvalue", "rvalue.magnitude",
                                 "rvalue.units", "wvalue", "wvalue.magnitude",
                                 "wvalue.units", "state", "quality", "none"])
-            bg_widget.addItems(["quality", "state", "none"])
+            bg_widget.addItems(["quality", "state", "value", "none"])
 
             model_widget.textChanged.connect(w.setModel)
-            fg_widget.currentIndexChanged[str].connect(w.setFgRole)
-            bg_widget.currentIndexChanged[str].connect(w.setBgRole)
+            format_widget.textChanged.connect(w.setFormat)
+            fg_widget.currentIndexChanged['QString'].connect(w.setFgRole)
+            bg_widget.currentIndexChanged['QString'].connect(w.setBgRole)
             prefix_widget.textChanged.connect(w.setPrefixText)
             suffix_widget.textChanged.connect(w.setSuffixText)
             model_index_widget.textChanged.connect(w.setModelIndex)
 
             model_widget.setText("sys/tg_test/1/double_scalar")
+            format_widget.setText(w.getFormat())
             fg_widget.setCurrentIndex(0)
             fg_widget.setEditable(True)
             bg_widget.setCurrentIndex(0)
+            bg_widget.setEditable(True)
 
             self.w_label = w
             self.w_model = model_widget
@@ -123,7 +128,7 @@ def demo():
     p1.w_model.setText("sys/tg_test/1/double_scalar")
     p2 = TaurusLabelTestPanel()
     p2.w_model.setText("sys/tg_test/1/double_scalar#label")
-    p2.w_bg.setCurrentIndex(2)
+    p2.w_bg.setCurrentIndex(3)  # bgRole='none'
     layout.addWidget(p1, 0, 0)
     layout.addWidget(p2, 0, 1)
     layout.addItem(Qt.QSpacerItem(10, 10), 1, 0, 1, 2)
