@@ -24,18 +24,16 @@
 #############################################################################
 
 """This module provides a set of basic Taurus widgets based on QLabel"""
+from __future__ import absolute_import
 
-__all__ = ["TaurusLabel"]
+from builtins import str
+from builtins import object
 
-__docformat__ = 'restructuredtext'
-
-import operator
+import collections
 import re
 
-# shame of me for importing PyTango!
-import PyTango
-
-from taurus.core.taurusbasetypes import TaurusElementType, TaurusEventType
+from taurus.core.taurusbasetypes import (TaurusElementType, TaurusEventType,
+                                         AttrQuality, TaurusDevState)
 from taurus.external.qt import Qt
 from taurus.qt.qtgui.base import TaurusBaseWidget
 from taurus.qt.qtgui.base import TaurusBaseController
@@ -43,6 +41,11 @@ from taurus.qt.qtgui.base import TaurusScalarAttributeControllerHelper
 from taurus.qt.qtgui.base import TaurusConfigurationControllerHelper
 from taurus.qt.qtgui.base import updateLabelBackground
 
+
+__all__ = ["TaurusLabel"]
+
+__docformat__ = 'restructuredtext'
+
 _QT_PLUGIN_INFO = {
     'module': 'taurus.qt.qtgui.display',
     'group': 'Taurus Display',
@@ -100,7 +103,7 @@ class TaurusLabelController(TaurusBaseController):
         self._trimmedText = self._shouldTrim(label, text)
         if self._trimmedText:
             text = "<a href='...'>...</a>"
-        label.setText(text)
+        label.setText_(text)
 
     def _shouldTrim(self, label, text):
         if not label.autoTrim:
@@ -115,9 +118,7 @@ class TaurusLabelController(TaurusBaseController):
             return
         toolTip = label.getFormatedToolTip()
         if self._trimmedText:
-            toolTip = u"<p><b>Value:</b> %s</p><hr>%s" %\
-                      (unicode(self._text, errors='replace'),
-                       unicode(str(toolTip), errors='replace'))
+            toolTip = u"<p><b>Value:</b> %s</p><hr>%s" % (self._text, toolTip)
         label.setToolTip(toolTip)
 
     _updateBackground = updateLabelBackground
@@ -170,10 +171,10 @@ class TaurusLabelControllerDesignMode(object):
         return 0.0
 
     def quality(self):
-        return PyTango.AttrQuality.ATTR_VALID
+        return AttrQuality.ATTR_VALID
 
     def state(self):
-        return PyTango.DevState.ON
+        return TaurusDevState.Ready
 
     def _updateToolTip(self, lcd):
         lcd.setToolTip("Some random value for design purposes only")
@@ -231,6 +232,7 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
     def __init__(self, parent=None, designMode=False):
         self._prefix = self.DefaultPrefix
         self._suffix = self.DefaultSuffix
+        self._permanentText = None
         self._bgRole = self.DefaultBgRole
         self._fgRole = self.DefaultFgRole
         self._modelIndex = self.DefaultModelIndex
@@ -251,6 +253,11 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
         if self._designMode:
             self.controllerUpdate()
 
+        # register configurable properties
+        self.registerConfigProperty(
+            self.getPermanentText, self._setPermanentText, "permanentText"
+            )
+
     def _calculate_controller_class(self):
         ctrl_map = _CONTROLLER_MAP
         if self._designMode:
@@ -314,6 +321,7 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
     def setModel(self, m):
         # force to build another controller
         self._controller = None
+        self._permanentText = None
         TaurusBaseWidget.setModel(self, m)
         if self.modelFragmentName:
             self.setFgRole(self.modelFragmentName)
@@ -339,7 +347,7 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
                 return
             if type(mi_value) == int:
                 mi_value = mi_value,
-            if not operator.isSequenceType(mi_value):
+            if not isinstance(mi_value, collections.Sequence):
                 return
             self._modelIndex = mi_value
         self._modelIndexStr = mi
@@ -357,16 +365,40 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
         return self._bgRole
 
     def setBgRole(self, bgRole):
-        self._bgRole = str(bgRole).lower()
+        """
+        Set the background role. The label background will be set according
+        to the current palette and the role. Valid roles are:
+        - 'none' : no background
+        - 'state' a color depending on the device state
+        - 'quality' a color depending on the attribute quality
+        - 'rvalue' a color depending on the rvalue of the attribute
+        - <arbitrary member name> a color based on the value of an arbitrary
+        member of the model object (warning: experimental feature!)
+
+        .. warning:: the <arbitrary member name> support is still experimental
+                     and its API may change in future versions
+        """
+        self._bgRole = str(bgRole)
         self.controllerUpdate()
 
     def resetBgRole(self):
+        """Reset the background role to its default value"""
         self.setBgRole(self.DefaultBgRole)
 
     def getFgRole(self):
+        """get the foreground role for this label (see :meth:`setFgRole`)"""
         return self._fgRole
 
     def setFgRole(self, fgRole):
+        """Set what is shown as the foreground (the text) of the label
+        Valid Roles are:
+
+        - 'rvalue' the read value of the attribute
+        - 'wvalue' the write value of the attribute
+        - 'none' : no text
+        - 'quality' - the quality of the attribute is displayed
+        - 'state' - the device state
+        """
         # warn about deprecated roles
         role = self._deprecatedRoles.get(fgRole, fgRole)
         if fgRole != role:
@@ -376,6 +408,7 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
         self.controllerUpdate()
 
     def resetFgRole(self):
+        """Reset the foreground role to its default value"""
         self.setFgRole(self.DefaultFgRole)
 
     def getPrefixText(self):
@@ -398,7 +431,28 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
     def resetSuffixText(self):
         self.setSuffixText(self.DefaultSuffix)
 
+    def getPermanentText(self):
+        return self._permanentText
+
+    def _setPermanentText(self, text):
+        self._permanentText = text
+        if text is not None:
+            self.setText_(text)
+
+    def setText_(self, text):
+        """Method to expose QLabel.setText"""
+        Qt.QLabel.setText(self, text)
+
+    def setText(self, text):
+        """Reimplementation of setText to set permanentText"""
+        self._setPermanentText(text)
+
     def setAutoTrim(self, trim):
+        """Enable/disable auto-trimming of the text. If trim is True, the text
+        in the label will be trimmed when it doesn't fit in the available space
+
+        :param trim: (bool)
+        """
         self._autoTrim = trim
         self.controllerUpdate()
 
@@ -418,11 +472,43 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
         self.dynamicTextInteractionFlags = True
 
     def getAutoTrim(self):
+        """
+        Whether auto-trimming of the text is enabled.
+
+        :return: (bool)
+        """
         return self._autoTrim
 
     def resetAutoTrim(self):
+        """Reset auto-trimming to its default value"""
         self.setAutoTrim(self.DefaultAutoTrim)
 
+    def displayValue(self, v):
+        """Reimplementation of displayValue for TaurusLabel"""
+        if self._permanentText is None:
+            value = TaurusBaseWidget.displayValue(self, v)
+        else:
+            value = self._permanentText
+
+        dev = None
+        attr = None
+
+        modeltype = self.getModelType()
+        if modeltype == TaurusElementType.Device:
+            dev = self.getModelObj()
+        elif modeltype == TaurusElementType.Attribute:
+            attr = self.getModelObj()
+            dev = attr.getParent()
+
+        try:
+            v = value.format(dev=dev, attr=attr)
+        except Exception as e:
+            self.warning(
+                "Error formatting display (%r). Reverting to raw string", e)
+            v = value
+
+        return v
+
     @classmethod
     def getQtDesignerPluginInfo(cls):
         d = TaurusBaseWidget.getQtDesignerPluginInfo()
@@ -433,9 +519,6 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
     #: with which this widget will get its data from. The convention used for
     #: the string can be found :ref:`here <model-concept>`.
     #:
-    #: In case the property :attr:`useParentModel` is set to True, the model
-    #: text must start with a '/' followed by the attribute name.
-    #:
     #: **Access functions:**
     #:
     #:     * :meth:`TaurusBaseWidget.getModel`
@@ -497,8 +580,8 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
     #: Valid values are:
     #:
     #:     #. ''/'None' - no value is displayed
-    #:     #. 'value' - the value is displayed
-    #:     #. 'w_value' - the write value is displayed
+    #:     #. 'rvalue' - the value is displayed
+    #:     #. 'wvalue' - the write value is displayed
     #:     #. 'quality' - the quality is displayed
     #:     #. 'state' - the device state is displayed
     #:
@@ -521,7 +604,8 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
     bgRole = Qt.pyqtProperty("QString", getBgRole, setBgRole,
                              resetBgRole, doc="background role")
 
-    #: This property holds the
+    #: Specifies wether the text will be trimmed when it doesn't fit in the
+    #: available space
     #:
     #: **Access functions:**
     #:
@@ -531,7 +615,7 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
     autoTrim = Qt.pyqtProperty("bool", getAutoTrim, setAutoTrim,
                                resetAutoTrim, doc="auto trim text")
 
-    #: This property holds the
+    #: Specifies whether the user can drag data from this widget
     #:
     #: **Access functions:**
     #:
@@ -567,8 +651,8 @@ class TaurusLabel(Qt.QLabel, TaurusBaseWidget):
 
 def demo():
     "Label"
-    import demo.tauruslabeldemo
-    return demo.tauruslabeldemo.main()
+    from .demo import tauruslabeldemo
+    return tauruslabeldemo.main()
 
 
 def main():
@@ -592,12 +676,10 @@ def main():
     if len(args) == 0:
         w = demo()
     else:
-        models = map(str.lower, args)
-
         w = Qt.QWidget()
         layout = Qt.QGridLayout()
         w.setLayout(layout)
-        for model in models:
+        for model in args:
             label = TaurusLabel()
             label.model = model
             layout.addWidget(label)
