@@ -25,16 +25,15 @@
 
 """This module provides a Taurus widget based on QLCDNumber"""
 
-__all__ = ["TaurusLCD"]
+from __future__ import absolute_import
 
-__docformat__ = 'restructuredtext'
+from builtins import str
+from builtins import object
 
-import operator
+import collections
 
-# shame of me for importing PyTango!
-import PyTango
-
-from taurus.core.taurusbasetypes import TaurusElementType, TaurusEventType
+from taurus.core.taurusbasetypes import (TaurusElementType, TaurusEventType,
+                                         AttrQuality, TaurusDevState)
 from taurus.external.qt import Qt
 from taurus.qt.qtgui.base import TaurusBaseWidget
 from taurus.qt.qtgui.base import TaurusBaseController
@@ -42,6 +41,11 @@ from taurus.qt.qtgui.base import TaurusScalarAttributeControllerHelper
 from taurus.qt.qtgui.base import TaurusConfigurationControllerHelper
 from taurus.qt.qtgui.base import updateLabelBackground
 
+
+__all__ = ["TaurusLCD"]
+
+__docformat__ = 'restructuredtext'
+
 _QT_PLUGIN_INFO = {
     'module': 'taurus.qt.qtgui.display',
     'group': 'Taurus Display',
@@ -71,52 +75,24 @@ class TaurusLCDController(TaurusBaseController):
         return self.widget()
 
     def _updateForeground(self, lcd):
-        self._updateLength(lcd)
-        self._updateValue(lcd)
+        value = None
+        if lcd.fgRole == 'value':
+            value = self.getDisplayValue()
+        elif lcd.fgRole == 'w_value':
+            value = self.getDisplayValue(True)
+        elif lcd.fgRole in ('', 'none'):
+            value = ""
+
+        if value is None:
+            value = "udef"
+
+        lcd.setNumDigits(len(value))
+        lcd.display(value)
 
     def _needsStateConnection(self):
         lcd = self.lcd()
         return 'state' in (lcd.fgRole, lcd.bgRole)
 
-    def _getCharsToDisplayFromFormat(self, fmt):
-        n = ''
-        for c in fmt:
-            if c == '.':
-                break
-            if c.isdigit():
-                n += c
-        return int(n)
-
-    def _updateLength(self, lcd):
-        model = self.modelObj()
-        n = 6
-        try:
-            fmt = model.getFormat()
-            if fmt:
-                n = self._getCharsToDisplayFromFormat(fmt)
-        except:
-            pass
-        lcd.setNumDigits(n)
-
-    def _updateValue(self, lcd):
-        fgRole, value = lcd.fgRole, ""
-        if fgRole == 'value':
-            w = self.getDisplayValue()
-            # @todo: sometimes getDisplayValue called but self.widget() is None.
-            # For the moment it is just protected by substituting the following
-            # line by the ones after it.
-            # value += self.getDisplayValue()
-            if w is not None:
-                value += self.getDisplayValue()
-        elif fgRole == 'w_value':
-            value += self.getDisplayValue(True)
-        elif fgRole in ('', 'none'):
-            pass
-        else:
-            lcd.display("udef")
-            return
-        lcd.display(value)
-
     _updateBackground = updateLabelBackground
 
 
@@ -135,9 +111,8 @@ class TaurusLCDControllerAttribute(TaurusScalarAttributeControllerHelper, Taurus
             if idx is not None and len(idx):
                 for i in idx:
                     value = value[i]
-            if self._last_config_value is None or value is None:
-                return widget.displayValue(value)
-        except Exception, e:
+            return widget.displayValue(value)
+        except Exception as e:
             return widget.getNoneValue()
 
 
@@ -170,10 +145,10 @@ class TaurusLCDControllerDesignMode(object):
         return 0.0
 
     def quality(self):
-        return PyTango.AttrQuality.ATTR_VALID
+        return AttrQuality.ATTR_VALID
 
     def state(self):
-        return PyTango.DevState.ON
+        return TaurusDevState.ON
 
     def _updateToolTip(self, lcd):
         lcd.setToolTip("Some random value for design purposes only")
@@ -309,7 +284,7 @@ class TaurusLCD(Qt.QLCDNumber, TaurusBaseWidget):
                 return
             if type(mi_value) == int:
                 mi_value = mi_value,
-            if not operator.isSequenceType(mi_value):
+            if not isinstance(mi_value, collections.Sequence):
                 return
             self._modelIndex = mi_value
         self._modelIndexStr = mi
@@ -348,9 +323,6 @@ class TaurusLCD(Qt.QLCDNumber, TaurusBaseWidget):
     #: with which this widget will get its data from. The convention used for
     #: the string can be found :ref:`here <model-concept>`.
     #:
-    #: In case the property :attr:`useParentModel` is set to True, the model
-    #: text must start with a '/' followed by the attribute name.
-    #:
     #: **Access functions:**
     #:
     #:     * :meth:`TaurusBaseWidget.getModel`
@@ -417,8 +389,8 @@ class TaurusLCD(Qt.QLCDNumber, TaurusBaseWidget):
 
 def demo():
     "LCD"
-    import demo.tauruslcddemo
-    return demo.tauruslcddemo.main()
+    from .demo import tauruslcddemo
+    return tauruslcddemo.main()
 
 
 def main():
