@@ -26,10 +26,13 @@
 """
 Extension of :mod:`guiqwt.plot`
 """
-__all__ = ["TaurusCurveDialog", "TaurusTrendDialog", "TaurusImageDialog"]
+from builtins import next
+from builtins import str
 
+import click
 import copy
 
+from future.utils import string_types
 from guiqwt.plot import ImageDialog, CurveDialog
 
 import taurus.core
@@ -42,6 +45,9 @@ from taurus.qt.qtgui.extra_guiqwt.builder import make
 from taurus.qt.qtgui.extra_guiqwt.curve import TaurusCurveItem, TaurusTrendParam, TaurusTrendItem
 
 
+__all__ = ["TaurusCurveDialog", "TaurusTrendDialog", "TaurusImageDialog"]
+
+
 class TaurusCurveDialog(CurveDialog, TaurusBaseWidget):
     '''A taurus dialog for showing 1D data.
     It behaves as a regular :class:`guiqwt.plot.CurveDialog` but it also offers
@@ -51,12 +57,13 @@ class TaurusCurveDialog(CurveDialog, TaurusBaseWidget):
     .. seealso:: :class:`TaurusCurveWidget`
     '''
     _modifiableByUser = True
-    modelChanged = Qt.pyqtSignal([], ['QStringList'], [str])
+    modelChanged = Qt.pyqtSignal([], ['QStringList'], ['QString'])
 
     def __init__(self, parent=None, designMode=False, toolbar=True, **kwargs):
         '''see :class:`guiqwt.plot.CurveDialog` for other valid initialization parameters'''
         CurveDialog.__init__(self, parent=parent, toolbar=toolbar, **kwargs)
         TaurusBaseWidget.__init__(self, 'TaurusCurveDialog')
+        self.deprecated(rel='4.1', dep='TaurusCurveDialog', alt='TaurusPlot / taurus tpg plot launcher')
         self.setWindowFlags(Qt.Qt.Widget)
         self._designMode = designMode
         self._modelNames = CaselessList()
@@ -81,7 +88,7 @@ class TaurusCurveDialog(CurveDialog, TaurusBaseWidget):
 
     def _splitModel(self, modelNames):
         '''convert str to list if needed (commas and whitespace are considered as separators)'''
-        if isinstance(modelNames, (basestring, Qt.QString)):
+        if isinstance(modelNames, string_types):
             modelNames = str(modelNames).replace(',', ' ')
             modelNames = modelNames.split()
         return modelNames
@@ -151,7 +158,7 @@ class TaurusCurveDialog(CurveDialog, TaurusBaseWidget):
             else:
                 self.warning('Invalid model "%s" (Skipping)' % mx_my)
             # cycle styles
-            style = self.style.next()
+            style = next(self.style)
             color = style[0]
             linestyle = style[1:]
             # add the item
@@ -197,12 +204,13 @@ class TaurusTrendDialog(CurveDialog, TaurusBaseWidget):
     '''
     _modifiableByUser = True
 
-    modelChanged = Qt.pyqtSignal([], ['QStringList'], [str])
+    modelChanged = Qt.pyqtSignal([], ['QStringList'], ['QString'])
 
     def __init__(self, parent=None, designMode=False, taurusparam=None, toolbar=True, **kwargs):
         '''see :class:`guiqwt.plot.CurveDialog` for other valid initialization parameters'''
         CurveDialog.__init__(self, parent=parent, toolbar=toolbar, **kwargs)
         TaurusBaseWidget.__init__(self, 'TaurusTrendDialog')
+        self.deprecated(rel='4.1', dep='TaurusTrendDialog', alt='TaurusTrend / taurus tpg trend launcher')
         self.setWindowFlags(Qt.Qt.Widget)
         self._designMode = designMode
         self._modelNames = CaselessList()
@@ -235,7 +243,7 @@ class TaurusTrendDialog(CurveDialog, TaurusBaseWidget):
 
     def _splitModel(self, modelNames):
         '''convert str to list if needed (commas and whitespace are considered as separators)'''
-        if isinstance(modelNames, (basestring, Qt.QString)):
+        if isinstance(modelNames, string_types):
             modelNames = str(modelNames).replace(',', ' ')
             modelNames = modelNames.split()
         return modelNames
@@ -287,7 +295,7 @@ class TaurusTrendDialog(CurveDialog, TaurusBaseWidget):
         # create and attach new TaurusCurveItems
         for m in modelNames:
             # cycle styles
-            style = self.style.next()
+            style = next(self.style)
             # add the item
             item = make.ttrend(m, color=style[0], linestyle=style[
                                1:], linewidth=2, taurusparam=copy.deepcopy(self.defaultTaurusparam))
@@ -367,7 +375,7 @@ class TaurusTrendDialog(CurveDialog, TaurusBaseWidget):
                 timetool = self.get_tool(TimeAxisTool)
             timetool.set_scale_y_t(True)
         elif mode == 'deltatime':
-            from taurus.qt.qtgui.plot import DeltaTimeScaleEngine
+            from taurus.qt.qtgui.extra_guiqwt.scales import DeltaTimeScaleEngine
             plot = self.get_plot()
             DeltaTimeScaleEngine.enableInAxis(plot, plot.xBottom, rotation=-45)
         elif mode == 'event':
@@ -620,49 +628,48 @@ def taurusTrendDlgMain():
     sys.exit(app.exec_())
 
 
-def taurusImageDlgMain():
+@click.command('image')
+@click.argument('model', nargs=1, required=False)
+@click.option('-c', '--color-mode', 'color_mode',
+              type=click.Choice(['gray', 'rgb']),
+              default='gray',
+              show_default=True,
+              help=('Color mode expected from the attribute')
+              )
+@click.option("--demo", is_flag=True, help="show a demo of the widget")
+@click.option('--window-name', 'window_name',
+              default='TaurusPlot (qwt5)',
+              help='Name of the window')
+def image_cmd(model, color_mode, demo, window_name):
     from taurus.qt.qtgui.application import TaurusApplication
-    import taurus.core
     import sys
 
-    # prepare options
-    parser = taurus.core.util.argparse.get_taurus_parser()
-    parser.set_usage("%prog [options] <model>")
-    parser.set_description(
-        'a Taurus application for plotting Image Attributes')
-    parser.add_option("--demo", action="store_true", dest="demo",
-                      default=False, help="show a demo of the widget")
-    parser.add_option("--rgb", action="store_true", dest="rgb_mode",
-                      default=False, help="assume image is RGB")
-    parser.add_option("--window-name", dest="window_name",
-                      default="Taurus Image", help="Name of the window")
-    app = TaurusApplication(
-        cmd_line_parser=parser, app_name="Taurus Image Dialog", app_version=taurus.Release.version)
-    args = app.get_command_line_args()
-    options = app.get_command_line_options()
+    app = TaurusApplication(cmd_line_parser=None,
+                            app_name="Taurus Image Dialog")
 
-    # check & process options
-    if options.demo:
-        if options.rgb_mode:
-            args.append('eval:randint(0,256,(10,20,3))')
+    rgb_mode = (color_mode == 'rgb')
+
+    # TODO:  is "-c rgb --demo" doing the right thing?? Check it.
+    if demo:
+        if color_mode == 'rgb':
+            model = 'eval:randint(0,256,(10,20,3))'
         else:
-            args.append('eval:rand(256,128)')
-    w = TaurusImageDialog(wintitle=options.window_name)
+            model = 'eval:rand(256,128)'
 
-    w.setRGBmode(options.rgb_mode)
+    w = TaurusImageDialog(wintitle=window_name)
+
+    w.setRGBmode(rgb_mode)
 
     # set model
-    if len(args) == 1:
-        w.setModel(args[0])
-    else:
-        parser.print_help(sys.stderr)
-        sys.exit(1)
+    if model:
+        w.setModel(model)
 
     w.show()
     sys.exit(app.exec_())
 
 
 if __name__ == "__main__":
-    taurusCurveDlgMain()
+    image_cmd()
+    # taurusCurveDlgMain()
     # taurusTrendDlgMain()
-#    taurusImageDlgMain()
+    # taurusImageDlgMain()
