@@ -28,13 +28,18 @@ taurustrend.py: Generic trend widget for Taurus
 """
 __all__ = ["TaurusTrend2DDialog"]
 
+import click
+
 from guiqwt.plot import ImageDialog
 from taurus.external.qt import Qt
 import taurus.core
+from taurus.qt.qtcore.configuration import BaseConfigurableClass
 from taurus.qt.qtgui.base import TaurusBaseWidget
 from taurus.qt.qtgui.extra_guiqwt.image import TaurusTrend2DItem
 from taurus.qt.qtgui.extra_guiqwt.tools import (TaurusModelChooserTool,
-                                                TimeAxisTool, AutoScrollTool)
+                                                TimeAxisTool, AutoScrollTool,
+                                                AutoScaleXTool, AutoScaleYTool,
+                                                AutoScaleZTool)
 
 
 class TaurusTrend2DDialog(ImageDialog, TaurusBaseWidget):
@@ -49,7 +54,17 @@ class TaurusTrend2DDialog(ImageDialog, TaurusBaseWidget):
     _modifiableByUser = True
 
     def __init__(self, parent=None, designMode=False, toolbar=True,
-                 stackMode='deltatime', buffersize=512, options=None, **kwargs):
+                 stackMode='deltatime', buffersize=512, options=None,
+                 autoscale='xyz', **kwargs):
+        """
+
+        :param param: param to be passed to XYImageItem constructor
+        :param buffersize: (int) size of the stack
+        :param stackMode: (str) can be 'datetime', 'timedelta' or 'event'
+        :param autoscale: (str) if autscale string contains 'x', the x axis 
+                          will be autoscaled. Same with 'y' and 'z'.
+                          Defaults to 'xyz'
+        """
         """see :class:`guiqwt.plot.ImageDialog` for other valid initialization
         parameters"""
         defaultOptions = dict(lock_aspect_ratio=False)
@@ -65,12 +80,69 @@ class TaurusTrend2DDialog(ImageDialog, TaurusBaseWidget):
         self.setStackMode(stackMode)
         self.setWindowFlags(Qt.Qt.Widget)
         # add some tools
-        for toolklass in (TaurusModelChooserTool, AutoScrollTool):
+        for toolklass in (TaurusModelChooserTool, AutoScrollTool,
+                          AutoScaleXTool, AutoScaleYTool, AutoScaleZTool
+                          ):
             self.add_tool(toolklass)
         self.get_tool(TaurusModelChooserTool).singleModel = True
+
+        if 'x' in autoscale.lower():
+            self.get_tool(AutoScaleXTool).setChecked(True)
+        if 'y' in autoscale.lower():
+            self.get_tool(AutoScaleYTool).setChecked(True)
+        if 'z' in autoscale.lower():
+            self.get_tool(AutoScaleZTool).setChecked(True)
+
         self.setModifiableByUser(self._modifiableByUser)
         self.setContextMenuPolicy(Qt.Qt.CustomContextMenu)
-        self.registerConfigDelegate(self.get_tool(AutoScrollTool))
+
+        # Config properties
+        self.setModelInConfig(True)
+        self.registerConfigDelegate(self.trendItem or BaseConfigurableClass(),
+                                    name='trendItem')
+        self.registerConfigDelegate(self.get_tool(AutoScrollTool),
+                                    name='AutoScrollTool')
+        self.registerConfigDelegate(self.get_tool(AutoScaleXTool),
+                                    name='AutoScaleXTool')
+        self.registerConfigDelegate(self.get_tool(AutoScaleYTool),
+                                    name='AutoScaleYTool')
+        self.registerConfigDelegate(self.get_tool(AutoScaleZTool),
+                                    name='AutoScaleZTool')
+        self.registerConfigProperty(self.getStackMode,
+                                    self.setStackMode,
+                                    'stackMode'
+                                    )
+        self.registerConfigProperty(self._get_axes_conf,
+                                    self._set_axes_conf,
+                                    'axes_confs'
+                                    )
+        
+    def _get_axes_conf(self):
+        p = self.get_plot()
+        conf = {}
+        for a in p.AXIS_IDS:
+            c = conf[a] = {}
+            c['limits'] = p.get_axis_limits(a)
+            c['title'] = p.get_axis_title(a)
+            c['color'] = p.get_axis_color(a)
+            c['scale'] = p.get_axis_scale(a)
+            c['unit'] = p.get_axis_unit(a)
+        return conf
+
+    def _set_axes_conf(self, axes_conf):
+        p = self.get_plot()
+        for a in p.AXIS_IDS:
+            c = axes_conf[a]
+            if 'limits' in c:
+                p.set_axis_limits(a, *c['limits'])
+            if 'title' in c:
+                p.set_axis_title(a, c['title'])
+            if 'color' in c:
+                p.set_axis_color(a, c['color'])
+            if 'scale' in c:
+                p.set_axis_scale(a, c['scale'], autoscale=False)
+            if 'unit' in c:
+                p.set_axis_unit(a, c['unit'])
 
     def keyPressEvent(self, event):
         if(event.key() == Qt.Qt.Key_Escape):
@@ -94,7 +166,7 @@ class TaurusTrend2DDialog(ImageDialog, TaurusBaseWidget):
             timetool = self.get_tool(TimeAxisTool)
             timetool.set_scale_y_t(True)
         elif mode == 'deltatime':
-            from taurus.qt.qtgui.plot import DeltaTimeScaleEngine
+            from taurus.qt.qtgui.extra_guiqwt.scales import DeltaTimeScaleEngine
             plot = self.get_plot()
             DeltaTimeScaleEngine.enableInAxis(plot, plot.xBottom, rotation=-45)
         elif mode == 'event':
@@ -127,7 +199,7 @@ class TaurusTrend2DDialog(ImageDialog, TaurusBaseWidget):
         self.trendItem = TaurusTrend2DItem(
             stackMode=self.getStackMode(), buffersize=self.buffersize)
         self.trendItem.setModel(model)
-        plot.add_item(self.trendItem)
+        plot.add_item(self.trendItem, autoscale=False)
         self.trendItem.set_readonly(not self.isModifiableByUser())
         plot.set_axis_title(plot.colormap_axis, 'value')
         plot.set_axis_unit('left', 'index')
@@ -146,6 +218,11 @@ class TaurusTrend2DDialog(ImageDialog, TaurusBaseWidget):
 
         self.trendItem.dataChanged.connect(self.update_cross_sections)
 
+        # unregister old trendItem and register the new one as config delegate
+        self.unregisterConfigurableItem ("trendItem", raiseOnError=False)
+        if self.getModelInConfig():
+            self.registerConfigDelegate(self.trendItem, name='trendItem')
+
     def getModel(self):
         """reimplemented from :class:`TaurusBaseWidget`"""
         if self.trendItem is None:
@@ -204,7 +281,7 @@ class TaurusTrend2DDialog(ImageDialog, TaurusBaseWidget):
     def getQtDesignerPluginInfo(cls):
         """reimplemented from :class:`TaurusBaseWidget`"""
         ret = TaurusBaseWidget.getQtDesignerPluginInfo()
-        ret['module'] = 'taurus.qt.qtgui.plot'
+        ret['module'] = 'taurus.qt.qtgui.extra_guiqwt'
         ret['group'] = 'Taurus Display'
         ret['icon'] = 'designer:qwtplot.png'
         return ret
@@ -231,68 +308,46 @@ class TaurusTrend2DDialog(ImageDialog, TaurusBaseWidget):
                                        TaurusBaseWidget.resetModifiableByUser)
 
 
-def taurusTrend2DMain():
+@click.command('trend2d')
+@click.argument('model', nargs=1, required=False)
+@click.option("-x", "--x-axis-mode", "x_axis_mode",
+              type=click.Choice(['t', 'd', 'e']),
+              default='d',
+              show_default=True,
+              help=("interpret X values as timestamps (t), "
+                    + "time deltas (d) or event numbers (e). ")
+              )
+@click.option('-b', '--buffer', 'max_buffer_size', type=int,
+              default=512,
+              show_default=True,
+              help=("maximum number of values to be stacked "
+                    + "(when reached, the oldest values will be "
+                    + "discarded)")
+              )
+@click.option("--demo", is_flag=True, help="show a demo of the widget")
+@click.option('--window-name', 'window_name',
+              default='TaurusPlot (qwt5)',
+              help='Name of the window')
+def trend2d_cmd(model, x_axis_mode, max_buffer_size,
+                demo, window_name):
     from taurus.qt.qtgui.application import TaurusApplication
-    import taurus.core
     import sys
 
-    # prepare options
-    parser = taurus.core.util.argparse.get_taurus_parser()
-    parser.set_usage("%prog [options] <model>")
-    parser.set_description('a Taurus application for plotting trends of ' +
-                           'arrays (aka "spectrograms")')
-    parser.add_option("-x", "--x-axis-mode", dest="x_axis_mode", default='d',
-                      metavar="t|d|e",
-                      help=("interpret X values as timestamps (t), " +
-                            "time deltas (d) or event numbers (e). " +
-                            "Accepted values: t|d|e")
-                      )
-    parser.add_option("-b", "--buffer", dest="max_buffer_size", default='512',
-                      help=("maximum number of values to be stacked " +
-                            "(when reached, the oldest values will be " +
-                            "discarded)")
-                      )
-    parser.add_option("-a", "--use-archiving",
-                      action="store_true", dest="use_archiving", default=False)
-    parser.add_option("--demo", action="store_true", dest="demo",
-                      default=False, help="show a demo of the widget")
-    parser.add_option("--window-name", dest="window_name",
-                      default="Taurus Trend 2D", help="Name of the window")
+    if demo:
+        model = 'eval:x=linspace(0,3,40);t=rand();sin(x+t)'
 
-    app = TaurusApplication(cmd_line_parser=parser, app_name="Taurus Trend 2D",
-                            app_version=taurus.Release.version)
-    args = app.get_command_line_args()
-    options = app.get_command_line_options()
+    app = TaurusApplication(cmd_line_parser=None, app_name="Taurus Trend 2D")
 
-    # check & process options
-    stackModeMap = dict(t='datetime', d='deltatime', e='event')
-    if options.x_axis_mode.lower() not in stackModeMap:
-        parser.print_help(sys.stderr)
-        sys.exit(1)
+    stackMode = dict(t='datetime', d='deltatime', e='event')[x_axis_mode]
 
-    stackMode = stackModeMap[options.x_axis_mode.lower()]
-
-    if options.demo:
-        args.append('eval:x=linspace(0,3,40);t=rand();sin(x+t)')
-
-    w = TaurusTrend2DDialog(stackMode=stackMode, wintitle=options.window_name,
-                            buffersize=int(options.max_buffer_size))
-
-    # set archiving
-    if options.use_archiving:
-        raise NotImplementedError('Archiving support is not yet implemented')
-        w.setUseArchiving(True)
-
-    # set model
-    if len(args) == 1:
-        w.setModel(args[0])
-    else:
-        parser.print_help(sys.stderr)
-        sys.exit(1)
+    w = TaurusTrend2DDialog(stackMode=stackMode, wintitle=window_name,
+                            buffersize=max_buffer_size)
+    if model:
+        w.setModel(model)
 
     w.show()
     sys.exit(app.exec_())
 
 
 if __name__ == "__main__":
-    taurusTrend2DMain()
+    trend2d_cmd()
