@@ -27,16 +27,19 @@
 nexusWidget.py:
 """
 
-__all__ = ["TaurusNexusBrowser"]
+from builtins import str
 
 import numpy
 import posixpath
+from functools import partial
 
+from taurus.external.qt import Qt, compat
 from PyMca5.PyMcaGui.io.hdf5 import HDF5Widget, HDF5Info, HDF5DatasetTable
-from taurus.external.qt import Qt
 
 from taurus.qt.qtgui.container import TaurusWidget
-from taurus.qt.qtgui.plot import TaurusPlot
+
+
+__all__ = ["TaurusNexusBrowser"]
 
 
 class NeXusInfoWidget(Qt.QTabWidget):
@@ -102,17 +105,19 @@ class TaurusNeXusBrowser(TaurusWidget):
         # connections
         self.__fileModel.sigFileAppended.connect(self.treeWidget.fileAppended)
         self.treeWidget.sigHDF5WidgetSignal.connect(self.onHDF5WidgetSignal)
-        self.openFileAction.triggered[()].connect(self.openFile)
+        self.openFileAction.triggered.connect(
+            partial(self.openFile, fname=None))
         self.togglePreviewAction.toggled.connect(self.__previewStack.setVisible)
 
         # configuration
         self.registerConfigProperty(
             self.togglePreviewAction.isChecked, self.togglePreviewAction.setChecked, 'showPreview')
 
+    @Qt.pyqtSlot()
+    @Qt.pyqtSlot('QString')
     def openFile(self, fname=None):
         if fname is None:
-            fname = unicode(Qt.QFileDialog.getOpenFileName(
-                self, "Choose NeXus File", "/home/cpascual/local/tmp/scantest.h5"))  # @TODO!!
+            fname, _ = compat.getOpenFileName(self, 'Choose NeXus File', '')
         if fname:
             self.__nexusFile = self.__fileModel.openFile(fname)
 
@@ -129,8 +134,13 @@ class TaurusNeXusBrowser(TaurusWidget):
             node = ddict['name']
             data = self.__nexusFile[node]
             if len(data.shape) == 1 and isinstance(data[0], (numpy.floating, numpy.integer, int, float)):
-                w = TaurusPlot()
-                w.attachRawData({"x": numpy.arange(len(data)), "y": data})
+                try:
+                    import pyqtgraph as pg
+                    w = pg.PlotWidget()
+                    w.plot(data)
+                except ImportError:
+                    w = HDF5DatasetTable.HDF5DatasetTable()
+                    w.setDataset(data)
             else:
                 w = HDF5DatasetTable.HDF5DatasetTable()
                 w.setDataset(data)
@@ -186,11 +196,14 @@ class TaurusNeXusBrowser(TaurusWidget):
 
 if __name__ == "__main__":
     import sys
+    from taurus.qt.qtgui.application import TaurusApplication
+
     if len(sys.argv) > 1:
         fname = sys.argv[1]
     else:
         fname = None
-    app = Qt.QApplication(sys.argv)
+
+    app = TaurusApplication(sys.argv, cmd_line_parser=None)
     w = TaurusNeXusBrowser()
     w.openFile(fname)
     w.show()
