@@ -28,5 +28,7 @@ Utilities for using the bundled icons in Taurus and for registering external
 sources of icons.
 """
 
-from icon import *
-from catalog import QIconCatalog
+from __future__ import absolute_import
+
+from .icon import *
+from .catalog import QIconCatalog
