@@ -70,14 +70,12 @@ class TaurusValueComboBox(Qt.QComboBox, TaurusBaseWritableWidget):
         '''reimplemented from :class:`TaurusBaseWritableWidget`'''
         TaurusBaseWritableWidget.preAttach(self)
         self.currentIndexChanged.connect(self.writeIndexValue)
-        self.applied.connect(self.writeValue)
 
     def postDetach(self):
         '''reimplemented from :class:`TaurusBaseWritableWidget`'''
         TaurusBaseWritableWidget.postDetach(self)
         try:
             self.currentIndexChanged.disconnect(self.writeIndexValue)
-            self.applied.disconnect(self.writeValue)
         except TypeError:
             # In new style-signal if a signal is disconnected without
             # previously was connected it, it raises a TypeError
@@ -110,7 +108,6 @@ class TaurusValueComboBox(Qt.QComboBox, TaurusBaseWritableWidget):
             func = bool
         else:
             return None
-        new_value = Qt.from_qvariant(new_value, func)
         return new_value
 
     def setValue(self, value):
@@ -118,13 +115,14 @@ class TaurusValueComboBox(Qt.QComboBox, TaurusBaseWritableWidget):
         Set the value for the widget to display, not the value of the
         attribute.
         """
-        index = self.findData(Qt.QVariant(value))
+        index = self.findData(value)
         self._setCurrentIndex(index)
 
     def updateStyle(self):
         '''reimplemented from :class:`TaurusBaseWritableWidget`'''
         if self.hasPendingOperations():
-            self.setStyleSheet('TaurusValueComboBox {color: blue; }')
+            self.setStyleSheet(
+                'TaurusValueComboBox {color: blue; font-weight: bold;}')
         else:
             self.setStyleSheet('TaurusValueComboBox {}')
         super(TaurusValueComboBox, self).updateStyle()
@@ -135,16 +133,19 @@ class TaurusValueComboBox(Qt.QComboBox, TaurusBaseWritableWidget):
 
     @Qt.pyqtSlot(int, name='currentIndexChanged')
     def writeIndexValue(self, index):
-        '''slot called to emit a valueChanged signal when the currentIndex is changed'''
+        """slot called to emit a valueChanged signal when the currentIndex is
+        changed (and trigger a write if AutoApply is enabled)
+        """
         self.emitValueChanged()
         if self.getAutoApply():
-            self.applied.emit()
+            self.writeValue()
 
     def keyPressEvent(self, event):
-        '''reimplemented to emit an 'applied()' signal when Enter (or Return)
-        key is pressed'''
+        """reimplemented to trigger a write when Enter (or Return) key is
+        pressed
+        """
         if event.key() in [Qt.Qt.Key_Return, Qt.Qt.Key_Enter]:
-            self.applied.emit()
+            self.writeValue()
             event.accept()
         else:
             return Qt.QComboBox.keyPressEvent(self, event)
@@ -178,7 +179,7 @@ class TaurusValueComboBox(Qt.QComboBox, TaurusBaseWritableWidget):
         bs = self.blockSignals(True)
         try:
             for k, v in names:
-                self.addItem(k, Qt.QVariant(v))
+                self.addItem(k, v)
 
             # Ok, now we should see if the current value matches any
             # of the newly added names. This is kinda a refresh:
@@ -199,7 +200,7 @@ class TaurusValueComboBox(Qt.QComboBox, TaurusBaseWritableWidget):
                         a '%s' placeholder which will be substituted with
                         str(value). It defaults to 'UNKNOWN(%s)'.
         """
-        item = self.findData(Qt.QVariant(value))
+        item = self.findData(value)
         if item < 0:
             if '%s' in default:
                 return default % str(value)
@@ -365,7 +366,7 @@ class TaurusAttrListComboBox(Qt.QComboBox, TaurusBaseWidget):
 def _taurusAttrListTest():
     """tests taurusAttrList. Model: an attribute containing a list of strings"""
     from taurus.qt.qtgui.application import TaurusApplication
-    a = TaurusApplication()
+    a = TaurusApplication(cmd_line_parser=None)
     # model = sys.argv[1]
     # model = "eval:['foo','bar']"
     model = "sys/tg_test/1/string_spectrum"
@@ -379,23 +380,30 @@ def _taurusValueComboboxTest():
     from taurus.qt.qtgui.application import TaurusApplication
     """tests TaurusValueCombobox """
     # model = sys.argv[1]
-    model = 'sys/tg_test/1/short_scalar'
+
     names = [
         ('name0', 0),
         ('name1', 1),
         ('name2', 2),
         ('name3', 3)
     ]
-    a = TaurusApplication()
-    w = TaurusValueComboBox()
-    w.setModel(model)
-    w.addValueNames(names)
-    #w.autoApply = True
+    a = TaurusApplication(cmd_line_parser=None)
+    w = Qt.QWidget()
+    w.setLayout(Qt.QVBoxLayout())
+
+    cs = []
+    for model in ['sys/tg_test/1/short_scalar'] * 2:
+        c = TaurusValueComboBox()
+        c.setModel(model)
+        c.addValueNames(names)
+        w.layout().addWidget(c)
+        cs.append(c)
+        # c.autoApply = True
     w.show()
     return a.exec_()
 
 if __name__ == '__main__':
     import sys
-    # main = _taurusValueComboboxTest #uncomment to test TaurusValueCombobox
-    main = _taurusAttrListTest  # uncomment to testtaurusAttrList
+    main = _taurusValueComboboxTest #uncomment to test TaurusValueCombobox
+    # main = _taurusAttrListTest  # uncomment to testtaurusAttrList
     sys.exit(main())
