@@ -25,23 +25,40 @@
 
 """This module contains taurus Qt form widgets"""
 
-__all__ = ["TaurusAttrForm", "TaurusCommandsForm", "TaurusForm"]
-
-__docformat__ = 'restructuredtext'
+from __future__ import print_function
+from __future__ import absolute_import
 
+import click
 from datetime import datetime
+from functools import partial
+
+from future.utils import string_types, binary_type
 
 from taurus.external.qt import Qt
-import PyTango
 
 import taurus.core
-from taurus.core import TaurusDevState
+from taurus.core import TaurusDevState, DisplayLevel
 
 from taurus.qt.qtcore.mimetypes import (TAURUS_ATTR_MIME_TYPE, TAURUS_DEV_MIME_TYPE,
                                         TAURUS_MODEL_LIST_MIME_TYPE, TAURUS_MODEL_MIME_TYPE)
 from taurus.qt.qtgui.container import TaurusWidget, TaurusScrollArea
 from taurus.qt.qtgui.button import QButtonBox, TaurusCommandButton
-from taurusmodelchooser import TaurusModelChooser
+from .taurusmodelchooser import TaurusModelChooser
+
+__all__ = ["TaurusAttrForm", "TaurusCommandsForm", "TaurusForm"]
+
+__docformat__ = 'restructuredtext'
+
+
+def _normalize_model_name_case(modelname):
+    """
+    Accepts a model name and returns it in lower case if the models is
+    case insensitive. Otherwise it returns the same model name
+    """
+    if taurus.Factory(taurus.getSchemeFromName(modelname)).caseSensitive:
+        return modelname
+    else:
+        return modelname.lower()
 
 
 class ParameterCB(Qt.QComboBox):
@@ -89,6 +106,8 @@ class TaurusForm(TaurusWidget):
                  buttons=None,
                  withButtons=True,
                  designMode=False):
+
+        self._children = []
         TaurusWidget.__init__(self, parent, designMode)
 
         if buttons is None:
@@ -96,7 +115,7 @@ class TaurusForm(TaurusWidget):
                 Qt.QDialogButtonBox.Reset
         self._customWidgetMap = {}
         self._model = []
-        self._children = []
+        # self._children = []
         self.setFormWidget(formWidget)
 
         self.setLayout(Qt.QVBoxLayout())
@@ -120,7 +139,7 @@ class TaurusForm(TaurusWidget):
 
         self.chooseModelsAction = Qt.QAction('Modify Contents', self)
         self.addAction(self.chooseModelsAction)
-        self.chooseModelsAction.triggered[()].connect(self.chooseModels)
+        self.chooseModelsAction.triggered.connect(self.chooseModels)
 
         self.showButtonsAction = Qt.QAction('Show Buttons', self)
         self.showButtonsAction.setCheckable(True)
@@ -130,13 +149,17 @@ class TaurusForm(TaurusWidget):
 
         self.changeLabelsAction = Qt.QAction('Change labels (all items)', self)
         self.addAction(self.changeLabelsAction)
-        self.changeLabelsAction.triggered[()].connect(self.onChangeLabelsAction)
+        self.changeLabelsAction.triggered.connect(self.onChangeLabelsAction)
 
         self.compactModeAction = Qt.QAction('Compact mode (all items)', self)
         self.compactModeAction.setCheckable(True)
         self.addAction(self.compactModeAction)
         self.compactModeAction.triggered[bool].connect(self.setCompact)
 
+        self.setFormatterAction = Qt.QAction('Set formatter (all items)', self)
+        self.addAction(self.setFormatterAction)
+        self.setFormatterAction.triggered.connect(self.onSetFormatter)
+
         self.resetModifiableByUser()
         self.setSupportedMimeTypes([TAURUS_MODEL_LIST_MIME_TYPE, TAURUS_DEV_MIME_TYPE,
                                     TAURUS_ATTR_MIME_TYPE, TAURUS_MODEL_MIME_TYPE, 'text/plain'])
@@ -158,7 +181,9 @@ class TaurusForm(TaurusWidget):
 
     def _splitModel(self, modelNames):
         '''convert str to list if needed (commas and whitespace are considered as separators)'''
-        if isinstance(modelNames, (basestring, Qt.QString)):
+        if isinstance(modelNames, binary_type):
+            modelNames = modelNames.decode()
+        if isinstance(modelNames, string_types):
             modelNames = str(modelNames).replace(',', ' ')
             modelNames = modelNames.split()
         return modelNames
@@ -170,6 +195,7 @@ class TaurusForm(TaurusWidget):
                       type strings (i.e. see :class:`PyTango.DeviceInfo`) and
                       whose values are tuples of classname,args,kwargs
         '''
+        # TODO: tango-centric
         self._customWidgetMap = cwmap
 
     def getCustomWidgetMap(self):
@@ -179,6 +205,7 @@ class TaurusForm(TaurusWidget):
                  type strings (i.e. see :class:`PyTango.DeviceInfo`) and whose
                  values are tuples of classname,args,kwargs
         '''
+        # TODO: tango-centric
         return self._customWidgetMap
 
     @Qt.pyqtSlot('QString', name='modelChanged')
@@ -198,7 +225,7 @@ class TaurusForm(TaurusWidget):
         if self.__modelChooserDlg is None:
             self.__modelChooserDlg = Qt.QDialog(self)
             self.__modelChooserDlg.setWindowTitle(
-                "%s - Model Chooser" % unicode(self.windowTitle()))
+                "%s - Model Chooser" % str(self.windowTitle()))
             self.__modelChooserDlg.modelChooser = TaurusModelChooser()
             layout = Qt.QVBoxLayout()
             layout.addWidget(self.__modelChooserDlg.modelChooser)
@@ -206,11 +233,11 @@ class TaurusForm(TaurusWidget):
             self.__modelChooserDlg.modelChooser.updateModels.connect(self.setModel)
 
         models_and_labels = []
-        models = [m.lower() for m in self.getModel()]
         indexdict = {}
-        for m in models:
-            indexdict[m] = indexdict.get(m, -1) + 1
-            item = self.getItemByModel(m, indexdict[m])
+        for m in self.getModel():
+            key = _normalize_model_name_case(m)
+            indexdict[key] = indexdict.get(key, -1) + 1
+            item = self.getItemByModel(m, indexdict[key])
             if item is None:
                 label = None
             else:
@@ -362,6 +389,26 @@ class TaurusForm(TaurusWidget):
     def resetWithButtons(self):
         self.setWithButtons(True)
 
+    def onSetFormatter(self):
+        """Reimplemented from TaurusBaseWidget"""
+        # Form delegates se to the taurusvalues
+        format = TaurusWidget.onSetFormatter(self)
+        if format is not None:
+            for item in self.getItems():
+                rw = item.readWidget(followCompact=True)
+                if hasattr(rw, 'setFormat'):
+                    rw.setFormat(format)
+        return format
+
+    def setFormat(self, format):
+        """
+        Reimplemented to call setFormat on the taurusvalues
+        """
+        TaurusWidget.setFormat(self, format)
+        for item in self.getItems():
+            if hasattr(item, 'setFormat'):
+                item.setFormat(format)
+
     def setCompact(self, compact):
         self._compact = compact
         for item in self.getItems():
@@ -454,6 +501,11 @@ class TaurusForm(TaurusWidget):
                 widget.setModifiableByUser(self.isModifiableByUser())
             except:
                 pass
+            try:
+                widget.setFormat(self.getFormat())
+            except Exception:
+                self.debug('Cannot set format %s to child %s',
+                           self.getFormat(), model)
             widget.setObjectName("__item%i" % i)
             self.registerConfigDelegate(widget)
             self._children.append(widget)
@@ -469,7 +521,8 @@ class TaurusForm(TaurusWidget):
         with the same model, the index parameter can be used to distinguish among them
         Please note that his index is only relative to same-model items!'''
         for child in self._children:
-            if child.getModel().lower() == model.lower():
+            if (_normalize_model_name_case(child.getModel()) ==
+                    _normalize_model_name_case(model)):
                 if index <= 0:
                     return child
                 else:
@@ -546,6 +599,7 @@ class TaurusForm(TaurusWidget):
 class TaurusCommandsForm(TaurusWidget):
     '''A form that shows commands available for a Device Server'''
 
+    # TODO: tango-centric
     def __init__(self, parent=None, designMode=False):
         TaurusWidget.__init__(self, parent, designMode)
 
@@ -574,7 +628,7 @@ class TaurusCommandsForm(TaurusWidget):
         self._defaultParameters = []
         self._sortKey = lambda x: x.cmd_name
 
-        self._operatorViewFilter = lambda x: x.disp_level == PyTango.DispLevel.OPERATOR
+        self._operatorViewFilter = lambda x: x.disp_level == DisplayLevel.OPERATOR
 
         # self.setLayout(Qt.QGridLayout())
         self.modelChanged.connect(self._updateCommandWidgets)
@@ -617,30 +671,41 @@ class TaurusCommandsForm(TaurusWidget):
         Inserts command buttons and parameter widgets in the layout, according to
         the commands from the model
         '''
-        #self.debug('In TaurusCommandsForm._updateCommandWidgets())')
+
         dev = self.getModelObj()
-        if dev is None or dev.state != TaurusDevState.Ready:
-            self.debug('Cannot connect to device')
+        if dev is None:
+            self._clearFrame()
+            return
+
+        try:
+            commands = sorted(dev.command_list_query(), key=self._sortKey)
+        except Exception as e:
+            self.warning('Problem querying commands from %s. Reason: %s',
+                         dev, e)
             self._clearFrame()
             return
-        commands = sorted(dev.command_list_query(), key=self._sortKey)
 
         for f in self.getViewFilters():
-            commands = filter(f, commands)
+            commands = list(filter(f, commands))
 
         self._clearFrame()
 
         layout = self._frame.layout()
 
+        model = self.getFullModelName()
+
         for row, c in enumerate(commands):
             self.debug('Adding button for command %s' % c.cmd_name)
             button = TaurusCommandButton(command=c.cmd_name, text=c.cmd_name)
             layout.addWidget(button, row, 0)
-            button.setUseParentModel(True)
+            button.setModel(model)
             self._cmdWidgets.append(button)
             button.commandExecuted.connect(self._onCommandExecuted)
+            
+            import taurus.core.tango.util.tango_taurus as tango_taurus
+            in_type = tango_taurus.FROM_TANGO_TO_TAURUS_TYPE[c.in_type]
 
-            if c.in_type != PyTango.CmdArgType.DevVoid:
+            if in_type is not None:
                 self.debug('Adding arguments for command %s' % c.cmd_name)
                 pwidget = ParameterCB()
                 if c.cmd_name.lower() in self._defaultParameters:
@@ -654,7 +719,7 @@ class TaurusCommandsForm(TaurusWidget):
                         button.setParameters(self._defaultParameters[
                                              c.cmd_name.lower()][0])
                 pwidget.editTextChanged.connect(button.setParameters)
-                pwidget.currentIndexChanged.connect(button.setParameters)
+                pwidget.currentIndexChanged['QString'].connect(button.setParameters)
                 pwidget.activated.connect(button.setFocus)
                 button.commandExecuted.connect(pwidget.rememberCurrentText)
                 layout.addWidget(pwidget, row, 1)
@@ -771,7 +836,7 @@ class TaurusAttrForm(TaurusWidget):
         TaurusWidget.__init__(self, parent, designMode)
 
         self._viewFilters = []
-        self._operatorViewFilter = lambda x: x.disp_level == PyTango.DispLevel.OPERATOR
+        self._operatorViewFilter = lambda x: x.disp_level == DisplayLevel.OPERATOR
 
         self.setLayout(Qt.QVBoxLayout())
 
@@ -801,21 +866,22 @@ class TaurusAttrForm(TaurusWidget):
     def _updateAttrWidgets(self):
         '''Populates the form with an item for each of the attributes shown
         '''
-        dev = self.getModelObj()
-        if dev is None or dev.state != TaurusDevState.Ready:
+        try:
+            dev = self.getModelObj()
+            attrlist = sorted(dev.attribute_list_query(), key=self._sortKey)
+            for f in self.getViewFilters():
+                attrlist = list(filter(f, attrlist))
+            attrnames = []
+            devname = self.getModelName()
+            for a in attrlist:
+                # ugly hack . But setUseParentModel does not work well
+                attrnames.append("%s/%s" % (devname, a.name))
+            self.debug('Filling with attribute list: %s'
+                       % ("; ".join(attrnames)))
+            self._form.setModel(attrnames)
+        except:
             self.debug('Cannot connect to device')
             self._form.setModel([])
-            return
-        attrlist = sorted(dev.attribute_list_query(), key=self._sortKey)
-        for f in self.getViewFilters():
-            attrlist = filter(f, attrlist)
-        attrnames = []
-        devname = self.getModelName()
-        for a in attrlist:
-            # ugly hack . But setUseParentModel does not work well
-            attrnames.append("%s/%s" % (devname, a.name))
-        self.debug('Filling with attribute list: %s' % ("; ".join(attrnames)))
-        self._form.setModel(attrnames)
 
     def setViewFilters(self, filterlist):
         '''sets the filters to be applied when displaying the attributes
@@ -882,7 +948,9 @@ def test1():
         models = sys.argv[1:]
     else:
         models = None
-    app = Qt.QApplication(sys.argv)
+    from taurus.qt.qtgui.application import TaurusApplication
+
+    app = TaurusApplication(sys.argv, cmd_line_parser=None)
     if models is None:
         models = ['sys/tg_test/1/state',
                   'sys/tg_test/1/float_scalar',
@@ -912,7 +980,9 @@ def test2():
 
     if model is None:
         model = 'bl97/pc/dummy-01'
-    app = Qt.QApplication(sys.argv)
+    from taurus.qt.qtgui.application import TaurusApplication
+
+    app = TaurusApplication(sys.argv, cmd_line_parser=None)
     dialog = TaurusAttrForm()
     dialog.setModel(model)
     dialog.show()
@@ -929,7 +999,9 @@ def test3():
 
     if model is None:
         model = 'bl97/pc/dummy-01'
-    app = Qt.QApplication(sys.argv)
+    from taurus.qt.qtgui.application import TaurusApplication
+
+    app = TaurusApplication(sys.argv, cmd_line_parser=None)
     dialog = TaurusCommandsForm()
     dialog.setModel(model)
     dialog.show()
@@ -942,14 +1014,16 @@ def test4():
     '''tests customwidgetma in taurusforms'''
     import sys
     from taurus.qt.qtgui.display import TaurusLabel
-    app = Qt.QApplication(sys.argv)
+    from taurus.qt.qtgui.application import TaurusApplication
+
+    app = TaurusApplication(sys.argv, cmd_line_parser=None)
 
     from taurus.qt.qtgui.panel import TaurusValue
 
     class DummyCW(TaurusValue):
 
         def setModel(self, model):
-            print "!!!!! IN DUMMYCW.SETMODEL", model
+            print("!!!!! IN DUMMYCW.SETMODEL", model)
             TaurusValue.setModel(self, model + '/double_scalar')
 
     models = ['sys/database/2', 'sys/tg_test/1', 'sys/tg_test/1/short_spectrum',
@@ -973,33 +1047,23 @@ def test4():
     sys.exit(app.exec_())
 
 
-def taurusFormMain():
-    '''A launcher for TaurusForm.'''
-    # NOTE: DON'T PUT TEST CODE HERE.
-    # THIS IS CALLED FROM THE LAUNCHER SCRIPT (<taurus>/scripts/taurusform)
-    # USE test1() instead.
+@click.command('form')
+@click.option('--window-name', 'window_name',
+              default='Taurus Form',
+              help='Name of the window')
+@click.option('--config', 'config_file', type=click.File('rb'),
+              help='configuration file for initialization')
+@click.argument('models', nargs=-1)
+def form_cmd(window_name, config_file, models):
+    """Shows a Taurus form populated with the given model names"""
     from taurus.qt.qtgui.application import TaurusApplication
-    from taurus.core.util import argparse
     import sys
-    import os
-
-    parser = argparse.get_taurus_parser()
-    parser.set_usage("%prog [options] [model1 [model2 ...]]")
-    parser.set_description("the taurus form panel application")
-    parser.add_option("--window-name", dest="window_name",
-                      default="TaurusForm", help="Name of the window")
-    parser.add_option("--config", "--config-file", dest="config_file", default=None,
-                      help="use the given config file for initialization")
-    app = TaurusApplication(cmd_line_parser=parser,
-                            app_name="taurusform",
-                            app_version=taurus.Release.version)
-    args = app.get_command_line_args()
-    options = app.get_command_line_options()
-
+    app = TaurusApplication(cmd_line_parser=None)
     dialog = TaurusForm()
     dialog.setModifiableByUser(True)
     dialog.setModelInConfig(True)
-    dialog.setWindowTitle(options.window_name)
+
+    dialog.setWindowTitle(window_name)
 
     # Make sure the window size and position are restored
     dialog.registerConfigProperty(dialog.saveGeometry, dialog.restoreGeometry,
@@ -1007,15 +1071,16 @@ def taurusFormMain():
 
     quitApplicationAction = Qt.QAction(
         Qt.QIcon.fromTheme("process-stop"), 'Close Form', dialog)
-    quitApplicationAction.triggered[()].connect(dialog.close)
+    quitApplicationAction.triggered.connect(dialog.close)
 
     saveConfigAction = Qt.QAction("Save current settings...", dialog)
     saveConfigAction.setShortcut(Qt.QKeySequence.Save)
-    saveConfigAction.triggered[()].connect(dialog.saveConfigFile)
-
+    saveConfigAction.triggered.connect(
+        partial(dialog.saveConfigFile, ofile=None))
     loadConfigAction = Qt.QAction("&Retrieve saved settings...", dialog)
     loadConfigAction.setShortcut(Qt.QKeySequence.Open)
-    loadConfigAction.triggered[()].connect(dialog.loadConfigFile)
+    loadConfigAction.triggered.connect(
+        partial(dialog.loadConfigFile, ifile=None))
 
     dialog.addActions(
         (saveConfigAction, loadConfigAction, quitApplicationAction))
@@ -1026,16 +1091,14 @@ def taurusFormMain():
         getattr(tauruscustomsettings, 'T_FORM_CUSTOM_WIDGET_MAP', {}))
 
     # set a model list from the command line or launch the chooser
-    if options.config_file is not None:
-        dialog.loadConfigFile(options.config_file)
-    elif len(args) > 0:
-        models = args
+    if config_file is not None:
+        dialog.loadConfigFile(config_file)
+    elif len(models) > 0:
         dialog.setModel(models)
     else:
         dialog.chooseModels()
 
     dialog.show()
-
     sys.exit(app.exec_())
 
 
@@ -1044,7 +1107,7 @@ def main():
     # test2()
     # test3()
     # test4()
-    taurusFormMain()
+    form_cmd()
 
 if __name__ == "__main__":
     main()
