@@ -24,17 +24,23 @@
 #############################################################################
 
 """This module provides an Input panel (usually used inside a TaurusDialog)"""
+from __future__ import print_function
 
-__all__ = ["TaurusInputPanel"]
-
-__docformat__ = 'restructuredtext'
+from builtins import str
+from builtins import object
 
 import collections
 import numpy
 
+from future.utils import string_types
+
 from taurus.external.qt import Qt
 from taurus.qt.qtgui.util.ui import UILoadable
 
+__all__ = ["TaurusInputPanel"]
+
+__docformat__ = 'restructuredtext'
+
 
 @UILoadable(with_ui='_ui')
 class TaurusInputPanel(Qt.QWidget):
@@ -120,7 +126,7 @@ class TaurusInputPanel(Qt.QWidget):
         self.setText(input_data['prompt'])
 
         data_type = input_data.get('data_type', 'String')
-        is_seq = not isinstance(data_type, (str, unicode)) and \
+        is_seq = not isinstance(data_type, string_types) and \
             isinstance(data_type, collections.Sequence)
         if is_seq:
             panel, getter = self.create_selection_panel(input_data)
@@ -160,7 +166,7 @@ class TaurusInputPanel(Qt.QWidget):
         self._ui.inputWidget = combobox = Qt.QComboBox()
         items = input_data['data_type']
         for item in items:
-            is_seq = not isinstance(item, (str, unicode)) and \
+            is_seq = not isinstance(item, string_types) and \
                 isinstance(item, collections.Sequence)
             if is_seq:
                 text, userData = item
@@ -172,7 +178,7 @@ class TaurusInputPanel(Qt.QWidget):
 
     def _get_combobox_value(self):
         combo = self._ui.inputWidget
-        return Qt.from_qvariant(combo.itemData(combo.currentIndex()))
+        return combo.itemData(combo.currentIndex())
 
     def _create_radiobutton_panel(self, input_data):
         panel = self._create_group_panel(input_data)
@@ -182,7 +188,7 @@ class TaurusInputPanel(Qt.QWidget):
         self._ui.inputWidget = buttongroup = Qt.QButtonGroup()
         buttongroup.setExclusive(True)
         for item in items:
-            is_seq = not isinstance(item, (str, unicode)) and \
+            is_seq = not isinstance(item, string_types) and \
                 isinstance(item, collections.Sequence)
             if is_seq:
                 text, userData = item
@@ -209,7 +215,7 @@ class TaurusInputPanel(Qt.QWidget):
         default_value = input_data.get('default_value')
         if default_value is None:
             default_value = ()
-        dft_is_seq = not isinstance(default_value, (str, unicode)) and \
+        dft_is_seq = not isinstance(default_value, string_types) and \
             isinstance(default_value, collections.Sequence)
         if not dft_is_seq:
             default_value = default_value,
@@ -218,14 +224,14 @@ class TaurusInputPanel(Qt.QWidget):
         listwidget.setSelectionMode(Qt.QAbstractItemView.MultiSelection)
 
         for item in items:
-            is_seq = not isinstance(item, (str, unicode)) and \
+            is_seq = not isinstance(item, string_types) and \
                 isinstance(item, collections.Sequence)
             if is_seq:
                 text, userData = item
             else:
                 text, userData = str(item), item
             item_widget = Qt.QListWidgetItem(text, listwidget)
-            item_widget.setData(Qt.Qt.UserRole, Qt.to_qvariant(userData))
+            item_widget.setData(Qt.Qt.UserRole, userData)
             if userData in default_value:
                 item_widget.setSelected(True)
         layout.addWidget(listwidget)
@@ -233,7 +239,7 @@ class TaurusInputPanel(Qt.QWidget):
 
     def _get_multi_selection_value(self):
         listwidget = self._ui.inputWidget
-        return [Qt.from_qvariant(item.data(Qt.Qt.UserRole)) for item in listwidget.selectedItems()]
+        return [item.data(Qt.Qt.UserRole) for item in listwidget.selectedItems()]
 
     def _create_group_panel(self, input_data):
         title = input_data.get('key', '')
@@ -402,7 +408,7 @@ def main():
     class Listener(object):
 
         def on_accept(self):
-            print "user selected", self.panel.value()
+            print("user selected", self.panel.value())
 
     d = dict(prompt="What's your favourite car brand?",
              data_type=["Mazda", "Skoda", "Citroen",
