@@ -26,27 +26,127 @@
 """
 AttributeChooser.py: widget for choosing (a list of) attributes from a tango DB
 """
+from __future__ import print_function
+from __future__ import absolute_import
 
-__all__ = ["TaurusModelSelectorTree", "TaurusModelChooser"]
+from builtins import bytes, str
 
 import sys
-from taurus.external.qt import Qt
+import pkg_resources
+
+import taurus
+from taurus.external.qt import Qt, QtCore
+from taurus.external.qt.compat import PY_OBJECT
 import taurus.core
 from taurus.qt.qtgui.container import TaurusWidget
 from taurus.qt.qtgui.tree import TaurusDbTreeWidget
 from taurus.core.util.containers import CaselessList
-from taurusmodellist import TaurusModelList
+from .taurusmodellist import TaurusModelList
 
-#@todo: tango-centric!!
-from taurus.core.tango.tangodatabase import TangoDevInfo, TangoAttrInfo
+__all__ = ["TaurusModelSelectorTree", "TaurusModelChooser",
+           "TaurusModelSelector", "TaurusModelSelectorItem"]
 
 
-class TaurusModelSelectorTree(TaurusWidget):
+class TaurusModelSelector(Qt.QTabWidget):
+    """TaurusModelSelector is a QTabWidget container for
+    TaurusModelSelectorItem.
+    """
+    # TODO add action to add new TaurusModelSelectorItem
+    # TODO add mechanism to allow manual plugin activation
+    #      (instead of relying on installation)
+    modelsAdded = Qt.pyqtSignal(PY_OBJECT)
+
+    def __init__(self, parent=None):
+        Qt.QTabWidget.__init__(self, parent=parent)
+
+        self.currentChanged.connect(self.__setTabItemModel)
+        # ---------------------------------------------------------------------
+        # Note: this is an experimental feature
+        # It may be removed or changed in future releases
+        # Discover the taurus.modelselector plugins
+        ep_name = 'taurus.qt.qtgui.panel.TaurusModelSelector.items'
+        for ep in pkg_resources.iter_entry_points(ep_name):
+            try:
+                ms_class = ep.load()
+                ms_item = ms_class(parent=self)
+                self.__addItem(ms_item, ep.name)
+
+            except Exception as e:
+                err = 'Invalid TaurusModelSelectorItem plugin: {}\n{}'.format(
+                    ep.module_name, e)
+                taurus.warning(err)
+        # ---------------------------------------------------------------------
+
+    def __setTabItemModel(self):
+        w = self.currentWidget()
+        c = self.cursor()
+        try:
+            if not w.model:
+                self.setCursor(QtCore.Qt.WaitCursor)
+                w.setModel(w.default_model)
+        except Exception as e:
+            taurus.warning('Problem setting up selector: %r', e)
+        finally:
+            self.setCursor(c)
+
+    def __addItem(self, widget, name, model=None):
+        if model is not None:
+            widget.default_model = model
+
+        widget.modelsAdded.connect(self.modelsAdded)
+        self.addTab(widget, name)
+
+
+class TaurusModelSelectorItem(TaurusWidget):
+    """Base class for ModelSelectorItem.
+    It defines the minimal API to be defined in the specialization
+    """
+    modelsAdded = Qt.pyqtSignal(PY_OBJECT)
+    _dragEnabled = True
+    # TODO add action for setModel
+
+    def __init__(self, parent=None, **kwargs):
+        TaurusWidget.__init__(self, parent)
+        self._default_model = None
+
+    def getSelectedModels(self):
+        raise NotImplementedError(('getSelectedModels must be implemented'
+                                   + ' in TaurusModelSelectorItem subclass'))
+
+    def getModelMimeData(self):
+        """ Reimplemented from TaurusBaseComponent
+        """
+        models = self.getSelectedModels()
+        md = Qt.QMimeData()
+        md.setText(", ".join(models))
+        models_bytes = [bytes(m, encoding='utf-8') for m in models]
+        md.setData(taurus.qt.qtcore.mimetypes.TAURUS_MODEL_LIST_MIME_TYPE,
+                   b"\r\n".join(models_bytes))
+        return md
+
+    def _get_default_model(self):
+        """
+        Reimplement to return a default model to initialize the widget
+        """
+        raise NotImplementedError(('default_model must be implemented'
+                                   + ' in TaurusModelSelectorItem subclass'))
+
+    def _set_default_model(self, model):
+        """
+        Set default model to initialize the widget
+        """
+        self._default_model = model
+
+    # Reimplement this property
+    default_model = property(fget=_get_default_model, fset=_set_default_model)
+
+
+class TaurusModelSelectorTree(TaurusModelSelectorItem):
 
     addModels = Qt.pyqtSignal('QStringList')
 
     def __init__(self, parent=None, selectables=None, buttonsPos=None, designMode=None):
-        TaurusWidget.__init__(self, parent)
+        TaurusModelSelectorItem.__init__(self, parent)
         if selectables is None:
             selectables = [taurus.core.taurusbasetypes.TaurusElementType.Attribute, taurus.core.taurusbasetypes.TaurusElementType.Member,
                            taurus.core.taurusbasetypes.TaurusElementType.Device]
@@ -56,7 +156,6 @@ class TaurusModelSelectorTree(TaurusWidget):
         self._deviceTree = TaurusDbTreeWidget(
             perspective=taurus.core.taurusbasetypes.TaurusElementType.Device)
         self._deviceTree.getQModel().setSelectables(self._selectables)
-        self._deviceTree.setUseParentModel(True)
 
         # toolbar
         self._toolbar = Qt.QToolBar("TangoSelector toolbar")
@@ -68,8 +167,6 @@ class TaurusModelSelectorTree(TaurusWidget):
         # defines the layout
         self.setButtonsPos(buttonsPos)
 
-        self._deviceTree.recheckTaurusParent()  # NOT WORKING????
-        # @todo: This is Workaround because UseSetParentModel is giving trouble again!
         self.modelChanged.connect(self._deviceTree.setModel)
 
     def setButtonsPos(self, buttonsPos):
@@ -107,8 +204,13 @@ class TaurusModelSelectorTree(TaurusWidget):
             raise ValueError("Invalid buttons position")
 
     def getSelectedModels(self):
-        # todo: this method is tango-centric, but it could be fixed...
         selected = []
+        try:
+            from taurus.core.tango.tangodatabase import (TangoDevInfo,
+                                                         TangoAttrInfo)
+        except:
+            return selected
+        # TODO: Tango-centric
         for item in self._deviceTree.selectedItems():
             nfo = item.itemData()
             if isinstance(nfo, TangoDevInfo):
@@ -136,6 +238,34 @@ class TaurusModelSelectorTree(TaurusWidget):
         return ret
 
 
+class TangoModelSelectorItem(TaurusModelSelectorTree):
+    """A taurus model selector item for Tango models
+    """
+    # TODO: Tango-centric (move to Taurus-Tango plugin)
+    def __init__(self, parent=None, selectables=None,
+                 buttonsPos=Qt.Qt.RightToolBarArea, designMode=None):
+        TaurusModelSelectorTree.__init__(
+            self, parent=parent, selectables=selectables,
+            buttonsPos=buttonsPos, designMode=designMode)
+
+    def onAddSelected(self):
+        """
+        Reimplemented from TaurusModelSelectorTree to emit modelsAdded
+        signal instead of addModels
+        """
+        self.modelsAdded.emit(self.getSelectedModels())
+
+    def _get_default_model(self):
+        """Reimplemented from TaurusModelSelectorItem"""
+        if self._default_model is None:
+            f = taurus.Factory('tango')
+            self._default_model = f.getAuthority().getFullName()
+        return self._default_model
+
+    default_model = property(fget=_get_default_model,
+                             fset=TaurusModelSelectorTree._set_default_model)
+
+
 class TaurusModelChooser(TaurusWidget):
     '''A widget that allows the user to select a list of models from a tree representing
     devices and attributes from a Tango server.
@@ -163,8 +293,12 @@ class TaurusModelChooser(TaurusWidget):
                             model. Otherwise (default) a list of models can be selected
         '''
         TaurusWidget.__init__(self, parent)
+
         if host is None:
-            host = taurus.Authority().getNormalName()
+            try:  # TODO: Tango-centric!
+                host = taurus.Factory('tango').getAuthority().getFullName()
+            except Exception as e:
+                taurus.info('Cannot populate Tango Tree: %r', e)
 
         self._allowDuplicates = False
 
@@ -221,12 +355,16 @@ class TaurusModelChooser(TaurusWidget):
             models = models[:1]
         if asMimeData:
             md = Qt.QMimeData()
-            md.setData(taurus.qt.qtcore.mimetypes.TAURUS_MODEL_LIST_MIME_TYPE, str(
-                "\r\n".join(models)))
+            md.setData(
+                taurus.qt.qtcore.mimetypes.TAURUS_MODEL_LIST_MIME_TYPE,
+                bytes("\r\n".join(models), encoding="utf8")
+            )
             md.setText(", ".join(models))
             if len(models) == 1:
                 md.setData(
-                    taurus.qt.qtcore.mimetypes.TAURUS_MODEL_MIME_TYPE, str(models[0]))
+                    taurus.qt.qtcore.mimetypes.TAURUS_MODEL_MIME_TYPE,
+                    bytes(models[0], encoding="utf8")
+                )
             return md
         return models
 
@@ -294,7 +432,9 @@ class TaurusModelChooser(TaurusWidget):
         self.setSingleModelMode(self, False)
 
     @staticmethod
-    def modelChooserDlg(parent=None, selectables=None, host=None, asMimeData=False, singleModel=False, windowTitle='Model Chooser'):
+    def modelChooserDlg(parent=None, selectables=None, host=None, asMimeData=False,
+                        singleModel=False, windowTitle='Model Chooser',
+                        listedModels=None):
         '''Static method that launches a modal dialog containing a TaurusModelChooser
 
         :param parent: (QObject) parent for the dialog
@@ -308,6 +448,8 @@ class TaurusModelChooser(TaurusWidget):
         :param singleModel: (bool) If True, the selection will be of just one
                             model. Otherwise (default) a list of models can be selected
         :param windowTitle: (str) Title of the dialog (default="Model Chooser")
+        :param listedModels: (list<str>) List of model names for initializing the 
+                             model list
 
         :return: (list,bool or QMimeData,bool) Returns a models,ok tuple. models can be
                  either a list of models or a QMimeData object, depending on
@@ -320,6 +462,8 @@ class TaurusModelChooser(TaurusWidget):
         layout = Qt.QVBoxLayout()
         w = TaurusModelChooser(
             parent=parent, selectables=selectables, host=host, singleModel=singleModel)
+        if listedModels is not None:
+            w.setListedModels(listedModels)
         layout.addWidget(w)
         dlg.setLayout(layout)
         w.updateModels.connect(dlg.accept)
@@ -346,7 +490,7 @@ def main(args):
         host = None
 
     app = Qt.QApplication(args)
-    print TaurusModelChooser.modelChooserDlg(host=host)
+    print(TaurusModelChooser.modelChooserDlg(host=host))
     sys.exit()
 
 if __name__ == "__main__":
