@@ -27,17 +27,22 @@
 taurusvalue.py:
 """
 
-__all__ = ["TaurusValue", "TaurusValuesFrame", "DefaultTaurusValueCheckBox", "DefaultLabelWidget",
+__all__ = ["TaurusValue", "TaurusValuesFrame", "DefaultTaurusValueCheckBox",
            "DefaultUnitsWidget", "TaurusPlotButton", "TaurusArrayEditorButton",
-           "TaurusValuesTableButton", "TaurusValuesTableButton_W", "TaurusDevButton"]
+           "TaurusValuesTableButton", "TaurusValuesTableButton_W",
+           "DefaultLabelWidget", "DefaultReadWidgetLabel", "TaurusDevButton",
+           "TaurusImageButton"]
 
 __docformat__ = 'restructuredtext'
 
+from future.utils import string_types
+from future.builtins import str
+
 import weakref
 import re
 from taurus.external.qt import Qt
 import taurus.core
-from taurus.core import DataType, DataFormat
+from taurus.core import DataType, DataFormat, TaurusEventType
 
 from taurus.core.taurusbasetypes import TaurusElementType
 from taurus.qt.qtcore.mimetypes import TAURUS_ATTR_MIME_TYPE, TAURUS_DEV_MIME_TYPE, TAURUS_MODEL_MIME_TYPE
@@ -90,30 +95,26 @@ class DefaultLabelWidget(TaurusLabel):
         try:
             config = self.taurusValueBuddy().getLabelConfig()
         except Exception:
-            config = 'label'
+            config = '{attr.label}'
         elementtype = self.taurusValueBuddy().getModelType()
         fullname = self.taurusValueBuddy().getModelObj().getFullName()
         if elementtype == TaurusElementType.Attribute:
             config = self.taurusValueBuddy().getLabelConfig()
             TaurusLabel.setModel(self, '%s#%s' % (fullname, config))
         elif elementtype == TaurusElementType.Device:
-            # @TODO: tango-centric!
-            # TaurusLabel.setModel(self, '%s/state#dev_alias'%fullname)
-            #
-            # The following is a workaround to avoid tango-centricity, but
-            # it has the drawback that the model is not set (e.g., no tooltip)
             devName = self.taurusValueBuddy().getModelObj().getSimpleName()
-            TaurusLabel.setModel(self, None)
+            TaurusLabel.setModel(self, model)
             self.setText(devName)
 
     _BCK_COMPAT_TAGS = {'<attr_name>': '{attr.name}',
                         '<attr_fullname>': '{attr.fullname}',
                         '<dev_alias>': '{dev.name}',
-                        '<dev_name>': '{dev_name}',
+                        '<dev_name>': '{dev.name}',
                         '<dev_fullname>': '{dev.fullname}',
                         }
 
     def getDisplayValue(self, cache=True, fragmentName=None):
+        """Reimplementation of getDisplayValue"""
         if self.modelObj is None or fragmentName is None:
             return self.getNoneValue()
         # support bck-compat tags
@@ -121,9 +122,8 @@ class DefaultLabelWidget(TaurusLabel):
             new = self._BCK_COMPAT_TAGS.get(old, '{attr.%s}' % old)
             self.deprecated(dep=old, alt=new)
             fragmentName = fragmentName.replace(old, new)
-        attr = self.getModelObj()
-        dev = attr.getParent()
-        return fragmentName.format(dev=dev, attr=attr)
+
+        return TaurusLabel.displayValue(self, fragmentName)
 
     def sizeHint(self):
         return Qt.QSize(Qt.QLabel.sizeHint(self).width(), 18)
@@ -141,9 +141,11 @@ class DefaultLabelWidget(TaurusLabel):
             r_action.setEnabled(self.taurusValueBuddy().hasPendingOperations())
         if self.taurusValueBuddy().isModifiableByUser():
             menu.addAction("Change label",
-                           self.taurusValueBuddy().onChangeLabelConfig)
+                           self.taurusValueBuddy()._onChangeLabelText)
             menu.addAction("Change Read Widget",
                            self.taurusValueBuddy().onChangeReadWidget)
+            menu.addAction("Set Formatter",
+                           self.taurusValueBuddy().onSetFormatter)
             cw_action = menu.addAction(
                 "Change Write Widget", self.taurusValueBuddy().onChangeWriteWidget)
             # disable the action if the taurusValue is readonly
@@ -156,16 +158,18 @@ class DefaultLabelWidget(TaurusLabel):
         event.accept()
 
     def getModelMimeData(self):
-        '''reimplemented to use the taurusValueBuddy model instead of its own model'''
+        """
+        reimplemented to use the taurusValueBuddy model instead of its own
+        model
+        """
         mimeData = TaurusLabel.getModelMimeData(self)
-        mimeData.setData(TAURUS_MODEL_MIME_TYPE,
-                         self.taurusValueBuddy().getModelName())
+        _modelname = str(self.taurusValueBuddy().getModelName())
+        modelname = _modelname.encode(encoding='utf8')
+        mimeData.setData(TAURUS_MODEL_MIME_TYPE, modelname)
         if self.taurusValueBuddy().getModelType() == TaurusElementType.Device:
-            mimeData.setData(TAURUS_DEV_MIME_TYPE,
-                             self.taurusValueBuddy().getModelName())
+            mimeData.setData(TAURUS_DEV_MIME_TYPE, modelname)
         elif self.taurusValueBuddy().getModelType() == TaurusElementType.Attribute:
-            mimeData.setData(TAURUS_ATTR_MIME_TYPE,
-                             self.taurusValueBuddy().getModelName())
+            mimeData.setData(TAURUS_ATTR_MIME_TYPE, modelname)
         return mimeData
 
     @classmethod
@@ -200,8 +204,19 @@ class CenteredLed(TaurusLed):
     DefaultAlignment = Qt.Qt.AlignHCenter | Qt.Qt.AlignVCenter
 
 
+class UnitLessLineEdit(TaurusValueLineEdit):
+    """A customised TaurusValueLineEdit that always shows the magnitude"""
+    def setModel(self, model):
+        if model is None or model == '':
+            return TaurusValueLineEdit.setModel(self, None)
+        return TaurusValueLineEdit.setModel(self, model + "#wvalue.magnitude")
+
+
+
 class DefaultUnitsWidget(TaurusLabel):
 
+    FORMAT = "{}"
+
     def __init__(self, *args):
         TaurusLabel.__init__(self, *args)
         self.setNoneValue('')
@@ -254,9 +269,9 @@ class TaurusArrayEditorButton(_AbstractTaurusValueButton):
 
 
 class TaurusImageButton(_AbstractTaurusValueButton):
-    '''A button that launches a TaurusPlot'''
+    '''A button that launches an ImageDialog'''
     _widgetClassName = 'TaurusImageDialog'
-    _icon = ':/mimetypes/image-x-generic.svg'
+    _icon = 'mimetypes:image-x-generic.svg'
 
 
 class TaurusValuesTableButton(_AbstractTaurusValueButton):
@@ -275,7 +290,7 @@ class TaurusValuesTableButton_W(TaurusValuesTableButton):
 class TaurusDevButton(_AbstractTaurusValueButton):
     '''A button that launches a TaurusAttrForm'''
     _widgetClassName = 'TaurusDevicePanel'
-    _icon = ':/places/folder-remote.svg'
+    _icon = 'places:folder-remote.svg'
     _text = 'Show Device'
 
 
@@ -309,6 +324,7 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
         self.call__init__wo_kw(Qt.QWidget, parent)
         self.call__init__(TaurusBaseWidget, name, designMode=designMode)
 
+        self.__error = False
         self.__modelClass = None
         self._designMode = designMode
 
@@ -317,7 +333,7 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
             layout = Qt.QHBoxLayout(self)
             dummy = ExpandingLabel()
             layout.addWidget(dummy)
-            dummy.setUseParentModel(True)
+            #dummy.setUseParentModel(True)
             dummy.setModel("#attr_fullname")
             dummy.setPrefixText("< TaurusValue: ")
             dummy.setSuffixText(" >")
@@ -373,7 +389,7 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
         itself.
         '''
         if followCompact and self.isCompact():
-            return self._readWidget.readWidget
+            return getattr(self._readWidget, 'readWidget', self._readWidget)
         return self._readWidget
 
     def writeWidget(self, followCompact=False):
@@ -431,6 +447,26 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
         # do the base class stuff too
         Qt.QWidget.setParent(self, parent)
 
+    def onSetFormatter(self):
+        """
+        Reimplemented to call onSetFormatter of the read widget (if provided)
+        """
+        rw = self.readWidget(followCompact=True)
+        if hasattr(rw, 'onSetFormatter'):
+            return rw.onSetFormatter()
+
+    def setFormat(self, format):
+        """
+        Reimplemented to call setFormat of the read widget (if provided)
+        """
+        TaurusBaseWidget.setFormat(self, format)
+        try:
+            rw = self.readWidget(followCompact=True)
+        except AttributeError:
+            return
+        if hasattr(rw, 'setFormat'):
+            rw.setFormat(format)
+
     def getAllowWrite(self):
         return self._allowWrite
 
@@ -560,22 +596,24 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
         modelobj = self.getModelObj()
         if modelobj is None:
             if returnAll:
-                return [TaurusValueLineEdit]
+                return [UnitLessLineEdit]
             else:
-                return TaurusValueLineEdit
+                return UnitLessLineEdit
         modelType = modelobj.getType()
         if modelobj.data_format == DataFormat._0D:
             if modelType == DataType.Boolean:
                 result = [DefaultTaurusValueCheckBox, TaurusValueLineEdit]
             else:
-                result = [TaurusValueLineEdit,
+                result = [UnitLessLineEdit,
                           TaurusValueSpinBox, TaurusWheelEdit]
         elif modelobj.data_format == DataFormat._1D:
+            result = [TaurusValuesTableButton_W, TaurusValueLineEdit]
             if modelType in (DataType.Float, DataType.Integer):
-                result = [TaurusArrayEditorButton,
-                          TaurusValuesTableButton_W, TaurusValueLineEdit]
-            else:
-                result = [TaurusValuesTableButton_W, TaurusValueLineEdit]
+                try:
+                    import PyQt4.Qwt5  # noqa
+                    result.insert(0, TaurusArrayEditorButton)
+                except:
+                    pass
         elif modelobj.data_format == DataFormat._2D:
             result = [TaurusValuesTableButton_W]
         else:
@@ -625,8 +663,13 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
         return self._customWidgetMap
 
     def onChangeLabelConfig(self):
+        self.deprecated(msg="onChangeLabelConfig is deprecated", rel="Jan2018")
+        self._onChangeLabelText()
+
+    def _onChangeLabelText(self):
         keys = ['{attr.label}', '{attr.name}', '{attr.fullname}', '{dev.name}',
                 '{dev.fullname}']
+
         try:
             current = keys.index(self.labelConfig)
         except:
@@ -641,7 +684,7 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
         labelConfig, ok = Qt.QInputDialog.getItem(self, 'Change Label', msg,
                                                   keys, current, True)
         if ok:
-            self.labelConfig = str(labelConfig)
+            self.labelConfig = labelConfig
 
     def onChangeReadWidget(self):
         classnames = ['None', 'Auto'] + \
@@ -804,7 +847,7 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
         try:
             klass = self.readWidgetClassFactory(self.readWidgetClassID)
             self._readWidget = self._newSubwidget(self._readWidget, klass)
-        except Exception, e:
+        except Exception as e:
             self._destroyWidget(self._readWidget)
             self._readWidget = Qt.QLabel('[Error]')
             msg = 'Error creating read widget:\n' + str(e)
@@ -1057,13 +1100,29 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
         self.extraWidgetClassID = 'Auto'
 
     def setCompact(self, compact):
+
+        # don't do anything if it is already done
         if compact == self._compact:
             return
+
+        #do not switch to compact mode if the write widget is None
+        if compact and self.writeWidget() is None:
+            self.debug('No write widget. Ignoring setCompact(True)')
+            return
+
+        # Backup the current RW format
+        rw = self.readWidget(followCompact=True)
+        format = rw.getFormat()
+
         self._compact = compact
         if self.getModel():
             self.updateReadWidget()
             self.updateWriteWidget()
 
+        # Apply the format to the new RW
+        rw = self.readWidget(followCompact=True)
+        rw.setFormat(format)
+
     def isCompact(self):
         return self._compact
 
@@ -1102,7 +1161,8 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
         for key in ('LabelWidget', 'ReadWidget', 'WriteWidget', 'UnitsWidget', 'CustomWidget', 'ExtraWidget'):
             # calls self.getLabelWidgetClass, self.getReadWidgetClass,...
             classID = getattr(self, 'get%sClass' % key)()
-            if isinstance(classID, (str, Qt.QString)) or allowUnpickable:
+            if (isinstance(classID, string_types)
+                    or allowUnpickable):
                 #configdict[key] = classID
                 configdict[key] = {'classid': classID}
                 widget = getattr(self, key[0].lower() + key[1:])()
@@ -1129,7 +1189,7 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
                 widget_configdict = configdict[key]
                 getattr(self, 'set%sClass' % key)(
                     widget_configdict.get('classid', None))
-                if widget_configdict.has_key('delegate'):
+                if 'delegate' in widget_configdict:
                     widget = getattr(self, key[0].lower() + key[1:])()
                     if isinstance(widget, BaseConfigurableClass):
                         widget.applyConfig(widget_configdict[
@@ -1153,7 +1213,10 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
         """Reimplemented from :meth:`TaurusBaseWidget.handleEvent`
         to update subwidgets on config events
         """
-        if evt_type == taurus.core.taurusbasetypes.TaurusEventType.Config and not self._designMode:
+        if self._designMode:
+            return
+
+        if self.__error or evt_type == TaurusEventType.Config:
             self.updateCustomWidget()
             self.updateLabelWidget()
             self.updateReadWidget()
@@ -1161,6 +1224,9 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
             self.updateUnitsWidget()
             self.updateExtraWidget()
 
+        # set/unset the error flag
+        self.__error = (evt_type == TaurusEventType.Error)
+
     def isValueChangedByUser(self):
         try:
             return self._writeWidget.isValueChangedByUser()
@@ -1214,7 +1280,29 @@ class TaurusValue(Qt.QWidget, TaurusBaseWidget):
 
     @Qt.pyqtSlot('QString')
     def setLabelConfig(self, config):
+        """Sets fragment configuration to the label widget.
+
+        :param config: fragment
+        :type config: str
+        """
         self._labelConfig = config
+        # backwards compatibility: this method used to work for setting
+        # an arbitrary text to the label widget
+        try:
+            self.getModelFragmentObj(config)
+            self._labelWidget._permanentText = None
+        except Exception:
+            try:
+                for old in re.findall('<.+?>', config):
+                    new = self._BCK_COMPAT_TAGS.get(old, old)
+                    self.deprecated(dep=old, alt=new)
+                    config = config.replace(old, new)
+
+                self._labelWidget.setText(config)
+            except:
+                self.debug("Setting permanent text to the label widget failed")
+            return
+
         self.updateLabelWidget()
 
     def resetLabelConfig(self):
@@ -1310,7 +1398,9 @@ if __name__ == "__main__":
 
     import sys
 
-    app = Qt.QApplication(sys.argv)
+    from taurus.qt.qtgui.application import TaurusApplication
+
+    app = TaurusApplication(sys.argv, cmd_line_parser=None)
     form = Qt.QMainWindow()
     # ly=Qt.QVBoxLayout(form)
     # container=Qt.QWidget()
