@@ -28,16 +28,19 @@ model in a table widget"""
 
 # todo: tango-centric!!!
 
-__all__ = ["TaurusDbTableWidget"]
-
-__docformat__ = 'restructuredtext'
+from __future__ import absolute_import
 
 from taurus.external.qt import Qt
 from taurus.core.taurusbasetypes import TaurusElementType
 from taurus.qt.qtcore.model import *
 from taurus.core.taurusauthority import TaurusAuthority
 from taurus.qt.qtgui.icon import getElementTypeIcon, getElementTypeIconName
-from taurustable import TaurusBaseTableWidget
+from .taurustable import TaurusBaseTableWidget
+
+
+__all__ = ["TaurusDbTableWidget"]
+
+__docformat__ = 'restructuredtext'
 
 
 class TaurusDbTableWidget(TaurusBaseTableWidget):
