@@ -35,7 +35,7 @@ The "new GUI wizard" and XML configuration files
 ------------------------------------------------
 
 Note that the configuration files can either be written by hand or by
-launching the "new GUI" wizard with `taurusgui --new-gui`, which will create
+launching the "new GUI" wizard with `taurus newgui`, which will create
 a new directory containing configuration, resource and launcher files.
 
 The new GUI wizard stores all the options in xml format in a file called
@@ -49,14 +49,18 @@ configuration options (in case of conflict, the options set in `config.py`
 prevail).
 
 """
+from __future__ import absolute_import
+
+from .paneldescriptionwizard import *
+from .taurusgui import *
+from .appsettingswizard import *
+try:
+    # this import is left here for bck-compat, but will be removed
+    # TODO: remove this
+    from sardana.taurus.qt.qtgui.macrolistener import (MacroBroker,
+                                                       DynamicPlotManager)
+except ImportError:
+    pass
+
 
 __docformat__ = 'restructuredtext'
-
-import utils
-from paneldescriptionwizard import *
-from taurusgui import *
-from appsettingswizard import *
-try:
-    from macrolistener import *
-except ImportError:
-    pass  # allow for sardana not being installed
