@@ -32,8 +32,9 @@ contents of the GUI. While the user may add/remove more elements at run
 time and those customizations will also be stored, this file defines what a
 user will find when launching the GUI for the first time.
 """
+from __future__ import print_function
 
-__all__ = ["AppSettingsWizard", "ExternalAppEditor"]
+from builtins import str
 
 import os
 import re
@@ -41,10 +42,11 @@ import sys
 import shutil
 import copy
 import datetime
+import glob
 from lxml import etree
 
-from taurus import tauruscustomsettings
-from taurus.external.qt import Qt
+from taurus import tauruscustomsettings, warning
+from taurus.external.qt import Qt, compat
 import taurus.qt.qtgui.panel
 import taurus.qt.qtgui.taurusgui.paneldescriptionwizard
 import taurus.qt.qtgui.input
@@ -52,6 +54,9 @@ from taurus.core.util.enumeration import Enumeration
 from taurus.qt.qtgui.util import ExternalAppAction
 
 
+__all__ = ["AppSettingsWizard", "ExternalAppEditor"]
+
+
 class BooleanWidget(Qt.QWidget):
     """
     This class represents the simple boolean widget with two RadioButtons
@@ -228,26 +233,30 @@ class ProjectPage(BasePage):
         self._projectDirBT.clicked.connect(self.onSelectDir)
 
     def onSelectDir(self):
-        dirname = unicode(Qt.QFileDialog.getExistingDirectory(
+        dirname = str(Qt.QFileDialog.getExistingDirectory(
             self, 'Choose the project directory', self._projectDirLE.text()))
         if not dirname:
             return
         self._projectDirLE.setText(dirname)
 
     def validatePage(self):
-        dirname = unicode(self._projectDirLE.text())
+        dirname = str(self._projectDirLE.text())
 
         if not os.path.exists(dirname):
             try:
                 os.makedirs(dirname)
-            except Exception, e:
+            except Exception as e:
                 Qt.QMessageBox.warning(self, 'Error creating project directory',
                                        'Could not create the project directory.\nReason:%s' % repr(
                                            e),
                                        Qt.QMessageBox.Cancel)
                 return False
-        fname = os.path.join(dirname, self.wizard().getXmlConfigFileName())
-        if os.path.exists(fname):
+
+        configs_found = glob.glob(os.path.join(dirname, "tgconf_*",
+                                  self.wizard().getXmlConfigFileName()))
+        # fname = os.path.join(dirname, self.wizard().getXmlConfigFileName())
+        if len(configs_found) == 1:
+            fname = configs_found[0]
             option = Qt.QMessageBox.question(self, 'Overwrite project?',
                                              'The "%s" file already exists in the project directory.\n Do you want to edit the existing project?' % (
                                                  os.path.basename(fname)),
@@ -255,7 +264,7 @@ class ProjectPage(BasePage):
             if option == Qt.QMessageBox.Yes:
                 try:
                     self.wizard().loadXml(fname)
-                except Exception, e:
+                except Exception as e:
                     Qt.QMessageBox.warning(self, 'Error loading project configuration',
                                            'Could not load the existing configuration.\nReason:%s' % repr(
                                                e),
@@ -274,7 +283,7 @@ class ProjectPage(BasePage):
         return True
 
     def _getProjectDir(self):
-        return unicode(self._projectDirLE.text())
+        return str(self._projectDirLE.text())
 
 
 class GeneralSettings(BasePage):
@@ -461,8 +470,10 @@ class CustomLogoPage(BasePage):
             return None
 
     def _selectImage(self):
-        fileName = Qt.QFileDialog.getOpenFileName(self, self.tr(
-            "Open File"), Qt.QDir.homePath(), self.tr("Images (*.png *.xpm *.jpg *.jpeg *.svg)"))
+        fileName, _ = compat.getOpenFileName(
+            self, self.tr("Open File"), Qt.QDir.homePath(),
+            self.tr("Images (*.png *.xpm *.jpg *.jpeg *.svg)")
+        )
         self._customLogoLineEdit.setText(fileName)
         self._changeImage()
 
@@ -570,10 +581,12 @@ class SynopticPage(BasePage):
 
     def _addSynoptic(self):
         pdir = self.wizard().__getitem__('projectDir')
-        fileNames = Qt.QFileDialog.getOpenFileNames(self, self.tr(
-            "Open File"), pdir, self.tr("JDW (*.jdw );; All files (*)"))
+        fileNames, _ = compat.getOpenFileNames(
+            self, self.tr("Open File"), pdir,
+            self.tr("JDW (*.jdw );; All files (*)")
+        )
         for fileName in fileNames:
-            fileName = unicode(fileName)
+            fileName = str(fileName)
             if fileName not in self._synoptics:
                 self._synoptics.append(fileName)
         self._refreshSynopticList()
@@ -712,13 +725,30 @@ class MacroServerInfoPage(BasePage):
 
     def _getMacroServerName(self):
         if (self._macroGroupBox.isChecked()) and len(self._confWidget.macroServerComboBox.currentText()):
-            return str(self._confWidget.macroServerComboBox.currentText())
+            try: # TODO: tango-centric
+                from taurus.core.tango.tangovalidator import (
+                    TangoDeviceNameValidator)
+            except ImportError:
+                warning('Cannot import tango (needed for sardana integration)')
+                return None
+            ms_name = str(self._confWidget.macroServerComboBox.currentText())
+            complete_name, _, _ = TangoDeviceNameValidator().getNames(ms_name)
+            return complete_name
         else:
             return None
 
     def _getDoorName(self):
         if (self._macroGroupBox.isChecked()) and len(self._confWidget.macroServerComboBox.currentText()):
-            return str(self._confWidget.doorComboBox.currentText())
+            door_name = str(self._confWidget.doorComboBox.currentText())
+            try: # TODO: tango-centric
+                from taurus.core.tango.tangovalidator import (
+                    TangoDeviceNameValidator)
+            except ImportError:
+                warning('Cannot import tango (needed for sardana integration)')
+                return None
+            complete_name, _, _ = TangoDeviceNameValidator().getNames(
+                door_name)
+            return complete_name
         else:
             return None
 
@@ -842,7 +872,8 @@ class PanelsPage(BasePage):
             name = AppSettingsWizard.getValueFromNode(
                 child, "name", default=None)
             if name:
-                self._panels.append((name, etree.tostring(child)))
+                self._panels.append(
+                    (name, etree.tostring(child, encoding='unicode')))
 
     def _addPanel(self):
         paneldesc, ok = taurus.qt.qtgui.taurusgui.paneldescriptionwizard.PanelDescriptionWizard.getDialog(
@@ -989,8 +1020,10 @@ class ExternalAppEditor(Qt.QDialog):
         self.checkData()
 
     def _selectExecFile(self):
-        filePath = Qt.QFileDialog.getOpenFileName(self, self.tr(
-            "Open File"), Qt.QDir.homePath(), self.tr("All files (*)"))
+        filePath, _ = compat.getOpenFileName(
+            self, self.tr("Open File"),
+            Qt.QDir.homePath(), self.tr("All files (*)")
+        )
         if len(filePath):
             self._execFileLineEdit.setText(filePath)
             self._setDefaultText()
@@ -1036,7 +1069,7 @@ class ExternalAppEditor(Qt.QDialog):
         icon = etree.SubElement(root, "icon")
         icon.text = self._getIcon()
 
-        return etree.tostring(root)
+        return etree.tostring(root, encoding='unicode')
 
     @staticmethod
     def getDialog():
@@ -1110,7 +1143,9 @@ class ExternalAppPage(BasePage):
             name = AppSettingsWizard.getValueFromNode(
                 child, "command", default=None)
             if name:
-                self._externalApps.append((name, etree.tostring(child)))
+                self._externalApps.append(
+                    (name, etree.tostring(child, encoding='unicode'))
+                )
 
     def _addApplication(self):
         name, xml, ok = ExternalAppEditor.getDialog()
@@ -1299,7 +1334,7 @@ class OutroPage(BasePage):
     def validatePage(self):
         try:
             self.createProject()
-        except Exception, e:
+        except Exception as e:
             Qt.QMessageBox.warning(self, 'Error creating project',
                                    'Could not create project files. \nReason:%s' % repr(
                                        e),
@@ -1312,67 +1347,90 @@ class OutroPage(BasePage):
     def createProject(self):
         # prepare a log file
         pdir = self.wizard().__getitem__('projectDir')
+        gui_name = self.wizard().__getitem__("guiName")
+        install_dir = os.path.join(pdir, "tgconf_{0}".format(gui_name))
+        if not os.path.exists(install_dir):
+            os.makedirs(install_dir)
         logfilename = os.path.join(pdir, 'wizard.log')
         logfile = open(logfilename, 'w')
         logfile.write('Project created by AppSettingsWizard on %s\n' %
                       datetime.datetime.now().isoformat())
         # copy files
         for i in range(self._substTable.rowCount()):
-            src = unicode(self._substTable.item(i, 0).text())
-            dst = os.path.join(pdir, unicode(
+            src = str(self._substTable.item(i, 0).text())
+            dst = os.path.join(install_dir, str(
                 self._substTable.item(i, 1).text()))
             if os.path.normpath(src) != os.path.normpath(dst):
                 shutil.copy(src, dst)
                 logfile.write('File copied: %s --> %s\n' % (src, dst))
         # write xml config file
-        xmlcfgfilename = os.path.join(
-            pdir, self.wizard().getXmlConfigFileName())
+        xmlcfgfilename = os.path.join(install_dir,
+                                      self.wizard().getXmlConfigFileName())
         f = open(xmlcfgfilename, 'w')
-        f.write(unicode(self._xml.toPlainText()))
+        f.write(str(self._xml.toPlainText()))
         f.close()
         logfile.write('XML Config file created: "%s"\n' % xmlcfgfilename)
         # write python config file
-        pycfgfilename = os.path.join(
-            pdir, '%s.py' % self.wizard().getConfigFilePrefix())
+        pycfgfilename = os.path.join(install_dir,
+                                '%s.py' % self.wizard().getConfigFilePrefix())
         f = open(pycfgfilename, 'w')
         f.write("XML_CONFIG = '%s'" % self.wizard().getXmlConfigFileName())
         f.close()
         logfile.write('Python config file created: "%s"\n' % pycfgfilename)
         # write __init__.py config file
-        initfilename = os.path.join(pdir, '__init__.py')
+        init_template = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)), 'res', 'init.template')
+        f = open(init_template, 'r')
+        template = f.read()
+        f.close()
+        initfilename = os.path.join(install_dir, '__init__.py')
         f = open(initfilename, 'w')
-        f.write('from config import *')
+        template = template.format(name=gui_name)
+        f.write(template)
         f.close()
         logfile.write('python init file created: "%s"\n' % initfilename)
-        # write launcher script
-        try:
-            launcherfilename = os.path.join(
-                pdir, self.wizard().__getitem__("guiName"))
-            f = open(launcherfilename, 'w')
-            f.write(('#!/bin/sh\n'
-                     '#Make sure to give this file execution permisions\n'
-                     'taurusgui %s $*') % os.path.basename(pdir.rstrip('/')))
-            f.close()
-            os.chmod(launcherfilename, 0755)
-            logfile.write('Unix launcher created: "%s"\n' % launcherfilename)
-        except:
-            logfile.write('Error creating Unix launcher: "%s"\n' %
-                          launcherfilename)
+        # write setup file
+        setup_template = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)), 'res', 'setup.template')
+        f = open(setup_template, 'r')
+        template = f.read()
+        f.close()
+        setup = os.path.join(pdir, "setup.py")
+        f = open(setup, 'w')
+        template = template.format(name=gui_name)
+        f.write(template)
+        f.close()
+        # write MANIFEST.in file
+        manifest_template = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)), 'res', 'manifest.template')
+        f = open(manifest_template, 'r')
+        template = f.read()
+        f.close()
+        manifestfile = os.path.join(pdir, "MANIFEST.in")
+        f = open(manifestfile, 'w')
+        template = template.format(name=gui_name)
+        f.write(template)
+        f.close()
         # if all went ok...
-        msg = 'Application project was successfully created. You can find the files in: "%s"' % pdir
-        msg += '\nTip: copy this directory into a directory that is in your Python path.'
+        msg = 'Application project was successfully created.' +\
+              'You can find the files in: "%s"' % pdir
+        msg += '\nTip: You can install it with:\n\tpip install %s' % pdir
+        msg += '\nTip: And then run the application with:\n\t %s' % gui_name
         details = ''
         warnings = self.wizard().getProjectWarnings()
         if warnings:
             msg += '\n\nHowever, some fine-tuning may be needed. Please check the details:\n'
-            for short, long in warnings:
-                details += '- %s: %s\n\n' % (short, long)
+            for _short, _long in warnings:
+                details += '- %s: %s\n\n' % (_short, _long)
         logfile.write(msg + details)
         logfile.close()
         dlg = Qt.QMessageBox(Qt.QMessageBox.Information,
                              'Application project created', msg, Qt.QMessageBox.Ok, self)
         dlg.setDetailedText(details)
         dlg.exec_()
+        print() 
+        print(msg + details)
+        print()
 
 
 class AppSettingsWizard(Qt.QWizard):
@@ -1453,13 +1511,13 @@ class AppSettingsWizard(Qt.QWizard):
         root = etree.fromstring(xml)
 
         # print self.Pages
-        for pageNumber in range(len(self.Pages.keys())):
+        for pageNumber in range(len(self.Pages)):
             self.page(pageNumber).fromXml(root)
 
     def getXml(self):
         try:
             return self.__getitem__("xml")
-        except Exception, e:
+        except Exception as e:
             return None
 
     def __setitem__(self, name, value):
@@ -1471,7 +1529,7 @@ class AppSettingsWizard(Qt.QWizard):
             if isinstance(p, BasePage):
                 try:
                     return p[name]()
-                except Exception, e:
+                except Exception as e:
                     pass
         return self._item_funcs[name]()
 
@@ -1509,6 +1567,9 @@ class AppSettingsWizard(Qt.QWizard):
         try:
             from sardana.taurus.qt.qtgui.extra_macroexecutor.common import \
                 TaurusMacroConfigurationDialog
+            # try to instantiate the dialog (e.g. this fails if using Qt5 with
+            # versions of sardana which do not support it)
+            _ = TaurusMacroConfigurationDialog()
             self.SARDANA_INSTALLED = True
         except:
             self.SARDANA_INSTALLED = False
@@ -1552,7 +1613,6 @@ class AppSettingsWizard(Qt.QWizard):
                  is the xml code and the second element is a dict where the keys are the
                  destination files and the values are the original paths.
         '''
-        pdir = self.__getitem__('projectDir')
         root = etree.Element("taurusgui_config")
         # general settings page
         guiName = etree.SubElement(root, "GUI_NAME")
@@ -1562,23 +1622,28 @@ class AppSettingsWizard(Qt.QWizard):
         # custom logo page
         customLogo = etree.SubElement(root, "CUSTOM_LOGO")
         src = self.__getitem__("customLogo")
-        if src is None or src.startswith(":"):
+        mod_dir = os.path.join(self.__getitem__('projectDir'),
+                            'tgconf_' + guiName.text)
+        mod_dir = os.path.abspath(mod_dir)  # make sure mod_dir is absolute
+        if src is None or ":" in src:
+            # using registered paths
+            # TODO: what if they use windows paths such as "C:\foo" ?
             dst = src
         else:
             # if src is absolute, it stays so, and if it is relative, we assume
-            # pdir as the root dir
-            src = os.path.join(pdir, src)
-            dst = self.substitutionName(src)
+            # mod_dir as the root dir
+            src = os.path.join(mod_dir, src)
+            dst = os.path.basename(self.substitutionName(src, mod_dir))
         customLogo.text = dst
         # synoptic page
         synopticList = self.__getitem__("synoptics")
         if synopticList:
             synoptics = etree.SubElement(root, "SYNOPTIC")
             for src in synopticList:
-                src = os.path.join(pdir, src)
-                # substitute the jdw files
-                dst = self.substitutionName(src)
-                child = etree.SubElement(synoptics, "synoptic", str=dst)
+                src = os.path.join(mod_dir, src)
+                dst = self.substitutionName(src, mod_dir)
+                child = etree.SubElement(synoptics, "synoptic",
+                                         str=os.path.basename(dst))
                 # substitute any referenced files within the jdrawfiles
                 f = open(src, 'r')
                 contents = f.read()
@@ -1586,13 +1651,13 @@ class AppSettingsWizard(Qt.QWizard):
                 for ref in re.findall(r'file_name:\"(.+?)\"', contents):
                     # this is ok for both relative and absolute references
                     refsrc = os.path.join(os.path.dirname(src), ref)
-                    refdst = self.substitutionName(refsrc)
+                    refdst = self.substitutionName(refsrc, mod_dir)
                     if ref != refdst:
-                        short = 'Manual editing needed in "%s"' % dst
-                        long = ('The synoptic file "%s" references a file that '
+                        _short = 'Manual editing needed in "%s"' % dst
+                        _long = ('The synoptic file "%s" references a file that '
                                 'has been copied to the project dir in order to make the project portable. '
                                 'Please edit "%s" and replace "%s" by "%s"') % (dst, dst, ref, refdst)
-                        self._projectWarnings.append((short, long))
+                        self._projectWarnings.append((_short, _long))
 
         # macroserver page
         if self.SARDANA_INSTALLED and self.__getitem__("macroServerName"):
@@ -1622,15 +1687,18 @@ class AppSettingsWizard(Qt.QWizard):
         monitor = etree.SubElement(root, "MONITOR")
         monitor.text = self.__getitem__("monitor")
 
-        return etree.tostring(root, pretty_print=True), copy.copy(self._substitutions)
+        return (etree.tostring(root, pretty_print=True, encoding='unicode'),
+                copy.copy(self._substitutions))
 
-    def substitutionName(self, src):
+    def substitutionName(self, src, mod_dir):
         name = os.path.basename(src)
         i = 2
-        while name in self._substitutions:
-            root, ext = os.path.splitext(name)
-            name = "%s_%i%s" % (root, i, ext)
-            i += 1
+        if os.path.dirname(os.path.abspath(src)) != os.path.abspath(mod_dir):
+            # do not change the name if it is the same dir!
+            while name in self._substitutions:
+                root, ext = os.path.splitext(name)
+                name = "%s_%i%s" % (root, i, ext)
+                i += 1
         self._substitutions[name] = src
         return name
 
