@@ -23,4 +23,5 @@
 ##
 ###########################################################################
 
-from config import *
+from __future__ import absolute_import
+from .config import *
